@@ -39,6 +39,7 @@
 /* Define slurm-specific aliases for use by plugins, see slurm_xlator.h. */
 strong_alias(cgroup_conf_init, slurm_cgroup_conf_init);
 strong_alias(cgroup_conf_destroy, slurm_cgroup_conf_destroy);
+strong_alias(autodetect_cgroup_version, slurm_autodetect_cgroup_version);
 
 #define DEFAULT_CGROUP_BASEDIR "/sys/fs/cgroup"
 
@@ -58,30 +59,22 @@ typedef struct {
 	int	(*step_resume)		(void);
 	int	(*step_destroy)		(cgroup_ctl_type_t sub);
 	bool	(*has_pid)		(pid_t pid);
-	cgroup_limits_t *(*root_constrain_get) (cgroup_ctl_type_t sub);
-	int	(*root_constrain_set)	(cgroup_ctl_type_t sub,
+	cgroup_limits_t *(*constrain_get) (cgroup_ctl_type_t sub,
+					   cgroup_level_t level);
+	int	(*constrain_set)	(cgroup_ctl_type_t sub,
+					 cgroup_level_t level,
 					 cgroup_limits_t *limits);
-	cgroup_limits_t *(*system_constrain_get) (cgroup_ctl_type_t sub);
-	int	(*system_constrain_set)	(cgroup_ctl_type_t sub,
-					 cgroup_limits_t *limits);
-	int	(*user_constrain_set)	(cgroup_ctl_type_t sub,
-					 stepd_step_rec_t *job,
-					 cgroup_limits_t *limits);
-	int	(*job_constrain_set)	(cgroup_ctl_type_t sub,
-					 stepd_step_rec_t *job,
-					 cgroup_limits_t *limits);
-	int	(*step_constrain_set)	(cgroup_ctl_type_t sub,
-					 stepd_step_rec_t *job,
-					 cgroup_limits_t *limits);
-	int     (*task_constrain_set)   (cgroup_ctl_type_t sub,
-					 cgroup_limits_t *limits,
-					 uint32_t taskid);
+        int	(*constrain_apply)	(cgroup_ctl_type_t sub,
+                                         cgroup_level_t level,
+                                         uint32_t task_id);
 	int	(*step_start_oom_mgr)	(void);
 	cgroup_oom_t *(*step_stop_oom_mgr) (stepd_step_rec_t *job);
 	int	(*task_addto)		(cgroup_ctl_type_t sub,
 					 stepd_step_rec_t *job, pid_t pid,
 					 uint32_t task_id);
 	cgroup_acct_t *(*task_get_acct_data) (uint32_t taskid);
+	long int (*get_acct_units)	(void);
+	bool (*has_feature) (cgroup_ctl_feature_t f);
 } slurm_ops_t;
 
 /*
@@ -100,18 +93,15 @@ static const char *syms[] = {
 	"cgroup_p_step_resume",
 	"cgroup_p_step_destroy",
 	"cgroup_p_has_pid",
-	"cgroup_p_root_constrain_get",
-	"cgroup_p_root_constrain_set",
-	"cgroup_p_system_constrain_get",
-	"cgroup_p_system_constrain_set",
-	"cgroup_p_user_constrain_set",
-	"cgroup_p_job_constrain_set",
-	"cgroup_p_step_constrain_set",
-	"cgroup_p_task_constrain_set",
+	"cgroup_p_constrain_get",
+	"cgroup_p_constrain_set",
+        "cgroup_p_constrain_apply",
 	"cgroup_p_step_start_oom_mgr",
 	"cgroup_p_step_stop_oom_mgr",
 	"cgroup_p_task_addto",
 	"cgroup_p_task_get_acct_data",
+	"cgroup_p_get_acct_units",
+	"cgroup_p_has_feature",
 };
 
 /* Local variables */
@@ -133,7 +123,6 @@ static void _clear_slurm_cgroup_conf();
 static void _pack_cgroup_conf(buf_t *buffer);
 static int _unpack_cgroup_conf(buf_t *buffer);
 static void _read_slurm_cgroup_conf(void);
-static char *_autodetect_cgroup_version(void);
 
 /* Local functions */
 static void _cgroup_conf_fini()
@@ -153,7 +142,6 @@ static void _clear_slurm_cgroup_conf()
 	xfree(slurm_cgroup_conf.cgroup_mountpoint);
 	xfree(slurm_cgroup_conf.cgroup_prepend);
 	slurm_cgroup_conf.constrain_cores = false;
-	slurm_cgroup_conf.task_affinity = false;
 	slurm_cgroup_conf.constrain_ram_space = false;
 	slurm_cgroup_conf.allowed_ram_space = 100;
 	slurm_cgroup_conf.max_ram_percent = 100;
@@ -167,8 +155,9 @@ static void _clear_slurm_cgroup_conf()
 	slurm_cgroup_conf.max_swap_percent = 100;
 	slurm_cgroup_conf.constrain_devices = false;
 	slurm_cgroup_conf.memory_swappiness = NO_VAL64;
-	xfree(slurm_cgroup_conf.allowed_devices_file);
 	xfree(slurm_cgroup_conf.cgroup_plugin);
+	slurm_cgroup_conf.ignore_systemd = false;
+	slurm_cgroup_conf.ignore_systemd_on_failure = false;
 }
 
 static void _pack_cgroup_conf(buf_t *buffer)
@@ -189,7 +178,6 @@ static void _pack_cgroup_conf(buf_t *buffer)
 	packstr(slurm_cgroup_conf.cgroup_prepend, buffer);
 
 	packbool(slurm_cgroup_conf.constrain_cores, buffer);
-	packbool(slurm_cgroup_conf.task_affinity, buffer);
 
 	packbool(slurm_cgroup_conf.constrain_ram_space, buffer);
 	packfloat(slurm_cgroup_conf.allowed_ram_space, buffer);
@@ -208,8 +196,10 @@ static void _pack_cgroup_conf(buf_t *buffer)
 	pack64(slurm_cgroup_conf.memory_swappiness, buffer);
 
 	packbool(slurm_cgroup_conf.constrain_devices, buffer);
-	packstr(slurm_cgroup_conf.allowed_devices_file, buffer);
 	packstr(slurm_cgroup_conf.cgroup_plugin, buffer);
+
+	packbool(slurm_cgroup_conf.ignore_systemd, buffer);
+	packbool(slurm_cgroup_conf.ignore_systemd_on_failure, buffer);
 }
 
 static int _unpack_cgroup_conf(buf_t *buffer)
@@ -234,7 +224,6 @@ static int _unpack_cgroup_conf(buf_t *buffer)
 			       &uint32_tmp, buffer);
 
 	safe_unpackbool(&slurm_cgroup_conf.constrain_cores, buffer);
-	safe_unpackbool(&slurm_cgroup_conf.task_affinity, buffer);
 
 	safe_unpackbool(&slurm_cgroup_conf.constrain_ram_space, buffer);
 	safe_unpackfloat(&slurm_cgroup_conf.allowed_ram_space, buffer);
@@ -253,10 +242,12 @@ static int _unpack_cgroup_conf(buf_t *buffer)
 	safe_unpack64(&slurm_cgroup_conf.memory_swappiness, buffer);
 
 	safe_unpackbool(&slurm_cgroup_conf.constrain_devices, buffer);
-	safe_unpackstr_xmalloc(&slurm_cgroup_conf.allowed_devices_file,
-			       &uint32_tmp, buffer);
 	safe_unpackstr_xmalloc(&slurm_cgroup_conf.cgroup_plugin,
 			       &uint32_tmp, buffer);
+
+	safe_unpackbool(&slurm_cgroup_conf.ignore_systemd, buffer);
+	safe_unpackbool(&slurm_cgroup_conf.ignore_systemd_on_failure, buffer);
+
 	return SLURM_SUCCESS;
 
 unpack_error:
@@ -276,7 +267,6 @@ static void _read_slurm_cgroup_conf(void)
 		{"CgroupMountpoint", S_P_STRING},
 		{"CgroupReleaseAgentDir", S_P_STRING},
 		{"ConstrainCores", S_P_BOOLEAN},
-		{"TaskAffinity", S_P_BOOLEAN},
 		{"ConstrainRAMSpace", S_P_BOOLEAN},
 		{"AllowedRAMSpace", S_P_FLOAT},
 		{"MaxRAMPercent", S_P_FLOAT},
@@ -294,10 +284,13 @@ static void _read_slurm_cgroup_conf(void)
 		{"AllowedDevicesFile", S_P_STRING},
 		{"MemorySwappiness", S_P_UINT64},
 		{"CgroupPlugin", S_P_STRING},
+		{"IgnoreSystemd", S_P_BOOLEAN},
+		{"IgnoreSystemdOnFailure", S_P_BOOLEAN},
 		{NULL} };
 	s_p_hashtbl_t *tbl = NULL;
 	char *conf_path = NULL, *tmp_str;
 	struct stat buf;
+	size_t sz;
 
 	/* Get the cgroup.conf path and validate the file */
 	conf_path = get_extra_conf_path("cgroup.conf");
@@ -309,7 +302,7 @@ static void _read_slurm_cgroup_conf(void)
 		debug("Reading cgroup.conf file %s", conf_path);
 
 		tbl = s_p_hashtbl_create(options);
-		if (s_p_parse_file(tbl, NULL, conf_path, false) ==
+		if (s_p_parse_file(tbl, NULL, conf_path, false, NULL) ==
 		    SLURM_ERROR) {
 			fatal("Could not open/read/parse cgroup.conf file %s",
 			      conf_path);
@@ -321,13 +314,21 @@ static void _read_slurm_cgroup_conf(void)
 			slurm_cgroup_conf.cgroup_automount = false;
 
 		if (!s_p_get_string(&slurm_cgroup_conf.cgroup_mountpoint,
-				    "CgroupMountpoint", tbl))
+				    "CgroupMountpoint", tbl)) {
 			slurm_cgroup_conf.cgroup_mountpoint =
 				xstrdup(DEFAULT_CGROUP_BASEDIR);
-
+		} else {
+			/* Remove the trailing / if any. */
+			tmp_str = slurm_cgroup_conf.cgroup_mountpoint;
+			sz = strlen(tmp_str);
+			if (*(tmp_str + sz - 1) == '/')
+				*(tmp_str + sz - 1) = '\0';
+			slurm_cgroup_conf.cgroup_mountpoint = xstrdup(tmp_str);
+			xfree(tmp_str);
+		}
 		if (s_p_get_string(&tmp_str, "CgroupReleaseAgentDir", tbl)) {
 			xfree(tmp_str);
-			log_flag(CGROUP, "Ignoring obsolete CgroupReleaseAgentDir option.");
+			fatal("Support for CgroupReleaseAgentDir option has been removed.");
 		}
 
 		/* cgroup prepend directory */
@@ -341,11 +342,6 @@ static void _read_slurm_cgroup_conf(void)
 		if (!s_p_get_boolean(&slurm_cgroup_conf.constrain_cores,
 				     "ConstrainCores", tbl))
 			slurm_cgroup_conf.constrain_cores = false;
-		if (!s_p_get_boolean(&slurm_cgroup_conf.task_affinity,
-				     "TaskAffinity", tbl))
-			slurm_cgroup_conf.task_affinity = false;
-		else if (slurm_cgroup_conf.task_affinity)
-			fatal("Support for TaskAffinity=yes in cgroup.conf has been removed. Consider adding task/affinity to TaskPlugins in slurm.conf instead.");
 
 		/* RAM and Swap constraints related conf items */
 		if (!s_p_get_boolean(&slurm_cgroup_conf.constrain_ram_space,
@@ -404,15 +400,27 @@ static void _read_slurm_cgroup_conf(void)
 				     "ConstrainDevices", tbl))
 			slurm_cgroup_conf.constrain_devices = false;
 
-		if (!s_p_get_string(&slurm_cgroup_conf.allowed_devices_file,
-				    "AllowedDevicesFile", tbl))
-			slurm_cgroup_conf.allowed_devices_file =
-				get_extra_conf_path(
-					"cgroup_allowed_devices_file.conf");
+		if (s_p_get_string(&tmp_str, "AllowedDevicesFile", tbl)) {
+			xfree(tmp_str);
+			info("WARNING: AllowedDevicesFile option is obsolete, please remove it from your configuration.");
+		}
 
 		(void) s_p_get_string(&slurm_cgroup_conf.cgroup_plugin,
 				      "CgroupPlugin", tbl);
 
+		if (!s_p_get_boolean(&slurm_cgroup_conf.ignore_systemd,
+				     "IgnoreSystemd", tbl))
+			slurm_cgroup_conf.ignore_systemd = false;
+		else
+			/* Implicitly set these other one. */
+			slurm_cgroup_conf.ignore_systemd_on_failure = true;
+
+		if (!slurm_cgroup_conf.ignore_systemd &&
+		    (!s_p_get_boolean(
+			    &slurm_cgroup_conf.ignore_systemd_on_failure,
+			    "IgnoreSystemdOnFailure", tbl)))
+			slurm_cgroup_conf.ignore_systemd_on_failure = false;
+
 		s_p_hashtbl_destroy(tbl);
 	}
 
@@ -422,7 +430,7 @@ static void _read_slurm_cgroup_conf(void)
 }
 
 /* Autodetect logic inspired from systemd source code */
-static char *_autodetect_cgroup_version(void)
+extern char *autodetect_cgroup_version(void)
 {
 	struct statfs fs;
 	int cgroup_ver = -1;
@@ -462,7 +470,6 @@ static char *_autodetect_cgroup_version(void)
 
 	log_flag(CGROUP, "%s: using cgroup version %d", __func__, cgroup_ver);
 
-
 	switch (cgroup_ver) {
 	case 1:
 		return "cgroup/v1";
@@ -526,10 +533,27 @@ extern void cgroup_free_limits(cgroup_limits_t *limits)
 
 	xfree(limits->allow_cores);
 	xfree(limits->allow_mems);
-	xfree(limits->device_major);
 	xfree(limits);
 }
 
+extern void cgroup_init_limits(cgroup_limits_t *limits)
+{
+	if (!limits)
+		return;
+
+	memset(limits, 0, sizeof(*limits));
+
+	limits->taskid = NO_VAL;
+	limits->device.type = DEV_TYPE_NONE;
+	limits->device.major = NO_VAL;
+	limits->device.minor = NO_VAL;
+	limits->limit_in_bytes = NO_VAL64;
+	limits->soft_limit_in_bytes = NO_VAL64;
+	limits->kmem_limit_in_bytes = NO_VAL64;
+	limits->memsw_limit_in_bytes = NO_VAL64;
+	limits->swappiness = NO_VAL64;
+}
+
 /*
  * get_slurm_cgroup_conf - load the Slurm cgroup configuration from the
  *      cgroup.conf  file and return a key pair <name,value> ordered list.
@@ -565,12 +589,6 @@ extern List cgroup_get_conf_list(void)
 					 "yes" : "no");
 	list_append(cgroup_conf_l, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("TaskAffinity");
-	key_pair->value = xstrdup_printf("%s", cg_conf->task_affinity ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("ConstrainRAMSpace");
 	key_pair->value = xstrdup_printf("%s", cg_conf->constrain_ram_space ?
@@ -639,11 +657,6 @@ extern List cgroup_get_conf_list(void)
 					 "yes" : "no");
 	list_append(cgroup_conf_l, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("AllowedDevicesFile");
-	key_pair->value = xstrdup(cg_conf->allowed_devices_file);
-	list_append(cgroup_conf_l, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("MemorySwappiness");
 	if (cg_conf->memory_swappiness != NO_VAL64)
@@ -656,6 +669,20 @@ extern List cgroup_get_conf_list(void)
 	key_pair->value = xstrdup(cg_conf->cgroup_plugin);
 	list_append(cgroup_conf_l, key_pair);
 
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("IgnoreSystemd");
+	key_pair->value = xstrdup_printf("%s",
+					 cg_conf->ignore_systemd ?
+					 "yes" : "no");
+	list_append(cgroup_conf_l, key_pair);
+
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("IgnoreSystemdOnFailure");
+	key_pair->value = xstrdup_printf("%s",
+					 cg_conf->ignore_systemd_on_failure ?
+					 "yes" : "no");
+	list_append(cgroup_conf_l, key_pair);
+
 	list_sort(cgroup_conf_l, (ListCmpF) sort_key_pairs);
 
 	slurm_rwlock_unlock(&cg_conf_lock);
@@ -754,12 +781,12 @@ extern int cgroup_g_init(void)
 	cgroup_conf_init();
 	type = slurm_cgroup_conf.cgroup_plugin;
 
-	/* Default is cgroup/v1 */
+	/* Default is autodetect */
 	if (!type)
-		type = "cgroup/v1";
+		type = "autodetect";
 
 	if (!xstrcmp(type, "autodetect")) {
-		if (!(type = _autodetect_cgroup_version())) {
+		if (!(type = autodetect_cgroup_version())) {
 			rc = SLURM_ERROR;
 			goto done;
 		}
@@ -887,77 +914,31 @@ extern bool cgroup_g_has_pid(pid_t pid)
 	return (*(ops.has_pid))(pid);
 }
 
-extern cgroup_limits_t *cgroup_g_root_constrain_get(cgroup_ctl_type_t sub)
+extern cgroup_limits_t *cgroup_g_constrain_get(cgroup_ctl_type_t sub,
+					       cgroup_level_t level)
 {
 	if (cgroup_g_init() < 0)
 		return NULL;
 
-	return (*(ops.root_constrain_get))(sub);
+	return (*(ops.constrain_get))(sub, level);
 }
 
-extern int cgroup_g_root_constrain_set(cgroup_ctl_type_t sub,
-				       cgroup_limits_t *limits)
+extern int cgroup_g_constrain_set(cgroup_ctl_type_t sub, cgroup_level_t level,
+				  cgroup_limits_t *limits)
 {
 	if (cgroup_g_init() < 0)
 		return false;
 
-	return (*(ops.root_constrain_set))(sub, limits);
+	return (*(ops.constrain_set))(sub, level, limits);
 }
 
-extern cgroup_limits_t *cgroup_g_system_constrain_get(cgroup_ctl_type_t sub)
-{
-	if (cgroup_g_init() < 0)
-		return NULL;
-
-	return (*(ops.system_constrain_get))(sub);
-}
-
-extern int cgroup_g_system_constrain_set(cgroup_ctl_type_t sub,
-					 cgroup_limits_t *limits)
+extern int cgroup_g_constrain_apply(cgroup_ctl_type_t sub, cgroup_level_t level,
+                                    uint32_t task_id)
 {
 	if (cgroup_g_init() < 0)
 		return false;
 
-	return (*(ops.system_constrain_set))(sub, limits);
-}
-
-extern int cgroup_g_user_constrain_set(cgroup_ctl_type_t sub,
-				       stepd_step_rec_t *job,
-				       cgroup_limits_t *limits)
-{
-	if (cgroup_g_init() < 0)
-		return false;
-
-	return (*(ops.user_constrain_set))(sub, job, limits);
-}
-
-extern int cgroup_g_job_constrain_set(cgroup_ctl_type_t sub,
-				      stepd_step_rec_t *job,
-				      cgroup_limits_t *limits)
-{
-	if (cgroup_g_init() < 0)
-		return false;
-
-	return (*(ops.job_constrain_set))(sub, job, limits);
-}
-
-extern int cgroup_g_step_constrain_set(cgroup_ctl_type_t sub,
-				       stepd_step_rec_t *job,
-				       cgroup_limits_t *limits)
-{
-	if (cgroup_g_init() < 0)
-		return false;
-
-	return (*(ops.step_constrain_set))(sub, job, limits);
-}
-
-extern int cgroup_g_task_constrain_set(cgroup_ctl_type_t sub,
-				       cgroup_limits_t *limits, uint32_t taskid)
-{
-	if (cgroup_g_init() < 0)
-		return false;
-
-	return (*(ops.task_constrain_set))(sub, limits, taskid);
+	return (*(ops.constrain_apply))(sub, level, task_id);
 }
 
 extern int cgroup_g_step_start_oom_mgr()
@@ -992,3 +973,19 @@ extern cgroup_acct_t *cgroup_g_task_get_acct_data(uint32_t taskid)
 
 	return (*(ops.task_get_acct_data))(taskid);
 }
+
+extern long int cgroup_g_get_acct_units()
+{
+	if (cgroup_g_init() < 0)
+		return false;
+
+	return (*(ops.get_acct_units))();
+}
+
+extern bool cgroup_g_has_feature(cgroup_ctl_feature_t f)
+{
+	if (cgroup_g_init() < 0)
+		return false;
+
+	return (*(ops.has_feature))(f);
+}
