@@ -70,10 +70,9 @@
 #define GOV_PERFORMANCE		0x04
 #define GOV_POWERSAVE		0x08
 #define GOV_USERSPACE		0x10
+#define GOV_SCHEDUTIL		0x20
 
 static uint16_t cpu_freq_count = 0;
-static uint32_t cpu_freq_govs = 0; /* Governors allowed. */
-static uint64_t debug_flags = NO_VAL; /* init value for slurmd, slurmstepd */
 static int set_batch_freq = -1;
 
 static struct cpu_freq_data {
@@ -287,8 +286,6 @@ cpu_freq_init(slurmd_conf_t *conf)
 	char value[LINE_LEN];
 	unsigned int i, j;
 
-	debug_flags = slurm_get_debug_flags(); /* init for slurmd */
-
 	xfree(slurmd_spooldir);
 	slurmd_spooldir = xstrdup(conf->spooldir);
 
@@ -332,43 +329,39 @@ cpu_freq_init(slurmd_conf_t *conf)
 		}
 		if (strstr(value, "conservative")) {
 			cpufreq[i].avail_governors |= GOV_CONSERVATIVE;
-			if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
-				info("cpu_freq: Conservative governor "
-				     "defined on cpu 0");
-			}
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: Conservative governor defined on cpu 0");
 		}
 		if (strstr(value, "ondemand")) {
 			cpufreq[i].avail_governors |= GOV_ONDEMAND;
-			if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
-				info("cpu_freq: OnDemand governor "
-				     "defined on cpu 0");
-			}
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: OnDemand governor defined on cpu 0");
 		}
 		if (strstr(value, "performance")) {
 			cpufreq[i].avail_governors |= GOV_PERFORMANCE;
-			if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
-				info("cpu_freq: Performance governor "
-				     "defined on cpu 0");
-			}
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: Performance governor defined on cpu 0");
 		}
 		if (strstr(value, "powersave")) {
 			cpufreq[i].avail_governors |= GOV_POWERSAVE;
-			if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
-				info("cpu_freq: PowerSave governor "
-				     "defined on cpu 0");
-			}
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: PowerSave governor defined on cpu 0");
 		}
 		if (strstr(value, "userspace")) {
 			cpufreq[i].avail_governors |= GOV_USERSPACE;
-			if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
-				info("cpu_freq: UserSpace governor "
-				     "defined on cpu 0");
-			}
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: UserSpace governor defined on cpu 0");
+		}
+		if (strstr(value, "schedutil")) {
+			cpufreq[i].avail_governors |= GOV_SCHEDUTIL;
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: SchedUtil governor defined on cpu 0");
 		}
 		fclose(fp);
 		if (_cpu_freq_cpu_avail(i) == SLURM_ERROR)
 			continue;
-		if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
+		if ((i == 0) &&
+		    (slurm_conf.debug_flags & DEBUG_FLAG_CPU_FREQ)) {
 			for (j = 0; j < cpufreq[i].nfreq; j++) {
 				info("cpu_freq: frequency %u defined on cpu 0",
 				     cpufreq[i].avail_freq[j]);
@@ -385,17 +378,6 @@ cpu_freq_fini(void)
 	xfree(slurmd_spooldir);
 }
 
-/*
- * reset debug flag (slurmd)
- */
-extern void
-cpu_freq_reconfig(void)
-{
-	/* reset local static variables */
-	cpu_freq_govs = 0;
-	debug_flags = slurm_get_debug_flags();
-}
-
 /*
  * Send the cpu_frequency table info to slurmstepd
  */
@@ -446,7 +428,7 @@ rwfail:
 
 /*
  * Validate the cpus and select the frequency to set
- * Called from task cpuset code with task launch request containing
+ * Called from task/affinity code with task launch request containing
  *  a pointer to a hex map string of the cpus to be used by this step
  */
 extern void
@@ -458,37 +440,32 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *job)
 	char *cpu_bind;
 	char *cpu_str;
 	char *savestr = NULL;
+	char cpu_bind_type_string[128];
 
 	if (set_batch_freq == -1) {
-		char *launch_params = slurm_get_launch_params();
-		if (xstrcasestr(launch_params, "batch_step_set_cpu_freq"))
+		if (xstrcasestr(slurm_conf.launch_params,
+				"batch_step_set_cpu_freq"))
 			set_batch_freq = 1;
 		else
 			set_batch_freq = 0;
-		xfree(launch_params);
 	}
 
-	if (((job->stepid == SLURM_BATCH_SCRIPT) && !set_batch_freq) ||
-	    (job->stepid == SLURM_EXTERN_CONT))
+	if (((job->step_id.step_id == SLURM_BATCH_SCRIPT) && !set_batch_freq) ||
+	    (job->step_id.step_id == SLURM_INTERACTIVE_STEP) ||
+	    (job->step_id.step_id == SLURM_EXTERN_CONT))
 		return;
 
-	debug_flags = slurm_get_debug_flags(); /* init for slurmstepd */
-	if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
-		char cpu_bind_type_string[128];
+	slurm_sprint_cpu_bind_type(cpu_bind_type_string, job->cpu_bind_type);
 
-		slurm_sprint_cpu_bind_type(cpu_bind_type_string,
-				           job->cpu_bind_type);
-		info("cpu_freq_cpuset_validate: request: min=(%12d  %8x) "
-		      "max=(%12d %8x) governor=%8x",
-		      job->cpu_freq_min, job->cpu_freq_min,
-		      job->cpu_freq_max, job->cpu_freq_max,
-		      job->cpu_freq_gov);
-		info("  jobid=%u, stepid=%u, tasks=%u cpu/task=%u, cpus=%u",
-		     job->jobid, job->stepid, job->node_tasks,
-		     job->cpus_per_task, job->cpus);
-		info("  cpu_bind_type=%s, cpu_bind map=%s",
-		     cpu_bind_type_string, job->cpu_bind);
-	}
+	log_flag(CPU_FREQ, "%s: request: min=(%12d  %8x) max=(%12d %8x) governor=%8x",
+		 __func__, job->cpu_freq_min, job->cpu_freq_min,
+		 job->cpu_freq_max, job->cpu_freq_max, job->cpu_freq_gov);
+	log_flag(CPU_FREQ, "  jobid=%u, stepid=%u, tasks=%u cpu/task=%u, cpus=%u",
+		 job->step_id.job_id, job->step_id.step_id,
+		 job->node_tasks, job->cpus_per_task,
+		 job->cpus);
+	log_flag(CPU_FREQ, "  cpu_bind_type=%4x, cpu_bind map=%s",
+		 job->cpu_bind_type, job->cpu_bind);
 
 	if (!cpu_freq_count)
 		return;
@@ -569,33 +546,30 @@ cpu_freq_cgroup_validate(stepd_step_rec_t *job, char *step_alloc_cores)
 	char *core_range;
 
 	if (set_batch_freq == -1) {
-		char *launch_params = slurm_get_launch_params();
-		if (xstrcasestr(launch_params, "batch_step_set_cpu_freq"))
+		if (xstrcasestr(slurm_conf.launch_params,
+				"batch_step_set_cpu_freq"))
 			set_batch_freq = 1;
 		else
 			set_batch_freq = 0;
-		xfree(launch_params);
 	}
 
-	if (((job->stepid == SLURM_BATCH_SCRIPT) && !set_batch_freq) ||
-	    (job->stepid == SLURM_EXTERN_CONT))
+	if (((job->step_id.step_id == SLURM_BATCH_SCRIPT) && !set_batch_freq) ||
+	    (job->step_id.step_id == SLURM_INTERACTIVE_STEP) ||
+	    (job->step_id.step_id == SLURM_EXTERN_CONT))
 		return;
 
-	debug_flags = slurm_get_debug_flags(); /* init for slurmstepd */
-	if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
-		info("cpu_freq_cgroup_validate: request: min=(%12d  %8x) "
-				"max=(%12d %8x) governor=%8x",
-		       job->cpu_freq_min, job->cpu_freq_min,
-		       job->cpu_freq_max, job->cpu_freq_max,
-		       job->cpu_freq_gov);
-		info("  jobid=%u, stepid=%u, tasks=%u cpu/task=%u, cpus=%u",
-		       job->jobid,job->stepid,job->node_tasks,
-		       job->cpus_per_task,job->cpus);
-		info("  cpu_bind_type=%4x, cpu_bind map=%s",
-		       job->cpu_bind_type, job->cpu_bind);
-		info("  step logical cores = %s, step physical cores = %s",
-		       job->step_alloc_cores, step_alloc_cores);
-	}
+	log_flag(CPU_FREQ, "%s: request: min=(%12d  %8x) max=(%12d %8x) governor=%8x",
+		 __func__, job->cpu_freq_min, job->cpu_freq_min,
+		 job->cpu_freq_max, job->cpu_freq_max, job->cpu_freq_gov);
+	log_flag(CPU_FREQ, "  jobid=%u, stepid=%u, tasks=%u cpu/task=%u, cpus=%u",
+		 job->step_id.job_id, job->step_id.step_id,
+		 job->node_tasks, job->cpus_per_task,
+		 job->cpus);
+	log_flag(CPU_FREQ, "  cpu_bind_type=%4x, cpu_bind map=%s",
+		 job->cpu_bind_type, job->cpu_bind);
+	log_flag(CPU_FREQ, "  step logical cores = %s, step physical cores = %s",
+		 job->step_alloc_cores, step_alloc_cores);
+
 	if (!cpu_freq_count)
 		return;
 
@@ -732,11 +706,21 @@ _cpu_freq_set_gov(stepd_step_rec_t *job, int cpuidx, char* gov )
 	char path[PATH_MAX];
 	FILE *fp;
 	int fd, rc;
+	uint32_t jobid;
+
+#ifdef HAVE_NATIVE_CRAY
+	if (job->het_job_id && (job->het_job_id != NO_VAL))
+		jobid = job->het_job_id;
+	else
+		jobid = job->step_id.job_id;
+#else
+	jobid = job->step_id.job_id;
+#endif
 
 	rc = SLURM_SUCCESS;
 	snprintf(path, sizeof(path), PATH_TO_CPU
 		 "cpu%u/cpufreq/scaling_governor", cpuidx);
-	fd = _set_cpu_owner_lock(cpuidx, job->jobid);
+	fd = _set_cpu_owner_lock(cpuidx, jobid);
 	if ((fp = fopen(path, "w"))) {
 		fputs(gov, fp);
 		fputc('\n', fp);
@@ -811,11 +795,21 @@ _cpu_freq_set_scaling_freq(stepd_step_rec_t *job, int cpx, uint32_t freq,
 	FILE *fp;
 	int fd, rc;
 	uint32_t newfreq;
+	uint32_t jobid;
+
+#ifdef HAVE_NATIVE_CRAY
+	if (job->het_job_id && (job->het_job_id != NO_VAL))
+		jobid = job->het_job_id;
+	else
+		jobid = job->step_id.job_id;
+#else
+	jobid = job->step_id.job_id;
+#endif
 
 	rc = SLURM_SUCCESS;
 	snprintf(path, sizeof(path), PATH_TO_CPU
 		 "cpu%u/cpufreq/%s", cpx, option);
-	fd = _set_cpu_owner_lock(cpx, job->jobid);
+	fd = _set_cpu_owner_lock(cpx, jobid);
 	if ((fp = fopen(path, "w"))) {
 		fprintf(fp, "%u\n", freq);
 		fclose(fp);
@@ -827,7 +821,7 @@ _cpu_freq_set_scaling_freq(stepd_step_rec_t *job, int cpx, uint32_t freq,
 		(void) fd_release_lock(fd);
 		(void) close(fd);
 	}
-	if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_CPU_FREQ) {
 		newfreq = _cpu_freq_get_scaling_freq(cpx, option);
 		if (newfreq != freq) {
 			error("Failed to set freq_scaling %s to %u (org=%u)",
@@ -910,7 +904,7 @@ _cpu_freq_govspec_string(uint32_t cpu_freq, int cpuidx)
 
 	if ((cpu_freq & CPU_FREQ_RANGE_FLAG) == 0)
 		return SLURM_ERROR;
-		
+
 	switch(cpu_freq)
 	{
 	case CPU_FREQ_CONSERVATIVE:
@@ -933,6 +927,10 @@ _cpu_freq_govspec_string(uint32_t cpu_freq, int cpuidx)
 		if (cpufreq[cpuidx].avail_governors & GOV_USERSPACE)
 			strcpy(cpufreq[cpuidx].new_governor, "userspace");
 		return SLURM_SUCCESS;
+	case CPU_FREQ_SCHEDUTIL:
+		if (cpufreq[cpuidx].avail_governors & GOV_SCHEDUTIL)
+			strcpy(cpufreq[cpuidx].new_governor, "schedutil");
+		return SLURM_SUCCESS;
 	default:
 		return SLURM_ERROR;
 	}
@@ -954,27 +952,27 @@ _cpu_freq_freqspec_num(uint32_t cpu_freq, int cpuidx)
 		{
 		case CPU_FREQ_LOW :
 			return cpufreq[cpuidx].avail_freq[0];
-	
+
 		case CPU_FREQ_MEDIUM :
 			if (cpufreq[cpuidx].nfreq == 1)
 				return cpufreq[cpuidx].avail_freq[0];
 			fx = (cpufreq[cpuidx].nfreq - 1) / 2;
 			return cpufreq[cpuidx].avail_freq[fx];
-	
+
 		case CPU_FREQ_HIGHM1 :
 			if (cpufreq[cpuidx].nfreq == 1)
 				return cpufreq[cpuidx].avail_freq[0];
 			fx = cpufreq[cpuidx].nfreq - 2;
 			return cpufreq[cpuidx].avail_freq[fx];
-	
+
 		case CPU_FREQ_HIGH :
 			fx = cpufreq[cpuidx].nfreq - 1;
 			return cpufreq[cpuidx].avail_freq[fx];
-		
+
 		default:
 			return NO_VAL;
 		}
-	}		
+	}
 
 	/* check for request above or below available values */
 	if (cpu_freq < cpufreq[cpuidx].avail_freq[0]) {
@@ -1039,7 +1037,7 @@ _cpu_freq_setup_data(stepd_step_rec_t *job, int cpx)
 	    && (job->cpu_freq_max == NO_VAL || job->cpu_freq_max==0)
 	    && (job->cpu_freq_gov == NO_VAL || job->cpu_freq_gov==0)) {
 		/* If no --cpu-freq, use default governor from conf file.  */
-		slurm_ctl_conf_t *conf = slurm_conf_lock();
+		slurm_conf_t *conf = slurm_conf_lock();
 		job->cpu_freq_gov = conf->cpu_freq_def;
 		slurm_conf_unlock();
 		if (job->cpu_freq_gov == NO_VAL)
@@ -1114,6 +1112,8 @@ _cpu_freq_check_gov(const char* arg, uint32_t illegal)
 		rc = CPU_FREQ_USERSPACE;
 	} else if (xstrncasecmp(arg, "onde", 4) == 0) {
 		rc = CPU_FREQ_ONDEMAND;
+	} else if (xstrncasecmp(arg, "sche", 4) == 0) {
+		rc = CPU_FREQ_SCHEDUTIL;
 	}
 	rc &= (~illegal);
 	if (rc == 0)
@@ -1172,14 +1172,10 @@ cpu_freq_set(stepd_step_rec_t *job)
 	            && cpufreq[i].new_max_freq == NO_VAL
 		    && cpufreq[i].new_governor[0] == '\0')
 			continue; /* Nothing to set on this CPU */
-		if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
-			info("cpu_freq: current_state cpu=%d org_min=%u "
-			     "org_freq=%u org_max=%u org_gpv=%s", i,
-			     cpufreq[i].org_min_freq,
-			     cpufreq[i].org_frequency,
-			     cpufreq[i].org_max_freq,
-			     cpufreq[i].org_governor);
-		}
+
+		log_flag(CPU_FREQ, "cpu_freq: current_state cpu=%d org_min=%u org_freq=%u org_max=%u org_gpv=%s",
+			 i, cpufreq[i].org_min_freq, cpufreq[i].org_frequency,
+			 cpufreq[i].org_max_freq, cpufreq[i].org_governor);
 
 		/* Max must be set before min, per
 		 * www.kernel.org/doc/Documentation/cpu-freq/user-guide.txt
@@ -1253,7 +1249,7 @@ cpu_freq_set(stepd_step_rec_t *job)
 			if (rc == SLURM_ERROR)
 				continue;
 		}
-		if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
+		if (slurm_conf.debug_flags & DEBUG_FLAG_CPU_FREQ) {
 			cpu_freq_debug(NULL, NULL,
 					freq_detail, sizeof(freq_detail),
 					NO_VAL, cpufreq[i].new_min_freq,
@@ -1288,9 +1284,9 @@ cpu_freq_reset(stepd_step_rec_t *job)
 	if (job->het_job_id && (job->het_job_id != NO_VAL))
 		jobid = job->het_job_id;
 	else
-		jobid = job->jobid;
+		jobid = job->step_id.job_id;
 #else
-	jobid = job->jobid;
+	jobid = job->step_id.job_id;
 #endif
 
 	for (i = 0; i < cpu_freq_count; i++) {
@@ -1338,7 +1334,7 @@ cpu_freq_reset(stepd_step_rec_t *job)
 				continue;
 		}
 
-		if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
+		if (slurm_conf.debug_flags & DEBUG_FLAG_CPU_FREQ) {
 			cpu_freq_debug(NULL, NULL,
 					freq_detail, sizeof(freq_detail),
 					NO_VAL, cpufreq[i].org_min_freq,
@@ -1377,6 +1373,8 @@ cpu_freq_to_string(char *buf, int buf_size, uint32_t cpu_freq)
 		snprintf(buf, buf_size, "UserSpace");
 	else if (cpu_freq == CPU_FREQ_ONDEMAND)
 		snprintf(buf, buf_size, "OnDemand");
+	else if (cpu_freq == CPU_FREQ_SCHEDUTIL)
+		snprintf(buf, buf_size, "SchedUtil");
 	else if (cpu_freq & CPU_FREQ_RANGE_FLAG)
 		snprintf(buf, buf_size, "Unknown");
 	else if (fuzzy_equal(cpu_freq, NO_VAL)) {
@@ -1466,46 +1464,31 @@ extern void
 cpu_freq_govlist_to_string(char* buf, uint16_t bufsz, uint32_t govs)
 {
 	char *list = NULL;
+	char *sep = "", *pos = NULL;
 
 	if ((govs & CPU_FREQ_CONSERVATIVE) == CPU_FREQ_CONSERVATIVE) {
-		if (list == NULL)
-			list = xstrdup("Conservative");
-		else {
-			xstrcatchar(list,',');
-			xstrcat(list,"Conservative");
-		}
-	}
-	if ((govs & CPU_FREQ_PERFORMANCE) == CPU_FREQ_PERFORMANCE) {
-		if (list == NULL)
-			list = xstrdup("Performance");
-		else {
-			xstrcatchar(list,',');
-			xstrcat(list,"Performance");
-		}
-	}
-	if ((govs & CPU_FREQ_POWERSAVE) == CPU_FREQ_POWERSAVE) {
-		if (list == NULL)
-			list = xstrdup("PowerSave");
-		else {
-			xstrcatchar(list,',');
-			xstrcat(list,"PowerSave");
-		}
+		xstrfmtcatat(list, &pos, "%s%s", sep, "Conservative");
+		sep = ",";
 	}
 	if ((govs & CPU_FREQ_ONDEMAND) == CPU_FREQ_ONDEMAND) {
-		if (list == NULL)
-			list = xstrdup("OnDemand");
-		else {
-			xstrcatchar(list,',');
-			xstrcat(list,"OnDemand");
-		}
+		xstrfmtcatat(list, &pos, "%s%s", sep, "OnDemand");
+		sep = ",";
+	}
+	if ((govs & CPU_FREQ_PERFORMANCE) == CPU_FREQ_PERFORMANCE) {
+		xstrfmtcatat(list, &pos, "%s%s", sep, "Performance");
+		sep = ",";
+	}
+	if ((govs & CPU_FREQ_POWERSAVE) == CPU_FREQ_POWERSAVE) {
+		xstrfmtcatat(list, &pos, "%s%s", sep, "PowerSave");
+		sep = ",";
+	}
+	if ((govs & CPU_FREQ_SCHEDUTIL) == CPU_FREQ_SCHEDUTIL) {
+		xstrfmtcatat(list, &pos, "%s%s", sep, "SchedUtil");
+		sep = ",";
 	}
 	if ((govs & CPU_FREQ_USERSPACE) == CPU_FREQ_USERSPACE) {
-		if (list == NULL)
-			list = xstrdup("UserSpace");
-		else {
-			xstrcatchar(list,',');
-			xstrcat(list,"UserSpace");
-		}
+		xstrfmtcatat(list, &pos, "%s%s", sep, "UserSpace");
+		sep = ",";
 	}
 
 	if (list) {
@@ -1613,10 +1596,6 @@ cpu_freq_verify_cmdline(const char *arg,
 	uint32_t frequency;
 	int rc = 0;
 
-	if (cpu_freq_govs == 0)
-		cpu_freq_govs = slurm_get_cpu_freq_govs();
-
-
 	if (arg == NULL || cpu_freq_min == NULL || cpu_freq_max == NULL
 			|| cpu_freq_gov == NULL) {
 		return -1;
@@ -1694,7 +1673,7 @@ cpu_freq_verify_cmdline(const char *arg,
 
 clean:
 	if (*cpu_freq_gov != NO_VAL) {
-		if (((*cpu_freq_gov & cpu_freq_govs)
+		if (((*cpu_freq_gov & slurm_conf.cpu_freq_govs)
 		    & ~CPU_FREQ_RANGE_FLAG) == 0) {
 			error("governor of %s is not allowed in slurm.conf",
 			      arg);
@@ -1702,7 +1681,7 @@ clean:
 			rc = -1;
 		}
 	}
-	if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_CPU_FREQ) {
 		cpu_freq_debug("command", "NO_VAL", NULL, 0,
 			       *cpu_freq_gov, *cpu_freq_min,
 			       *cpu_freq_max, NO_VAL);
