@@ -62,7 +62,7 @@ typedef enum {
 typedef struct {
 	int (*init)(void);
 	int (*fini)(void);
-	data_t *(*get_oas)(void);
+	data_t *(*get_oas)(openapi_spec_flags_t *flags);
 } slurm_openapi_ops_t;
 
 /*
@@ -128,6 +128,7 @@ typedef struct {
 typedef struct {
 	data_t *src_paths;
 	data_t *dst_paths;
+	openapi_spec_flags_t flags;
 } merge_path_server_t;
 
 typedef struct {
@@ -137,9 +138,16 @@ typedef struct {
 
 typedef struct {
 	data_t *paths;
-	const char *server_path;
+	data_t *server_path;
+	openapi_spec_flags_t flags;
 } merge_path_t;
 
+typedef struct {
+	data_t *server_path;
+	char *operation;
+	char *at;
+} id_merge_path_t;
+
 typedef struct {
 	char *path;
 	char *at;
@@ -150,6 +158,7 @@ struct openapi_s {
 	List paths;
 	int path_tag_counter;
 	data_t **spec;
+	openapi_spec_flags_t *spec_flags;
 
 	slurm_openapi_ops_t *ops;
 	int context_cnt;
@@ -221,7 +230,7 @@ static const char *_get_entry_type_string(entry_type_t type)
 	}
 }
 
-static void _free_entry_list(entry_t *entry, path_t *path,
+static void _free_entry_list(entry_t *entry, int tag,
 			     entry_method_t *method)
 {
 	entry_t *itr = entry;
@@ -231,9 +240,9 @@ static void _free_entry_list(entry_t *entry, path_t *path,
 
 	while (itr->type) {
 		debug5("%s: remove path tag:%d method:%s entry:%s name:%s",
-		       __func__, (path ? path->tag : -1),
+		       __func__, tag,
 		       (method ? get_http_method_string(method->method) :
-				       "UNKNOWN"),
+				       "N/A"),
 		       itr->entry, itr->name);
 		xfree(itr->entry);
 		xfree(itr->name);
@@ -245,22 +254,22 @@ static void _free_entry_list(entry_t *entry, path_t *path,
 
 static void _list_delete_path_t(void *x)
 {
-	entry_method_t *method;
+	entry_method_t *em;
 
 	if (!x)
 		return;
 
 	path_t *path = x;
 	xassert(path->tag != -1);
-	method = path->methods;
+	em = path->methods;
 
-	while (method->method) {
+	while (em->entries) {
 		debug5("%s: remove path tag:%d method:%s", __func__, path->tag,
-		       get_http_method_string(method->method));
+		       get_http_method_string(em->method));
 
-		_free_entry_list(method->entries, path, method);
-		method->entries = NULL;
-		method++;
+		_free_entry_list(em->entries, path->tag, em);
+		em->entries = NULL;
+		em++;
 	}
 
 	xfree(path->methods);
@@ -330,11 +339,52 @@ again:
 	return entries;
 
 fail:
+	for (entry = entries; entry <= entries + count; entry++)
+		xfree(entry->entry);
+
 	xfree(entries);
 	xfree(buffer);
 	return NULL;
 }
 
+static int _print_path_tag_methods(void *x, void *arg)
+{
+	path_t *path = (path_t *) x;
+	int *tag = (int *) arg;
+	char *methods_str = NULL;
+
+	if (path->tag != *tag)
+		return 0;
+
+	for (entry_method_t *em = path->methods; em->entries; em++)
+		xstrfmtcat(methods_str, "%s%s (%d)", methods_str ? ", " : "",
+			   get_http_method_string(em->method), em->method);
+
+	if (methods_str)
+		debug4("%s:    methods: %s", __func__, methods_str);
+	else
+		debug4("%s:    no methods found in path tag %d",
+		       __func__, path->tag);
+	xfree(methods_str);
+
+	/*
+	 * We found the (unique) tag, so return -1 to exit early. The item's
+	 * index returned by list_for_each_ro() will be negative.
+	 */
+	return -1;
+}
+
+extern void print_path_tag_methods(openapi_t *oas, int tag)
+{
+	if (get_log_level() < LOG_LEVEL_DEBUG4)
+		return;
+
+	xassert(oas->magic == MAGIC_OAS);
+
+	if (list_for_each_ro(oas->paths, _print_path_tag_methods, &tag) >= 0)
+		error("%s: Tag %d not found in oas->paths", __func__, tag);
+}
+
 static bool _match_server_path(const data_t *server_path, const data_t *path,
 			       const data_t *match_path)
 {
@@ -544,11 +594,14 @@ static data_for_each_cmd_t _populate_methods(const char *key,
 	const data_t *para;
 	int count = 0;
 	entry_t *entry;
+	http_request_method_t method_type = get_http_method(key);
 
-	if ((method->method = get_http_method(key)) == HTTP_REQUEST_INVALID)
+	if (method_type == HTTP_REQUEST_INVALID)
 		/* Ignore none HTTP method dictionary keys */
 		return DATA_FOR_EACH_CONT;
 
+	method->method = method_type;
+
 	if (data_get_type(data) != DATA_TYPE_DICT)
 		fatal("%s: unexpected data type %s instead of dictionary",
 		      __func__, data_type_to_string(data_get_type(data)));
@@ -559,23 +612,26 @@ static data_for_each_cmd_t _populate_methods(const char *key,
 	if (!method->entries) {
 		/* only add entries on first method parse */
 		method->entries = xcalloc((count + 1), sizeof(entry_t));
-		/* count is already bounded */
-		memcpy(method->entries, args->entries,
-		       (count * sizeof(entry_t)));
-	}
-
-	/* unlink strings from source */
-	for (entry = args->entries; entry->type; entry++) {
-		entry->entry = NULL;
-		entry->name = NULL;
+		/* Copy spec entry list into method entry list */
+		entry_t *dest = method->entries;
+		for (entry_t *src = args->entries; src->type; src++) {
+			dest->entry = xstrdup(src->entry);
+			dest->name = xstrdup(src->name);
+			dest->type = src->type;
+			dest->parameter = src->parameter;
+			dest++;
+		}
 	}
 
 	/* point to new entries clone */
 	nargs.entries = method->entries;
 
 	para = data_key_get_const(data, "parameters");
-	if (!para)
+	if (!para) {
+		/* increment to next method entry */
+		args->method++;
 		return DATA_FOR_EACH_CONT;
+	}
 	if (data_get_type(para) != DATA_TYPE_LIST)
 		return DATA_FOR_EACH_FAIL;
 	if (data_list_for_each_const(para, _populate_parameters, &nargs) < 0)
@@ -630,7 +686,7 @@ extern int register_path_tag(openapi_t *oas, const char *str_path)
 	rc = path->tag;
 
 cleanup:
-	_free_entry_list(entries, path, args.method);
+	_free_entry_list(entries, (path ? path->tag : -1), NULL);
 	entries = NULL;
 
 	return rc;
@@ -797,7 +853,6 @@ extern int find_path_tag(openapi_t *oas, const data_t *dpath, data_t *params,
 			 http_request_method_t method)
 {
 	path_t *path;
-	int tag = -1;
 	match_path_from_data_t args = {
 		.params = params,
 		.dpath = dpath,
@@ -807,10 +862,16 @@ extern int find_path_tag(openapi_t *oas, const data_t *dpath, data_t *params,
 	xassert(data_get_type(params) == DATA_TYPE_DICT);
 
 	path = list_find_first(oas->paths, _match_path_from_data, &args);
-	if (path)
-		tag = path->tag;
+	if (!path)
+		return -1;
 
-	return tag;
+	/* Make sure the path tag actually contains the method requested */
+	for (entry_method_t *em = path->methods; em->entries; em++) {
+		if (em->method == method)
+			return path->tag;
+	}
+
+	return -2;
 }
 
 static void _oas_plugrack_foreach(const char *full_type, const char *fq_path,
@@ -899,8 +960,11 @@ extern int init_openapi(openapi_t **oas, const char *plugins,
 	t->ops = xcalloc((t->plugin_count + 1), sizeof(*t->ops));
 	t->context = xcalloc((t->plugin_count + 1), sizeof(*t->context));
 	t->spec = xcalloc((t->plugin_count + 1), sizeof(*t->spec));
+	t->spec_flags = xcalloc((t->plugin_count + 1), sizeof(*t->spec_flags));
 
 	for (size_t i = 0; (i < t->plugin_count); i++) {
+		openapi_spec_flags_t flags = OAS_FLAG_NONE;
+
 		if (t->plugin_handles[i] == PLUGIN_INVALID_HANDLE) {
 			error("Invalid plugin to load?");
 			rc = ESLURM_PLUGIN_INVALID;
@@ -915,13 +979,18 @@ extern int init_openapi(openapi_t **oas, const char *plugins,
 			break;
 		}
 
-		t->spec[t->context_cnt] = (*(t->ops[t->context_cnt].get_oas))();
+		t->spec[t->context_cnt] =
+			(*(t->ops[t->context_cnt].get_oas))(&flags);
+		t->spec_flags[t->context_cnt] = flags;
 		if (!t->spec[t->context_cnt]) {
 			error("unable to load OpenAPI spec");
 			rc = ESLURM_PLUGIN_INCOMPLETE;
 			break;
 		}
 
+		debug2("%s: loaded plugin %s with flags 0x%"PRIx64,
+		       __func__, t->plugin_types[i], flags);
+
 		t->context_cnt++;
 	}
 
@@ -955,6 +1024,7 @@ extern void destroy_openapi(openapi_t *oas)
 	for (size_t i = 0; oas->spec[i]; i++)
 		FREE_NULL_DATA(oas->spec[i]);
 	xfree(oas->spec);
+	xfree(oas->spec_flags);
 	xfree(oas->ops);
 
 	for (size_t i = 0; i < oas->plugin_count; i++) {
@@ -962,6 +1032,7 @@ extern void destroy_openapi(openapi_t *oas)
 		xfree(oas->plugin_types[i]);
 	}
 	xfree(oas->plugin_types);
+	xfree(oas->plugin_handles);
 	if ((rc = plugrack_destroy(oas->rack)))
 		fatal_abort("unable to clean up plugrack: %s",
 			    slurm_strerror(rc));
@@ -1058,13 +1129,97 @@ data_for_each_cmd_t _merge_path_strings(data_t *data, void *arg)
 	return DATA_FOR_EACH_CONT;
 }
 
+data_for_each_cmd_t _merge_operationId_strings(data_t *data, void *arg)
+{
+	id_merge_path_t *args = arg;
+	char *p;
+
+	if (data_convert_type(data, DATA_TYPE_STRING) != DATA_TYPE_STRING)
+		return DATA_FOR_EACH_FAIL;
+
+	p = data_get_string(data);
+
+	/* sub out '.' for '_' to avoid breaking compilers */
+	for (int s = strlen(p), i = 0; i < s; i++)
+		if (p[i] == '.')
+			p[i] = '_';
+
+	xstrfmtcatat(args->operation, &args->at, "%s%s",
+		     (args->operation ? "_" : ""), data_get_string(data));
+
+	return DATA_FOR_EACH_CONT;
+}
+
+/*
+ * Merge plugin id with operationIds in paths.
+ * All operationIds must be globaly unique.
+ */
+data_for_each_cmd_t _differentiate_path_operationId(const char *key,
+						    data_t *data, void *arg)
+{
+	data_t *merge[3] = { 0 }, *merged = NULL;
+	id_merge_path_t *args = arg;
+	data_t *op;
+
+	if (data_get_type(data) != DATA_TYPE_DICT)
+		return DATA_FOR_EACH_CONT;
+
+	if (!(op = data_key_get(data, "operationId"))) {
+		debug2("%s: unexpected missing operationId",
+		      __func__);
+		return DATA_FOR_EACH_CONT;
+	}
+
+	/* force operationId to be a string */
+	if (data_convert_type(op, DATA_TYPE_STRING) != DATA_TYPE_STRING) {
+		error("%s: unexpected type for operationId: %s",
+		      __func__, data_type_to_string(data_get_type(op)));
+		return DATA_FOR_EACH_FAIL;
+	}
+
+	merge[0] = args->server_path;
+	merge[1] = parse_url_path(data_get_string_const(op), false, true);
+	merged = data_list_join((const data_t **)merge, true);
+	FREE_NULL_DATA(merge[1]);
+	if (data_list_for_each(merged, _merge_operationId_strings, args) < 0) {
+		FREE_NULL_DATA(merged);
+		return DATA_FOR_EACH_FAIL;
+	}
+
+	data_set_string_own(op, args->operation);
+	args->operation = NULL;
+	FREE_NULL_DATA(merged);
+
+	return DATA_FOR_EACH_CONT;
+}
+
+data_for_each_cmd_t _find_first_server(data_t *data, void *arg)
+{
+	data_t **srv = arg;
+	data_t *url;
+
+	if (data_get_type(data) != DATA_TYPE_DICT)
+		return DATA_FOR_EACH_FAIL;
+
+	url = data_key_get(data, "url");
+
+	if (data_convert_type(url, DATA_TYPE_STRING) == DATA_TYPE_STRING) {
+		*srv = parse_url_path(data_get_string(url), false, false);
+		return DATA_FOR_EACH_STOP;
+	}
+
+	return DATA_FOR_EACH_FAIL;
+}
+
 data_for_each_cmd_t _merge_path(const char *key, data_t *data, void *arg)
 {
 	merge_path_t *args = arg;
-	data_t *e;
+	data_t *e, *servers;
 	data_t *merge[3] = { 0 }, *merged = NULL;
 	merge_path_strings_t mp_args = { 0 };
 	data_for_each_cmd_t rc = DATA_FOR_EACH_CONT;
+	id_merge_path_t id_merge = { 0 };
+	bool free_0 = false; /* free merge[0] ? */
 
 	if (data_get_type(data) != DATA_TYPE_DICT) {
 		rc = DATA_FOR_EACH_FAIL;
@@ -1072,15 +1227,24 @@ data_for_each_cmd_t _merge_path(const char *key, data_t *data, void *arg)
 	}
 
 	/* merge the paths together cleanly */
-	if (!data_key_get(data, "servers")) {
-		merge[0] = parse_url_path(args->server_path, false, true);
+	if (!(servers = data_key_get(data, "servers"))) {
+		merge[0] = id_merge.server_path = args->server_path;
 		merge[1] = parse_url_path(key, false, true);
 	} else {
 		/* servers is specified: only cleanup the path */
-		merge[0] = parse_url_path(key, false, true);
-	}
-	merged = data_list_join((const data_t **)merge, true);
+		/* only handling 1 server for now */
+		xassert(data_get_list_length(servers) == 1);
 
+		(void) data_list_for_each(servers, _find_first_server,
+					  &merge[0]);
+		id_merge.server_path = merge[0];
+		free_0 = true;
+		xassert(merge[0]);
+
+		merge[1] = parse_url_path(key, false, true);
+	}
+
+	merged = data_list_join((const data_t **)merge, true);
 	if (data_list_for_each(merged, _merge_path_strings, &mp_args) < 0) {
 		rc = DATA_FOR_EACH_FAIL;
 		goto cleanup;
@@ -1098,11 +1262,18 @@ data_for_each_cmd_t _merge_path(const char *key, data_t *data, void *arg)
 	data_set_dict(e);
 	data_copy(e, data);
 
-cleanup:
+	if ((args->flags & OAS_FLAG_MANGLE_OPID) &&
+	    data_dict_for_each(e, _differentiate_path_operationId,
+			       &id_merge) < 0) {
+		rc = DATA_FOR_EACH_FAIL;
+		goto cleanup;
+	}
 
-	FREE_NULL_DATA(merged);
-	FREE_NULL_DATA(merge[0]);
+cleanup:
+	if (free_0)
+		FREE_NULL_DATA(merge[0]);
 	FREE_NULL_DATA(merge[1]);
+	FREE_NULL_DATA(merged);
 	xfree(mp_args.path);
 
 	return rc;
@@ -1113,6 +1284,7 @@ data_for_each_cmd_t _merge_path_server(data_t *data, void *arg)
 	merge_path_server_t *args = arg;
 	merge_path_t p_args = {
 		.paths = args->dst_paths,
+		.flags = args->flags,
 	};
 	data_t *url;
 
@@ -1125,7 +1297,8 @@ data_for_each_cmd_t _merge_path_server(data_t *data, void *arg)
 	if (data_convert_type(url, DATA_TYPE_STRING) != DATA_TYPE_STRING)
 		return DATA_FOR_EACH_FAIL;
 
-	p_args.server_path = data_get_string(url);
+	p_args.server_path = parse_url_path(data_get_string_const(url),
+					    false, false);
 
 	if (args->src_paths &&
 	    (data_dict_for_each(args->src_paths, _merge_path, &p_args) < 0))
@@ -1208,6 +1381,7 @@ extern int get_openapi_specification(openapi_t *oas, data_t *resp)
 				.dst_paths = paths,
 				.src_paths = data_key_get(oas->spec[i],
 							  "paths"),
+				.flags = oas->spec_flags[i],
 			};
 
 			if (data_list_for_each(src_srvs, _merge_path_server,
@@ -1217,8 +1391,9 @@ extern int get_openapi_specification(openapi_t *oas, data_t *resp)
 		} else {
 			/* servers is not populated, default to '/' */
 			merge_path_t p_args = {
-				.server_path = "/",
+				.server_path = NULL,
 				.paths = paths,
+				.flags = oas->spec_flags[i],
 			};
 			data_t *src_paths = data_key_get(oas->spec[i], "paths");
 
