@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  run_in_deamon.c - functions to determine if you are a given daemon or not
+ *  run_in_daemon.c - functions to determine if you are a given daemon or not
  *****************************************************************************
  *  Copyright (C) 2020 SchedMD LLC
  *  Written by Danny Auble <da@schedmd.com>
@@ -41,7 +41,7 @@
 #include "src/common/xstring.h"
 
 strong_alias(run_in_daemon, slurm_run_in_daemon);
-strong_alias(running_in_slurmctld, slurm_running_in_daemon);
+strong_alias(running_in_daemon, slurm_running_in_daemon);
 strong_alias(running_in_slurmctld, slurm_running_in_slurmctld);
 strong_alias(running_in_slurmd, slurm_running_in_slurmd);
 strong_alias(running_in_slurmdbd, slurm_running_in_slurmdbd);
@@ -93,12 +93,26 @@ extern bool running_in_daemon(void)
 			     "slurmctld,slurmd,slurmdbd,slurmstepd,slurmrestd");
 }
 
-extern bool running_in_slurmctld(void)
+static bool _running_in_slurmctld(bool reset)
 {
 	static bool run = false, set = false;
+
+	if (reset)
+		set = run = false;
+
 	return run_in_daemon(&run, &set, "slurmctld");
 }
 
+extern bool running_in_slurmctld(void)
+{
+	return _running_in_slurmctld(false);
+}
+
+extern bool running_in_slurmctld_reset(void)
+{
+	return _running_in_slurmctld(true);
+}
+
 extern bool running_in_slurmd(void)
 {
 	static bool run = false, set = false;
