@@ -55,16 +55,13 @@ typedef enum {
 	ACCT_STORAGE_INFO_AGENT_COUNT
 } acct_storage_info_t;
 
-extern int with_slurmdbd;
 extern uid_t db_api_uid;
 
-extern int slurm_acct_storage_init(char *loc); /* load the plugin */
+extern int slurm_acct_storage_init(void); /* load the plugin */
 extern int slurm_acct_storage_fini(void); /* unload the plugin */
 
 /*
  * get a new connection to the storage unit
- * IN: callbacks - Make an agent to manage queued requests, contains
- *                 trigger callbacks
  * IN: conn_num - If running more than one connection to the database
  *     this can be used to tell which connection is doing what
  * OUT: persist_conn_flags - If using a persistant connection to talk to the
@@ -73,7 +70,6 @@ extern int slurm_acct_storage_fini(void); /* unload the plugin */
  * RET: pointer used to access db
  */
 extern void *acct_storage_g_get_connection(
-	const slurm_trigger_callbacks_t *callbacks,
 	int conn_num, uint16_t *persist_conn_flags,
 	bool rollback, char *cluster_name);
 
@@ -583,11 +579,23 @@ extern int acct_storage_g_shutdown(void *db_conn);
 
 /*********************** CLUSTER ACCOUNTING STORAGE **************************/
 
+/*
+ * Send all relavant information to the DBD.
+ * RET: SLURM_SUCCESS on success SLURM_ERROR else
+ */
+extern void acct_storage_g_send_all(void *db_conn, time_t event_time,
+				    slurm_msg_type_t msg_type);
+
 extern int clusteracct_storage_g_node_down(void *db_conn,
 					   node_record_t *node_ptr,
 					   time_t event_time,
 					   char *reason, uint32_t reason_uid);
 
+/*
+ * See slurmdbd acct_storage_p_node_inx().
+ */
+extern char *acct_storage_g_node_inx(void *db_conn, char *nodes);
+
 extern int clusteracct_storage_g_node_up(void *db_conn, node_record_t *node_ptr,
 					 time_t event_time);
 
@@ -615,6 +623,12 @@ extern int jobacct_storage_job_start_direct(void *db_conn,
 extern int jobacct_storage_g_job_start(void *db_conn,
 				       job_record_t *job_ptr);
 
+/*
+ * load into the storage heavy information about a job,
+ * typically when it begins execution, but possibly earlier
+ */
+extern int jobacct_storage_g_job_heavy(void *db_conn, job_record_t *job_ptr);
+
 /*
  * load into the storage the end of a job
  */
