@@ -35,8 +35,8 @@
 #include "slurm/slurm_errno.h"
 
 #include "src/common/log.h"
-#include "src/common/node_select.h"
 #include "src/common/read_config.h"
+#include "src/common/select.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_step_layout.h"
 #include "src/common/slurmdb_defs.h"
@@ -131,12 +131,13 @@ slurm_step_layout_t *slurm_step_layout_create(
  * RET a pointer to an slurm_step_layout_t structure
  * NOTE: allocates memory that should be xfreed by caller
  */
-slurm_step_layout_t *fake_slurm_step_layout_create(
+extern slurm_step_layout_t *fake_slurm_step_layout_create(
 	const char *tlist,
 	uint16_t *cpus_per_node,
 	uint32_t *cpu_count_reps,
 	uint32_t node_cnt,
-	uint32_t task_cnt)
+	uint32_t task_cnt,
+	uint16_t protocol_version)
 {
 	uint32_t cpn = 1;
 	int cpu_cnt = 0, cpu_inx = 0, i, j;
@@ -152,6 +153,7 @@ slurm_step_layout_t *fake_slurm_step_layout_create(
 	step_layout = xmalloc(sizeof(slurm_step_layout_t));
 	step_layout->node_list = xstrdup(tlist);
 	step_layout->node_cnt = node_cnt;
+	step_layout->start_protocol_ver = protocol_version;
 	step_layout->tasks = xcalloc(node_cnt, sizeof(uint16_t));
 	step_layout->tids = xcalloc(node_cnt, sizeof(uint32_t *));
 
