@@ -178,11 +178,6 @@ extern void slurmdb_pack_job_cond(void *in, uint16_t protocol_version,
 				  buf_t *buffer);
 extern int slurmdb_unpack_job_cond(void **object, uint16_t protocol_version,
 				   buf_t *buffer);
-extern void slurmdb_pack_job_modify_cond(void *in, uint16_t protocol_version,
-					 buf_t *buffer);
-extern int slurmdb_unpack_job_modify_cond(void **object,
-					  uint16_t protocol_version,
-					  buf_t *buffer);
 extern void slurmdb_pack_job_rec(void *object, uint16_t protocol_version,
 				 buf_t *buffer);
 extern int slurmdb_unpack_job_rec(void **job, uint16_t protocol_version,
