@@ -129,6 +129,8 @@ extern slurmdbd_msg_type_t str_2_slurmdbd_msg_type(char *msg_type)
 		return DBD_JOB_COMPLETE;
 	} else if (!xstrcasecmp(msg_type, "Job Start")) {
 		return DBD_JOB_START;
+	} else if (!xstrcasecmp(msg_type, "Job Heavy")) {
+		return DBD_JOB_HEAVY;
 	} else if (!xstrcasecmp(msg_type, "ID RC")) {
 		return DBD_ID_RC;
 	} else if (!xstrcasecmp(msg_type, "Job Suspend")) {
@@ -468,6 +470,12 @@ extern char *slurmdbd_msg_type_2_str(slurmdbd_msg_type_t msg_type, int get_enum)
 		} else
 			return "Job Start";
 		break;
+	case DBD_JOB_HEAVY:
+		if (get_enum) {
+			return "DBD_JOB_HEAVY";
+		} else
+			return "Job Heavy";
+		break;
 	case DBD_ID_RC:
 		if (get_enum) {
 			return "DBD_ID_RC";
@@ -910,6 +918,9 @@ extern void slurmdbd_free_msg(persist_msg_t *msg)
 	case DBD_JOB_START:
 		slurmdbd_free_job_start_msg(msg->data);
 		break;
+	case DBD_JOB_HEAVY:
+		slurmdbd_free_job_heavy_msg(msg->data);
+		break;
 	case DBD_JOB_SUSPEND:
 		slurmdbd_free_job_suspend_msg(msg->data);
 		break;
@@ -1075,15 +1086,14 @@ extern void slurmdbd_free_job_start_msg(void *in)
 		xfree(msg->array_task_str);
 		xfree(msg->constraints);
 		xfree(msg->container);
-		xfree(msg->env);
+		xfree(msg->env_hash);
 		xfree(msg->gres_used);
 		xfree(msg->mcs_label);
 		xfree(msg->name);
 		xfree(msg->nodes);
 		xfree(msg->node_inx);
 		xfree(msg->partition);
-		xfree(msg->script);
-		FREE_NULL_BUFFER(msg->script_buf);
+		xfree(msg->script_hash);
 		xfree(msg->submit_line);
 		xfree(msg->tres_alloc_str);
 		xfree(msg->tres_req_str);
@@ -1093,6 +1103,21 @@ extern void slurmdbd_free_job_start_msg(void *in)
 	}
 }
 
+extern void slurmdbd_free_job_heavy_msg(void *in)
+{
+	dbd_job_heavy_msg_t *msg = in;
+
+	if (!msg)
+		return;
+
+	xfree(msg->env);
+	xfree(msg->env_hash);
+	xfree(msg->script);
+	FREE_NULL_BUFFER(msg->script_buf);
+	xfree(msg->script_hash);
+	xfree(msg);
+}
+
 extern void slurmdbd_free_id_rc_msg(void *in)
 {
 	dbd_id_rc_msg_t *msg = (dbd_id_rc_msg_t *)in;
