@@ -96,16 +96,6 @@ typedef struct slurm_switch_ops {
 					    char ***env, uint32_t nodeid,
 					    uint32_t procid, uint32_t nnodes,
 					    uint32_t nprocs, uint32_t rank);
-	int          (*clear_node)        ( void );
-	int          (*alloc_nodeinfo)    ( switch_node_info_t **nodeinfo );
-	int          (*build_nodeinfo)    ( switch_node_info_t *nodeinfo );
-	int          (*pack_nodeinfo)     ( switch_node_info_t *nodeinfo,
-					    buf_t *buffer,
-					    uint16_t protocol_version );
-	int          (*unpack_nodeinfo)   ( switch_node_info_t **nodeinfo,
-					    buf_t *buffer,
-					    uint16_t protocol_version );
-	int          (*free_nodeinfo)     ( switch_node_info_t **nodeinfo );
 	int          (*step_complete)     ( switch_jobinfo_t *jobinfo,
 					    char *nodelist );
 	int          (*step_allocated)    ( switch_jobinfo_t *jobinfo,
@@ -116,6 +106,7 @@ typedef struct slurm_switch_ops {
 	int          (*job_step_post_suspend)( stepd_step_rec_t *job );
 	int          (*job_step_pre_resume)( stepd_step_rec_t *job );
 	int          (*job_step_post_resume)( stepd_step_rec_t *job );
+	void         (*job_complete)      ( uint32_t job_id );
 } slurm_switch_ops_t;
 
 /*
@@ -145,12 +136,6 @@ static const char *syms[] = {
 	"switch_p_job_fini",
 	"switch_p_job_postfini",
 	"switch_p_job_attach",
-	"switch_p_clear_node_state",
-	"switch_p_alloc_node_info",
-	"switch_p_build_node_info",
-	"switch_p_pack_node_info",
-	"switch_p_unpack_node_info",
-	"switch_p_free_node_info",
 	"switch_p_job_step_complete",
 	"switch_p_job_step_allocated",
 	"switch_p_libstate_clear",
@@ -159,6 +144,7 @@ static const char *syms[] = {
 	"switch_p_job_step_post_suspend",
 	"switch_p_job_step_pre_resume",
 	"switch_p_job_step_post_resume",
+	"switch_p_job_complete",
 };
 
 static slurm_switch_ops_t  *ops            = NULL;
@@ -626,62 +612,6 @@ extern int switch_g_job_attach(dynamic_plugin_data_t *jobinfo, char ***env,
 		(data, env, nodeid, procid, nnodes, nprocs, gid);
 }
 
-/*
- * node switch state monitoring functions
- * required for IBM Federation switch
- */
-extern int switch_g_clear_node_state(void)
-{
-	if ( switch_init(0) < 0 )
-		return SLURM_ERROR;
-
-	return (*(ops[switch_context_default].clear_node))();
-}
-
-extern int switch_g_alloc_node_info(switch_node_info_t **switch_node)
-{
-	if ( switch_init(0) < 0 )
-		return SLURM_ERROR;
-
-	return (*(ops[switch_context_default].alloc_nodeinfo))( switch_node );
-}
-
-extern int switch_g_build_node_info(switch_node_info_t *switch_node)
-{
-	if ( switch_init(0) < 0 )
-		return SLURM_ERROR;
-
-	return (*(ops[switch_context_default].build_nodeinfo))( switch_node );
-}
-
-extern int switch_g_pack_node_info(switch_node_info_t *switch_node,
-				   buf_t *buffer, uint16_t protocol_version)
-{
-	if ( switch_init(0) < 0 )
-		return SLURM_ERROR;
-
-	return (*(ops[switch_context_default].pack_nodeinfo))
-		(switch_node, buffer, protocol_version);
-}
-
-extern int switch_g_unpack_node_info(switch_node_info_t **switch_node,
-				     buf_t *buffer, uint16_t protocol_version)
-{
-	if ( switch_init(0) < 0 )
-		return SLURM_ERROR;
-
-	return (*(ops[switch_context_default].unpack_nodeinfo))
-		(switch_node, buffer, protocol_version);
-}
-
-extern int switch_g_free_node_info(switch_node_info_t **switch_node)
-{
-	if ( switch_init(0) < 0 )
-		return SLURM_ERROR;
-
-	return (*(ops[switch_context_default].free_nodeinfo))( switch_node );
-}
-
 extern int switch_g_job_step_complete(dynamic_plugin_data_t *jobinfo,
 				      char *nodelist)
 {
@@ -749,3 +679,11 @@ extern int switch_g_job_step_post_resume(stepd_step_rec_t *job)
 
 	return (*(ops[switch_context_default].job_step_post_resume)) ( job );
 }
+
+extern void switch_g_job_complete(uint32_t job_id)
+{
+	if (switch_init(0) < 0)
+		return;
+
+	(*(ops[switch_context_default].job_complete))(job_id);
+}
