@@ -397,9 +397,10 @@ extern int cluster_first_reg(char *host, uint16_t port, uint16_t rpc_version)
 		out_msg.msg_type = ACCOUNTING_FIRST_REG;
 		out_msg.flags = SLURM_GLOBAL_AUTH_KEY;
 		out_msg.data = &update;
+		slurm_msg_set_r_uid(&out_msg, SLURM_AUTH_UID_ANY);
 		slurm_send_node_msg(fd, &out_msg);
 		/* We probably need to add matching recv_msg function
-		 * for an arbitray fd or should these be fire
+		 * for an arbitrary fd or should these be fire
 		 * and forget?  For this, that we can probably
 		 * forget about it */
 		close(fd);
@@ -910,19 +911,7 @@ extern int archive_write_file(buf_t *buffer, char *cluster_name,
 		error("Can't save archive, create file %s error %m", new_file);
 		rc = SLURM_ERROR;
 	} else {
-		int amount;
-		uint32_t pos = 0, nwrite = get_buf_offset(buffer);
-		char *data = (char *)get_buf_data(buffer);
-		while (nwrite > 0) {
-			amount = write(fd, &data[pos], nwrite);
-			if ((amount < 0) && (errno != EINTR)) {
-				error("Error writing file %s, %m", new_file);
-				rc = SLURM_ERROR;
-				break;
-			}
-			nwrite -= amount;
-			pos    += amount;
-		}
+		safe_write(fd, get_buf_data(buffer), get_buf_offset(buffer));
 		fsync(fd);
 		close(fd);
 	}
@@ -931,4 +920,12 @@ extern int archive_write_file(buf_t *buffer, char *cluster_name,
 	slurm_mutex_unlock(&local_file_lock);
 
 	return rc;
+
+rwfail:
+	error("Error writing file %s, %m", new_file);
+	close(fd);
+	xfree(new_file);
+	slurm_mutex_unlock(&local_file_lock);
+
+	return SLURM_ERROR;
 }
