@@ -42,7 +42,9 @@
 #include "as_mysql_federation.h"
 #include "as_mysql_usage.h"
 #include "as_mysql_wckey.h"
-#include "src/common/node_select.h"
+
+#include "src/common/select.h"
+#include "src/common/slurm_time.h"
 
 extern int as_mysql_get_fed_cluster_id(mysql_conn_t *mysql_conn,
 				       const char *cluster,
@@ -819,7 +821,8 @@ extern List as_mysql_remove_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 			   object, cluster_month_table, now);
 		rc = remove_common(mysql_conn, DBD_REMOVE_CLUSTERS, now,
 				   user_name, cluster_table, name_char,
-				   assoc_char, object, ret_list, &jobs_running);
+				   assoc_char, object, ret_list, &jobs_running,
+				   NULL);
 		xfree(object);
 		if (rc != SLURM_SUCCESS)
 			break;
@@ -1147,12 +1150,17 @@ extern List as_mysql_get_cluster_events(mysql_conn_t *mysql_conn, uint32_t uid,
 		else
 			xstrcat(extra, " where (");
 
+		xstrfmtcat(extra,
+			   "(CONVERT(SUBSTRING_INDEX(tres,'%d=',-1),"
+			   "UNSIGNED INTEGER)",
+			   TRES_CPU);
+
 		if (event_cond->cpus_max) {
-			xstrfmtcat(extra, "count between %u and %u)",
+			xstrfmtcat(extra, " between %u and %u))",
 				   event_cond->cpus_min, event_cond->cpus_max);
 
 		} else {
-			xstrfmtcat(extra, "count='%u')",
+			xstrfmtcat(extra, "='%u'))",
 				   event_cond->cpus_min);
 
 		}
@@ -1223,10 +1231,24 @@ extern List as_mysql_get_cluster_events(mysql_conn_t *mysql_conn, uint32_t uid,
 		else
 			xstrcat(extra, " where (");
 
-		xstrfmtcat(extra,
-			   "(time_start < %ld) "
-			   "&& (time_end >= %ld || time_end = 0))",
-			   event_cond->period_end, event_cond->period_start);
+		if (event_cond->cond_flags & SLURMDB_EVENT_COND_OPEN)
+			xstrfmtcat(extra,
+				   "(time_start >= %ld) && (time_end = 0))",
+				   event_cond->period_start);
+		else
+			xstrfmtcat(extra,
+				   "(time_start < %ld) "
+				   "&& (time_end >= %ld || time_end = 0))",
+				   event_cond->period_end,
+				   event_cond->period_start);
+
+	} else if (event_cond->cond_flags & SLURMDB_EVENT_COND_OPEN) {
+		if (extra)
+			xstrcat(extra, " && (");
+		else
+			xstrcat(extra, " where (");
+
+		xstrfmtcat(extra, "time_end = 0)");
 	}
 
 	if (event_cond->reason_list
@@ -1293,7 +1315,13 @@ empty:
 	xfree(tmp);
 	xstrfmtcat(tmp, "%s", event_req_inx[0]);
 	for(i=1; i<EVENT_REQ_COUNT; i++) {
-		xstrfmtcat(tmp, ", %s", event_req_inx[i]);
+		bool include = true;
+		if (event_cond && event_cond->format_list)
+			include = list_find_first(event_cond->format_list,
+						  slurm_find_char_in_list,
+						  event_req_inx[i]);
+		xstrfmtcat(tmp, ", %s%s",
+			   include ? "" : "'' as ", event_req_inx[i]);
 	}
 
 	if (event_cond && event_cond->cluster_list &&
@@ -1530,6 +1558,7 @@ extern int as_mysql_fini_ctld(mysql_conn_t *mysql_conn,
 	time_t now = time(NULL);
 	char *query = NULL;
 	bool free_it = false;
+	bool affected_rows = false;
 
 	if (check_connection(mysql_conn) != SLURM_SUCCESS)
 		return ESLURM_DB_CONNECTION;
@@ -1552,7 +1581,11 @@ extern int as_mysql_fini_ctld(mysql_conn_t *mysql_conn,
 	if (rc != SLURM_SUCCESS)
 		return SLURM_ERROR;
 
-	if (!last_affected_rows(mysql_conn) || !slurmdbd_conf->track_ctld ||
+	affected_rows = last_affected_rows(mysql_conn);
+	if (affected_rows)
+		as_mysql_add_feds_to_update_list(mysql_conn);
+
+	if (!affected_rows || !slurmdbd_conf->track_ctld ||
 	    (cluster_rec->flags & CLUSTER_FLAG_EXT))
 		return rc;
 
@@ -1703,13 +1736,18 @@ add_it:
 	query = xstrdup_printf(
 		"insert into \"%s_%s\" (cluster_nodes, tres, "
 		"time_start, reason) "
-		"values ('%s', '%s', %ld, 'Cluster Registered TRES');",
+		"values ('%s', '%s', %ld, 'Cluster Registered TRES') "
+		"on duplicate key update time_end=0, tres=VALUES(tres);",
 		mysql_conn->cluster_name, event_table,
 		cluster_nodes, *tres_str_in, event_time);
 
 	rc = mysql_db_query(mysql_conn, query);
 	xfree(query);
 
+	if (trigger_reroll(mysql_conn, event_time))
+		debug("Need to reroll usage from %s, cluster %s changes happened before last rollup.",
+		      slurm_ctime2(&event_time), mysql_conn->cluster_name);
+
 	if (rc != SLURM_SUCCESS)
 		goto end_it;
 
