@@ -43,8 +43,8 @@
 
 #include "src/common/assoc_mgr.h"
 #include "src/common/gres.h"
-#include "src/common/node_select.h"
 #include "src/common/parse_time.h"
+#include "src/common/select.h"
 #include "src/common/slurm_jobacct_gather.h"
 #include "src/common/slurm_time.h"
 
@@ -278,13 +278,82 @@ no_wckeyid:
 	return wckeyid;
 }
 
+
+static uint64_t _get_hash_inx(mysql_conn_t *mysql_conn,
+			      job_record_t *job_ptr,
+			      uint64_t flag)
+{
+	char *query, *hash;
+	char *hash_col = NULL, *type_col = NULL, *type_table = NULL;
+	MYSQL_RES *result = NULL;
+	MYSQL_ROW row;
+	uint64_t hash_inx = 0;
+
+	switch (flag) {
+	case JOB_SEND_ENV:
+		hash_col = "env_hash";
+		type_col = "env_vars";
+		type_table = job_env_table;
+		hash = job_ptr->details->env_hash;
+		break;
+	case JOB_SEND_SCRIPT:
+		hash_col = "script_hash";
+		type_col = "batch_script";
+		type_table = job_script_table;
+		hash = job_ptr->details->script_hash;
+		break;
+	default:
+		error("unknown hash type bit %"PRIu64, flag);
+		return NO_VAL64;
+		break;
+	}
+
+	if (!hash)
+		return 0;
+
+	query = xstrdup_printf(
+		"select hash_inx from \"%s_%s\" where %s = '%s';",
+		mysql_conn->cluster_name, type_table,
+		hash_col, hash);
+
+	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
+		xfree(query);
+		return NO_VAL64;
+	}
+
+	xfree(query);
+
+	if ((row = mysql_fetch_row(result))) {
+		debug3("%u has an %s we have already seen, no need to add again",
+		       job_ptr->job_id, type_col);
+		hash_inx = slurm_atoull(row[0]);
+	} else {
+		query = xstrdup_printf(
+			"insert into \"%s_%s\" (%s) values ('%s') "
+			"on duplicate key update last_used=VALUES(last_used), "
+			"hash_inx=LAST_INSERT_ID(hash_inx);",
+			mysql_conn->cluster_name, type_table,
+			hash_col, hash);
+
+		hash_inx = mysql_db_insert_ret_id(mysql_conn, query);
+		if (!hash_inx)
+			hash_inx = NO_VAL64;
+		else
+			job_ptr->bit_flags |= flag;
+		xfree(query);
+	}
+	mysql_free_result(result);
+
+	return hash_inx;
+
+}
+
 /* extern functions */
 
 extern int as_mysql_job_start(mysql_conn_t *mysql_conn, job_record_t *job_ptr)
 {
 	int rc = SLURM_SUCCESS;
 	char *nodes = NULL, *jname = NULL;
-	int track_steps = 0;
 	char *partition = NULL;
 	char *query = NULL;
 	int reinit = 0;
@@ -293,9 +362,11 @@ extern int as_mysql_job_start(mysql_conn_t *mysql_conn, job_record_t *job_ptr)
 	uint32_t job_state;
 	uint32_t array_task_id =
 		(job_ptr->array_job_id) ? job_ptr->array_task_id : NO_VAL;
+	uint32_t het_job_offset =
+		(job_ptr->het_job_id) ? job_ptr->het_job_offset : NO_VAL;
 	uint64_t job_db_inx = job_ptr->db_index;
 	job_array_struct_t *array_recs = job_ptr->array_recs;
-	char *tres_alloc_str = NULL;
+	MYSQL_RES *result = NULL;
 
 	if ((!job_ptr->details || !job_ptr->details->submit_time)
 	    && !job_ptr->resize_time) {
@@ -361,7 +432,6 @@ extern int as_mysql_job_start(mysql_conn_t *mysql_conn, job_record_t *job_ptr)
 
 	slurm_mutex_lock(&rollup_lock);
 	if (check_time < global_last_rollup) {
-		MYSQL_RES *result = NULL;
 		MYSQL_ROW row;
 
 		/* check to see if we are hearing about this time for the
@@ -433,23 +503,16 @@ extern int as_mysql_job_start(mysql_conn_t *mysql_conn, job_record_t *job_ptr)
 
 no_rollup_change:
 
-	if (job_ptr->name && job_ptr->name[0]) {
+	if (job_ptr->name && job_ptr->name[0])
 		jname = job_ptr->name;
-		if (!xstrcmp(jname, "interactive"))
-			track_steps = 1;
-	} else {
+	else
 		jname = "allocation";
-		track_steps = 1;
-	}
 
 	if (job_ptr->nodes && job_ptr->nodes[0])
 		nodes = job_ptr->nodes;
 	else
 		nodes = "None assigned";
 
-	if (job_ptr->batch_flag)
-		track_steps = 1;
-
 	/* Grab the wckey once to make sure it is placed. */
 	if (job_ptr->assoc_id && (!job_ptr->db_index || job_ptr->wckey))
 		wckeyid = _get_wckeyid(mysql_conn, &job_ptr->wckey,
@@ -462,7 +525,39 @@ no_rollup_change:
 	else if (job_ptr->partition)
 		partition = job_ptr->partition;
 
+	/* Mark the database so we know we have recieved the start record. */
+	job_ptr->db_flags |= SLURMDB_JOB_FLAG_START_R;
+
 	if (!job_ptr->db_index) {
+		uint64_t env_hash_inx = 0, script_hash_inx = 0;
+		/*
+		 * Here we check to see if the env has been added to the
+		 * database or not to inform the slurmctld to send it.
+		 * This only happens if !db_index no need to do this on an
+		 * update.
+		 */
+		if (job_ptr->details->env_hash) {
+			env_hash_inx = _get_hash_inx(
+				mysql_conn, job_ptr, JOB_SEND_ENV);
+
+			if (env_hash_inx == NO_VAL64)
+				return SLURM_ERROR;
+		}
+
+		/*
+		 * Here we check to see if the script has been added to the
+		 * database or not to inform the slurmctld to send it.
+		 * This only happens if !db_index no need to do this on an
+		 * update.
+		 */
+		if (job_ptr->details->script_hash) {
+			script_hash_inx = _get_hash_inx(
+				mysql_conn, job_ptr, JOB_SEND_SCRIPT);
+
+			if (script_hash_inx == NO_VAL64)
+				return SLURM_ERROR;
+		}
+
 		query = xstrdup_printf(
 			"insert into \"%s_%s\" "
 			"(id_job, mod_time, id_array_job, id_array_task, "
@@ -470,8 +565,9 @@ no_rollup_change:
 			"id_assoc, id_qos, id_user, "
 			"id_group, nodelist, id_resv, timelimit, "
 			"time_eligible, time_submit, time_start, "
-			"job_name, track_steps, state, priority, cpus_req, "
-			"nodes_alloc, mem_req, flags, state_reason_prev",
+			"job_name, state, priority, cpus_req, "
+			"nodes_alloc, mem_req, flags, state_reason_prev, "
+			"env_hash_inx, script_hash_inx",
 			mysql_conn->cluster_name, job_table);
 
 		if (wckeyid)
@@ -492,7 +588,7 @@ no_rollup_change:
 		else
 			xstrcat(query, ", array_task_str, array_task_pending");
 
-		if (job_ptr->tres_alloc_str || tres_alloc_str)
+		if (job_ptr->tres_alloc_str)
 			xstrcat(query, ", tres_alloc");
 		if (job_ptr->tres_req_str)
 			xstrcat(query, ", tres_req");
@@ -500,10 +596,6 @@ no_rollup_change:
 			xstrcat(query, ", work_dir");
 		if (job_ptr->details->features)
 			xstrcat(query, ", constraints");
-		if (job_ptr->details->script)
-			xstrcat(query, ", batch_script");
-		if (job_ptr->details->env_sup)
-			xstrcat(query, ", env_vars");
 		if (job_ptr->details->submit_line)
 			xstrcat(query, ", submit_line");
 		if (job_ptr->container)
@@ -513,20 +605,22 @@ no_rollup_change:
 			   ") values (%u, UNIX_TIMESTAMP(), "
 			   "%u, %u, %u, %u, %u, %u, %u, %u, "
 			   "'%s', %u, %u, %ld, %ld, %ld, "
-			   "'%s', %u, %u, %u, %u, %u, %"PRIu64", %u, %u",
+			   "'%s', %u, %u, %u, %u, %"PRIu64", %u, %u, "
+			   "%"PRIu64", %"PRIu64,
 			   job_ptr->job_id,
 			   job_ptr->array_job_id, array_task_id,
-			   job_ptr->het_job_id, job_ptr->het_job_offset,
+			   job_ptr->het_job_id, het_job_offset,
 			   job_ptr->assoc_id, job_ptr->qos_id,
 			   job_ptr->user_id, job_ptr->group_id, nodes,
 			   job_ptr->resv_id, job_ptr->time_limit,
 			   begin_time, submit_time, start_time,
-			   jname, track_steps, job_state,
+			   jname, job_state,
 			   job_ptr->priority, job_ptr->details->min_cpus,
 			   job_ptr->total_nodes,
 			   job_ptr->details->pn_min_memory,
 			   job_ptr->db_flags,
-			   job_ptr->state_reason_prev_db);
+			   job_ptr->state_reason_prev_db,
+			   env_hash_inx, script_hash_inx);
 
 		if (wckeyid)
 			xstrfmtcat(query, ", %u", wckeyid);
@@ -548,9 +642,7 @@ no_rollup_change:
 		else
 			xstrcat(query, ", NULL, 0");
 
-		if (tres_alloc_str)
-			xstrfmtcat(query, ", '%s'", tres_alloc_str);
-		else if (job_ptr->tres_alloc_str)
+		if (job_ptr->tres_alloc_str)
 			xstrfmtcat(query, ", '%s'", job_ptr->tres_alloc_str);
 		if (job_ptr->tres_req_str)
 			xstrfmtcat(query, ", '%s'", job_ptr->tres_req_str);
@@ -560,12 +652,6 @@ no_rollup_change:
 		if (job_ptr->details->features)
 			xstrfmtcat(query, ", '%s'",
 				   job_ptr->details->features);
-		if (job_ptr->details->script)
-			xstrfmtcat(query, ", '%s'",
-				   job_ptr->details->script);
-		if (job_ptr->details->env_sup)
-			xstrfmtcat(query, ", '%s'",
-				   job_ptr->details->env_sup[0]);
 		if (job_ptr->details->submit_line)
 			xstrfmtcat(query, ", '%s'",
 				   job_ptr->details->submit_line);
@@ -580,24 +666,26 @@ no_rollup_change:
 			   "nodelist='%s', id_resv=%u, timelimit=%u, "
 			   "time_submit=%ld, time_eligible=%ld, "
 			   "time_start=%ld, mod_time=UNIX_TIMESTAMP(), "
-			   "job_name='%s', track_steps=%u, id_qos=%u, "
+			   "job_name='%s', id_qos=%u, "
 			   "state=greatest(state, %u), priority=%u, "
 			   "cpus_req=%u, nodes_alloc=%u, "
 			   "mem_req=%"PRIu64", id_array_job=%u, id_array_task=%u, "
 			   "het_job_id=%u, het_job_offset=%u, flags=%u, "
-			   "state_reason_prev=%u",
+			   "state_reason_prev=%u, env_hash_inx=%"PRIu64
+			   ", script_hash_inx=%"PRIu64,
 			   job_ptr->assoc_id, job_ptr->user_id,
 			   job_ptr->group_id, nodes,
 			   job_ptr->resv_id, job_ptr->time_limit,
 			   submit_time, begin_time, start_time,
-			   jname, track_steps, job_ptr->qos_id, job_state,
+			   jname, job_ptr->qos_id, job_state,
 			   job_ptr->priority, job_ptr->details->min_cpus,
 			   job_ptr->total_nodes,
 			   job_ptr->details->pn_min_memory,
 			   job_ptr->array_job_id, array_task_id,
-			   job_ptr->het_job_id, job_ptr->het_job_offset,
+			   job_ptr->het_job_id, het_job_offset,
 			   job_ptr->db_flags,
-			   job_ptr->state_reason_prev_db);
+			   job_ptr->state_reason_prev_db,
+			   env_hash_inx, script_hash_inx);
 
 		if (wckeyid)
 			xstrfmtcat(query, ", id_wckey=%u", wckeyid);
@@ -622,9 +710,7 @@ no_rollup_change:
 			xstrfmtcat(query, ", array_task_str=NULL, "
 				   "array_task_pending=0");
 
-		if (tres_alloc_str)
-			xstrfmtcat(query, ", tres_alloc='%s'", tres_alloc_str);
-		else if (job_ptr->tres_alloc_str)
+		if (job_ptr->tres_alloc_str)
 			xstrfmtcat(query, ", tres_alloc='%s'",
 				   job_ptr->tres_alloc_str);
 		if (job_ptr->tres_req_str)
@@ -637,14 +723,6 @@ no_rollup_change:
 			xstrfmtcat(query, ", constraints='%s'",
 				   job_ptr->details->features);
 
-		if (job_ptr->details->script)
-			xstrfmtcat(query, ", batch_script='%s'",
-				   job_ptr->details->script);
-
-		if (job_ptr->details->env_sup)
-			xstrfmtcat(query, ", env_vars='%s'",
-				   job_ptr->details->env_sup[0]);
-
 		if (job_ptr->details->submit_line)
 			xstrfmtcat(query, ", submit_line='%s'",
 				   job_ptr->details->submit_line);
@@ -695,9 +773,7 @@ no_rollup_change:
 			xstrfmtcat(query, "array_task_str=NULL, "
 				   "array_task_pending=0, ");
 
-		if (tres_alloc_str)
-			xstrfmtcat(query, "tres_alloc='%s', ", tres_alloc_str);
-		else if (job_ptr->tres_alloc_str)
+		if (job_ptr->tres_alloc_str)
 			xstrfmtcat(query, "tres_alloc='%s', ",
 				   job_ptr->tres_alloc_str);
 		if (job_ptr->tres_req_str)
@@ -710,14 +786,6 @@ no_rollup_change:
 			xstrfmtcat(query, "constraints='%s', ",
 				   job_ptr->details->features);
 
-		if (job_ptr->details->script)
-			xstrfmtcat(query, "batch_script='%s', ",
-				   job_ptr->details->script);
-
-		if (job_ptr->details->env_sup)
-			xstrfmtcat(query, "env_vars='%s', ",
-				   job_ptr->details->env_sup[0]);
-
 		if (job_ptr->details->submit_line)
 			xstrfmtcat(query, "submit_line='%s', ",
 				   job_ptr->details->submit_line);
@@ -741,7 +809,7 @@ no_rollup_change:
 			   job_ptr->resv_id, job_ptr->time_limit,
 			   job_ptr->details->pn_min_memory,
 			   job_ptr->array_job_id, array_task_id,
-			   job_ptr->het_job_id, job_ptr->het_job_offset,
+			   job_ptr->het_job_id, het_job_offset,
 			   job_ptr->db_flags, job_ptr->state_reason_prev_db,
 			   begin_time, job_ptr->db_index);
 
@@ -749,6 +817,11 @@ no_rollup_change:
 		rc = mysql_db_query(mysql_conn, query);
 	}
 
+	xfree(query);
+
+	if (rc != SLURM_SUCCESS)
+		return rc;
+
 	/* now we will reset all the steps */
 	if (IS_JOB_RESIZING(job_ptr)) {
 		/* FIXME : Verify this is still needed */
@@ -756,7 +829,47 @@ no_rollup_change:
 			as_mysql_suspend(mysql_conn, job_db_inx, job_ptr);
 	}
 
-	xfree(tres_alloc_str);
+	xfree(query);
+
+	return rc;
+}
+
+extern int as_mysql_job_heavy(mysql_conn_t *mysql_conn, job_record_t *job_ptr)
+{
+	char *query = NULL, *pos = NULL;
+	int rc = SLURM_SUCCESS;
+	struct job_details *details = job_ptr->details;
+
+	if (check_connection(mysql_conn) != SLURM_SUCCESS)
+		return ESLURM_DB_CONNECTION;
+
+	xassert(details);
+
+	debug2("%s() called", __func__);
+
+	/*
+	 * make sure we handle any quotes that may be in the comment
+	 */
+	if (details->env_hash && details->env_sup && details->env_sup[0])
+		xstrfmtcatat(
+			query, &pos,
+			"update \"%s_%s\" set env_vars = '%s' "
+			"where env_hash='%s';",
+			mysql_conn->cluster_name, job_env_table,
+			details->env_sup[0], details->env_hash);
+	if (details->script_hash && details->script)
+		xstrfmtcatat(
+			query, &pos,
+			"update \"%s_%s\" set batch_script = '%s' "
+			"where script_hash='%s';",
+			mysql_conn->cluster_name, job_script_table,
+			details->script, details->script_hash);
+
+	if (!query)
+		return rc;
+
+	DB_DEBUG(DB_JOB, mysql_conn->conn, "query\n%s", query);
+	rc = mysql_db_query(mysql_conn, query);
 	xfree(query);
 
 	return rc;
@@ -777,6 +890,10 @@ extern List as_mysql_modify_job(mysql_conn_t *mysql_conn, uint32_t uid,
 	ListIterator itr;
 	List id_switch_list = NULL;
 	id_switch_t *id_switch;
+	bool is_admin;
+
+	is_admin = is_user_min_admin_level(mysql_conn, uid,
+					   SLURMDB_ADMIN_OPERATOR);
 
 	if (!job_cond || !job) {
 		error("we need something to change");
@@ -784,15 +901,22 @@ extern List as_mysql_modify_job(mysql_conn_t *mysql_conn, uint32_t uid,
 	} else if (check_connection(mysql_conn) != SLURM_SUCCESS)
 		return NULL;
 
+	if (!is_admin && (job->admin_comment || job->system_comment)) {
+		errno = ESLURM_ACCESS_DENIED;
+		return NULL;
+	}
+
 	if (job->derived_ec != NO_VAL)
 		xstrfmtcat(vals, ", derived_ec=%u", job->derived_ec);
 
 	if (job->derived_es)
 		xstrfmtcat(vals, ", derived_es='%s'", job->derived_es);
 
+	if (job->admin_comment)
+		xstrfmtcat(vals, ", admin_comment='%s'", job->admin_comment);
+
 	if (job->system_comment)
-		xstrfmtcat(vals, ", system_comment='%s'",
-			   job->system_comment);
+		xstrfmtcat(vals, ", system_comment='%s'", job->system_comment);
 
 	if (job->wckey)
 		xstrfmtcat(vals, ", wckey='%s'", job->wckey);
@@ -823,9 +947,7 @@ extern List as_mysql_modify_job(mysql_conn_t *mysql_conn, uint32_t uid,
 		char tmp_char[25];
 		char *vals_mod = NULL;
 
-		if ((uid != job_rec->uid) &&
-		    !is_user_min_admin_level(mysql_conn, uid,
-					     SLURMDB_ADMIN_OPERATOR)) {
+		if ((uid != job_rec->uid) && !is_admin) {
 			errno = ESLURM_ACCESS_DENIED;
 			rc = SLURM_ERROR;
 			break;
@@ -1010,7 +1132,6 @@ extern int as_mysql_job_complete(mysql_conn_t *mysql_conn,
 	int rc = SLURM_SUCCESS, job_state;
 	time_t submit_time, end_time;
 	uint32_t exit_code = 0;
-	char *tres_alloc_str = NULL;
 
 	if (!job_ptr->db_index
 	    && ((!job_ptr->details || !job_ptr->details->submit_time)
@@ -1059,26 +1180,11 @@ extern int as_mysql_job_complete(mysql_conn_t *mysql_conn,
 			job_state = job_ptr->job_state & JOB_STATE_BASE;
 	}
 
-	slurm_mutex_lock(&rollup_lock);
-	if (end_time < global_last_rollup) {
+	if (trigger_reroll(mysql_conn, end_time))
 		debug("Need to reroll usage from %s Job %u from %s %s then and we are just now hearing about it.",
 		      slurm_ctime2(&end_time),
 		      job_ptr->job_id, mysql_conn->cluster_name,
 		      IS_JOB_RESIZING(job_ptr) ? "resized" : "ended");
-		global_last_rollup = end_time;
-		slurm_mutex_unlock(&rollup_lock);
-
-		query = xstrdup_printf("update \"%s_%s\" set "
-				       "hourly_rollup=%ld, "
-				       "daily_rollup=%ld, monthly_rollup=%ld",
-				       mysql_conn->cluster_name,
-				       last_ran_table, end_time,
-				       end_time, end_time);
-		DB_DEBUG(DB_JOB, mysql_conn->conn, "query\n%s", query);
-		(void) mysql_db_query(mysql_conn, query);
-		xfree(query);
-	} else
-		slurm_mutex_unlock(&rollup_lock);
 
 	if (!job_ptr->db_index) {
 		if (!(job_ptr->db_index =
@@ -1118,9 +1224,7 @@ extern int as_mysql_job_complete(mysql_conn_t *mysql_conn,
 	if (job_ptr->derived_ec != NO_VAL)
 		xstrfmtcat(query, ", derived_ec=%u", job_ptr->derived_ec);
 
-	if (tres_alloc_str)
-		xstrfmtcat(query, ", tres_alloc='%s'", tres_alloc_str);
-	else if (job_ptr->tres_alloc_str)
+	if (job_ptr->tres_alloc_str)
 		xstrfmtcat(query, ", tres_alloc='%s'", job_ptr->tres_alloc_str);
 
 	if (job_ptr->comment)
@@ -1141,17 +1245,19 @@ extern int as_mysql_job_complete(mysql_conn_t *mysql_conn,
 		 */
 		exit_code = 256;
 	}
+	xstrfmtcat(query, ", exit_code=%d, ", exit_code);
 
-	xstrfmtcat(query,
-		   ", exit_code=%d, kill_requid=%d where job_db_inx=%"PRIu64";",
-		   exit_code, job_ptr->requid,
-		   job_ptr->db_index);
+	if (job_ptr->requid == (uid_t) -1)
+		xstrfmtcat(query, "kill_requid=null ");
+	else
+		xstrfmtcat(query, "kill_requid=%u ", job_ptr->requid);
+
+	xstrfmtcat(query, "where job_db_inx=%"PRIu64";", job_ptr->db_index);
 
 	DB_DEBUG(DB_JOB, mysql_conn->conn, "query\n%s", query);
 	rc = mysql_db_query(mysql_conn, query);
 	xfree(query);
 
-	xfree(tres_alloc_str);
 	return rc;
 }
 
@@ -1415,13 +1521,16 @@ extern int as_mysql_step_complete(mysql_conn_t *mysql_conn,
 
 	/* The stepid could be negative so use %d not %u */
 	query = xstrdup_printf(
-		"update \"%s_%s\" set time_end=%d, state=%u, "
-		"kill_requid=%d, exit_code=%d",
+		"update \"%s_%s\" set time_end=%d, state=%u, exit_code=%d, ",
 		mysql_conn->cluster_name, step_table, (int)now,
 		comp_status,
-		step_ptr->requid,
 		exit_code);
 
+	if (step_ptr->requid == (uid_t) -1)
+		xstrfmtcat(query, "kill_requid=null");
+	else
+		xstrfmtcat(query, "kill_requid=%u", step_ptr->requid);
+
 
 	if (jobacct) {
 		slurmdb_stats_t stats;
