@@ -45,15 +45,10 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-#if HAVE_JSON_C_INC
-#  include <json-c/json.h>
-#elif HAVE_JSON_INC
-#  include <json/json.h>
-#endif
-
 #include "slurm/slurm.h"
 
 #include "src/common/assoc_mgr.h"
+#include "src/common/data.h"
 #include "src/common/fd.h"
 #include "src/common/run_command.h"
 #include "src/common/xsignal.h"
@@ -144,6 +139,12 @@ typedef struct bb_pools {
 	uint64_t free;
 } bb_pools_t;
 
+typedef struct {
+	int i;
+	int num_pools;
+	bb_pools_t *pools;
+} data_pools_arg_t;
+
 typedef struct {
 	bool hurry;
 	uint32_t job_id;
@@ -535,9 +536,9 @@ static void _recover_bb_state(void)
 	safe_unpack16(&protocol_version, buffer);
 	if (protocol_version == NO_VAL16) {
 		if (!ignore_state_errors)
-			fatal("Can not recover burst_buffer/datawarp state, data version incompatible, start with '-i' to ignore this. Warning: using -i will lose the data that can't be recovered.");
+			fatal("Can not recover burst_buffer/lua state, data version incompatible, start with '-i' to ignore this. Warning: using -i will lose the data that can't be recovered.");
 		error("**********************************************************************");
-		error("Can not recover burst_buffer/datawarp state, data version incompatible");
+		error("Can not recover burst_buffer/lua state, data version incompatible");
 		error("**********************************************************************");
 		return;
 	}
@@ -660,62 +661,101 @@ static void _bb_free_pools(bb_pools_t *pools, int num_ent)
 	xfree(pools);
 }
 
-static void _json_parse_pools_object(json_object *jobj, bb_pools_t *pools)
+static int _data_get_val_from_key(data_t *data, char *key, data_type_t type,
+				  bool required, void *out_val)
 {
-	enum json_type type;
-	struct json_object_iter iter;
-	int64_t x;
-	const char *p;
+	int rc = SLURM_SUCCESS;
+	data_t *data_tmp = NULL;
+	char **val_str;
+	int64_t *val_int;
 
-	json_object_object_foreachC(jobj, iter) {
-		type = json_object_get_type(iter.val);
-		switch (type) {
-		case json_type_int:
-			x = json_object_get_int64(iter.val);
-			if (xstrcmp(iter.key, "granularity") == 0) {
-				pools->granularity = x;
-			} else if (xstrcmp(iter.key, "quantity") == 0) {
-				pools->quantity = x;
-			} else if (xstrcmp(iter.key, "free") == 0) {
-				pools->free = x;
-			}
-			break;
-		case json_type_string:
-			p = json_object_get_string(iter.val);
-			if (xstrcmp(iter.key, "id") == 0) {
-				pools->name = xstrdup(p);
-			}
-			break;
-		default:
-			break;
-		}
+	data_tmp = data_key_get(data, key);
+	if (!data_tmp) {
+		if (required)
+			return SLURM_ERROR;
+		return SLURM_SUCCESS; /* Not specified */
 	}
+
+	if (data_get_type(data_tmp) != type) {
+		error("%s: %s is the wrong data type", __func__, key);
+		return SLURM_ERROR;
+	}
+
+	switch (type) {
+	case DATA_TYPE_STRING:
+		val_str = out_val;
+		*val_str = xstrdup(data_get_string(data_tmp));
+		break;
+	case DATA_TYPE_INT_64:
+		val_int = out_val;
+		*val_int = data_get_int(data_tmp);
+		break;
+	default:
+		rc = SLURM_ERROR;
+		break;
+	}
+
+	return rc;
 }
 
-static bb_pools_t * _json_parse_pools_array(json_object *jobj, char *key,
-					    int *num)
+/*
+ * IN data - A dictionary describing a pool.
+ * IN/OUT arg - pointer to data_pools_arg_t. This function populates a pool
+ *              in arg->pools.
+ *
+ * RET data_for_each_cmd_t
+ */
+static data_for_each_cmd_t _foreach_parse_pool(data_t *data, void *arg)
 {
-	json_object *jarray;
-	int i;
-	json_object *jvalue;
-	bb_pools_t *pools;
+	data_pools_arg_t *data_arg = arg;
+	data_for_each_cmd_t rc = DATA_FOR_EACH_CONT;
+	bb_pools_t *pools = data_arg->pools;
+	int i = data_arg->i;
 
-	jarray = jobj;
-	json_object_object_get_ex(jobj, key, &jarray);
-
-	*num = json_object_array_length(jarray);
-	pools = xcalloc(*num, sizeof(bb_pools_t));
-
-	for (i = 0; i < *num; i++) {
-		pools[i].free = NO_VAL64;
-		pools[i].granularity = NO_VAL64;
-		pools[i].quantity = NO_VAL64;
-
-		jvalue = json_object_array_get_idx(jarray, i);
-		_json_parse_pools_object(jvalue, &pools[i]);
+	if (i > data_arg->num_pools) {
+		/* This should never happen. */
+		error("%s: Got more pools than are in the dict. Cannot parse pools.",
+		      __func__);
+		rc = DATA_FOR_EACH_FAIL;
+		goto next;
 	}
 
-	return pools;
+	pools[i].free = NO_VAL64;
+	pools[i].granularity = NO_VAL64;
+	pools[i].quantity = NO_VAL64;
+
+	if (_data_get_val_from_key(data, "id", DATA_TYPE_STRING, true,
+				   &pools[i].name) != SLURM_SUCCESS) {
+		error("%s: Failure parsing id", __func__);
+		rc = DATA_FOR_EACH_FAIL;
+		goto next;
+	}
+
+	if (_data_get_val_from_key(data, "free", DATA_TYPE_INT_64, false,
+				   &pools[i].free) != SLURM_SUCCESS) {
+		error("%s: Failure parsing free", __func__);
+		rc = DATA_FOR_EACH_FAIL;
+		goto next;
+	}
+
+	if (_data_get_val_from_key(data, "granularity", DATA_TYPE_INT_64, false,
+				   &pools[i].granularity) != SLURM_SUCCESS) {
+		error("%s: Failure parsing granularity", __func__);
+		rc = DATA_FOR_EACH_FAIL;
+		goto next;
+	}
+
+	if (_data_get_val_from_key(data, "quantity", DATA_TYPE_INT_64, false,
+				   &pools[i].quantity) != SLURM_SUCCESS) {
+		error("%s: Failure parsing quantity", __func__);
+		rc = DATA_FOR_EACH_FAIL;
+		goto next;
+	}
+
+next:
+	data_arg->i += 1;
+
+	return rc;
 }
 
 static bb_pools_t *_bb_get_pools(int *num_pools, uint32_t timeout, int *out_rc)
@@ -724,8 +764,9 @@ static bb_pools_t *_bb_get_pools(int *num_pools, uint32_t timeout, int *out_rc)
 	char *resp_msg = NULL;
 	char *lua_func_name = "slurm_bb_pools";
 	bb_pools_t *pools = NULL;
-	json_object *j;
-	json_object_iter iter;
+	data_pools_arg_t arg;
+	data_t *data = NULL;
+	data_t *data_tmp = NULL;
 	DEF_TIMERS;
 
 	*num_pools = 0;
@@ -747,23 +788,41 @@ static bb_pools_t *_bb_get_pools(int *num_pools, uint32_t timeout, int *out_rc)
 		return NULL;
 	}
 
-	j = json_tokener_parse(resp_msg);
-	if (j == NULL) {
-		error("json parser failed on \"%s\"",
-		      resp_msg);
-		xfree(resp_msg);
-		return NULL;
+	rc = data_g_deserialize(&data, resp_msg, strlen(resp_msg),
+				MIME_TYPE_JSON);
+	if ((rc != SLURM_SUCCESS) || !data) {
+		error("%s: Problem parsing \"%s\": %s",
+		      __func__, resp_msg, slurm_strerror(rc));
+		goto cleanup;
 	}
-	xfree(resp_msg);
 
-	json_object_object_foreachC(j, iter) {
-		if (pools) {
-			error("Multiple pool objects");
-			break;
-		}
-		pools = _json_parse_pools_array(j, iter.key, num_pools);
+	data_tmp = data_resolve_dict_path(data, "/pools");
+	if (!data_tmp || (data_get_type(data_tmp) != DATA_TYPE_LIST)) {
+		error("%s: Did not find pools dictionary; problem parsing \"%s\"",
+		      __func__, resp_msg);
+		goto cleanup;
 	}
-	json_object_put(j);	/* Frees json memory */
+
+	*num_pools = (int) data_get_list_length(data_tmp);
+	if (*num_pools == 0) {
+		error("%s: No pools found, problem parsing \"%s\"",
+		      __func__, resp_msg);
+		goto cleanup;
+	}
+
+	pools = xcalloc(*num_pools, sizeof(*pools));
+	arg.num_pools = *num_pools;
+	arg.pools = pools;
+	arg.i = 0;
+	rc = data_list_for_each(data_tmp, _foreach_parse_pool, &arg);
+	if (rc <= 0) {
+		error("%s: Failed to parse pools: \"%s\"", __func__, resp_msg);
+		goto cleanup;
+	}
+
+cleanup:
+	xfree(resp_msg);
+	FREE_NULL_DATA(data);
 
 	return pools;
 }
@@ -1024,7 +1083,7 @@ fini:
 	xfree(resp_msg);
 	xfree(stage_out_args->job_script);
 	xfree(stage_out_args);
-	free_command_argv(argv);
+	xfree_array(argv);
 
 	return NULL;
 }
@@ -1111,7 +1170,8 @@ static int _xlate_batch(job_desc_msg_t *job_desc)
 	 * Any command line --bb options get added to the script
 	 */
 	if (job_desc->burst_buffer) {
-		bb_add_bb_to_script(&job_desc->script, job_desc->burst_buffer);
+		run_command_add_to_script(&job_desc->script,
+					  job_desc->burst_buffer);
 		xfree(job_desc->burst_buffer);
 	}
 
@@ -1503,6 +1563,12 @@ extern int init(void)
                 return rc;
 	lua_script_path = get_extra_conf_path("burst_buffer.lua");
 
+	if ((rc = data_init(MIME_TYPE_JSON_PLUGIN, NULL)) != SLURM_SUCCESS) {
+		error("%s: unable to load JSON serializer: %s",
+		      __func__, slurm_strerror(rc));
+		return rc;
+	}
+
 	/*
 	 * slurmscriptd calls bb_g_init() and then bb_g_run_script(). We only
 	 * need to initialize lua to run the script. We don't want
@@ -1567,6 +1633,7 @@ extern int fini(void)
 
 	slurm_lua_fini();
 	xfree(lua_script_path);
+	/* Don't call data_fini(), that is taken care of elsewhere. */
 
 	return SLURM_SUCCESS;
 }
@@ -1861,13 +1928,15 @@ extern int bb_p_state_pack(uid_t uid, buf_t *buffer, uint16_t protocol_version)
  *
  * Returns a Slurm errno.
  */
-extern int bb_p_job_validate(job_desc_msg_t *job_desc, uid_t submit_uid)
+extern int bb_p_job_validate(job_desc_msg_t *job_desc, uid_t submit_uid,
+			     char **err_msg)
 {
 	uint64_t bb_size = 0;
 	int rc;
 
 	xassert(job_desc);
 	xassert(job_desc->tres_req_cnt);
+	xassert(err_msg);
 
 	rc = _parse_bb_opts(job_desc, &bb_size, submit_uid);
 	if (rc != SLURM_SUCCESS)
@@ -1877,13 +1946,14 @@ extern int bb_p_job_validate(job_desc_msg_t *job_desc, uid_t submit_uid)
 	    (job_desc->burst_buffer[0] == '\0'))
 		return rc;
 
-	log_flag(BURST_BUF, "job_user_id:%u, submit_uid:%d",
+	log_flag(BURST_BUF, "job_user_id:%u, submit_uid:%u",
 		 job_desc->user_id, submit_uid);
 	log_flag(BURST_BUF, "burst_buffer:\n%s",
 		 job_desc->burst_buffer);
 
 	if (job_desc->user_id == 0) {
 		info("User root can not allocate burst buffers");
+		*err_msg = xstrdup("User root can not allocate burst buffers");
 		return ESLURM_BURST_BUFFER_PERMISSION;
 	}
 
@@ -1898,6 +1968,7 @@ extern int bb_p_job_validate(job_desc_msg_t *job_desc, uid_t submit_uid)
 			}
 		}
 		if (!found_user) {
+			*err_msg = xstrdup("User not found in AllowUsers");
 			rc = ESLURM_BURST_BUFFER_PERMISSION;
 			goto fini;
 		}
@@ -1913,6 +1984,7 @@ extern int bb_p_job_validate(job_desc_msg_t *job_desc, uid_t submit_uid)
 			}
 		}
 		if (found_user) {
+			*err_msg = xstrdup("User found in DenyUsers");
 			rc = ESLURM_BURST_BUFFER_PERMISSION;
 			goto fini;
 		}
@@ -1929,45 +2001,6 @@ fini:
 	return rc;
 }
 
-/*
- * For interactive jobs, build a script containing the relevant burst buffer
- * commands, as needed by the Lua API.
- */
-static int _build_bb_script(job_record_t *job_ptr, char *script_file)
-{
-	char *out_buf = NULL;
-	int rc, fd;
-
-	xassert(job_ptr);
-	xassert(job_ptr->burst_buffer);
-
-	/* Open file */
-	(void) unlink(script_file);
-	fd = creat(script_file, 0600);
-	if (fd < 0) {
-		rc = errno;
-		error("Error creating file %s, %m", script_file);
-		return rc;
-	}
-
-	/* Write burst buffer specification to the file. */
-	xstrcat(out_buf, "#!/bin/bash\n");
-	xstrcat(out_buf, job_ptr->burst_buffer);
-	safe_write(fd, out_buf, strlen(out_buf));
-
-	xfree(out_buf);
-	(void) close(fd);
-
-	return SLURM_SUCCESS;
-
-rwfail:
-	error("Failed to write %s to %s", out_buf, script_file);
-	xfree(out_buf);
-	(void) close(fd);
-
-	return SLURM_ERROR;
-}
-
 /*
  * Secondary validation of a job submit request with respect to burst buffer
  * options. Performed after establishing job ID and creating script file.
@@ -2039,12 +2072,12 @@ extern int bb_p_job_validate2(job_record_t *job_ptr, char **err_msg)
 		(void) mkdir(job_dir, 0700);
 		xstrfmtcat(script_file, "%s/script", job_dir);
 		if (job_ptr->batch_flag == 0) {
-			rc = _build_bb_script(job_ptr, script_file);
+			rc = bb_build_bb_script(job_ptr, script_file);
 			if (rc != SLURM_SUCCESS) {
 				/*
 				 * There was an error writing to the script,
-				 * that error was logged by _build_bb_script().
-				 * Bail out now.
+				 * and that error was logged by
+				 * bb_build_bb_script(). Bail out now.
 				 */
 				goto fini;
 			}
@@ -2397,7 +2430,7 @@ fini:
 	xfree(resp_msg);
 	xfree(teardown_args->job_script);
 	xfree(teardown_args);
-	free_command_argv(argv);
+	xfree_array(argv);
 
 	return NULL;
 }
@@ -2460,7 +2493,7 @@ static void *_start_stage_in(void *x)
 	argv[1] = xstrdup_printf("%u", stage_in_args->uid);
 	argv[2] = xstrdup_printf("%u", stage_in_args->gid);
 	argv[3] = xstrdup_printf("%s", stage_in_args->pool);
-	argv[4] = xstrdup_printf("%lu", stage_in_args->bb_size);
+	argv[4] = xstrdup_printf("%"PRIu64, stage_in_args->bb_size);
 	argv[5] = xstrdup_printf("%s", stage_in_args->job_script);
 
 	timeout = bb_state.bb_config.other_timeout;
@@ -2493,7 +2526,7 @@ static void *_start_stage_in(void *x)
 
 	if (rc == SLURM_SUCCESS) {
 		xfree(resp_msg);
-		free_command_argv(argv);
+		xfree_array(argv);
 		argc = 2;
 		argv = xcalloc(argc + 1, sizeof (char *)); /* NULL-terminated */
 		argv[0] = xstrdup_printf("%u", stage_in_args->job_id);
@@ -2539,7 +2572,7 @@ static void *_start_stage_in(void *x)
 
 	if (get_real_size) {
 		xfree(resp_msg);
-		free_command_argv(argv);
+		xfree_array(argv);
 		argc = 1;
 		argv = xcalloc(argc + 1, sizeof(char *)); /* NULL terminated */
 		argv[0] = xstrdup_printf("%u", stage_in_args->job_id);
@@ -2593,7 +2626,7 @@ static void *_start_stage_in(void *x)
 			 * returns something bigger.
 			 */
 			if (((uint64_t)real_size) > bb_job->req_size) {
-				info("%pJ total_size increased from %"PRIu64" to %"PRIu64,
+				info("%pJ total_size increased from %"PRIu64" to %ld",
 				     job_ptr,
 				     bb_job->req_size, real_size);
 				bb_job->total_size = real_size;
@@ -2649,7 +2682,7 @@ fini:
 	xfree(stage_in_args->job_script);
 	xfree(stage_in_args->pool);
 	xfree(stage_in_args);
-	free_command_argv(argv);
+	xfree_array(argv);
 
 	return NULL;
 }
@@ -3041,7 +3074,7 @@ fini:
 	xfree(resp_msg);
 	xfree(pre_run_args->job_script);
 	xfree(pre_run_args);
-	free_command_argv(argv);
+	xfree_array(argv);
 
 	return NULL;
 }
@@ -3120,7 +3153,7 @@ extern int bb_p_job_begin(job_record_t *job_ptr)
 
 	/* resp_msg already logged by _run_lua_script. */
 	xfree(resp_msg);
-	free_command_argv(argv);
+	xfree_array(argv);
 
 	if (rc != SLURM_SUCCESS) {
 		error("paths for %pJ failed", job_ptr);
