@@ -47,9 +47,11 @@
 
 #include "slurm/slurm_errno.h"
 #include "src/common/slurm_xlator.h"
+#include "src/common/cli_filter.h"
 #include "src/common/data.h"
 #include "src/common/slurm_opt.h"
-#include "src/common/cli_filter.h"
+#include "src/common/xmalloc.h"
+#include "src/common/xstring.h"
 #include "src/plugins/cli_filter/common/cli_filter_common.h"
 
 static char **stored_data = NULL;
