@@ -48,9 +48,13 @@
 
 #include "slurm/slurm_errno.h"
 #include "src/common/slurm_xlator.h"
-#include "src/common/uid.h"
-#include "src/common/slurm_opt.h"
+
 #include "src/common/cli_filter.h"
+#include "src/common/read_config.h"
+#include "src/common/slurm_opt.h"
+#include "src/common/uid.h"
+#include "src/common/xmalloc.h"
+#include "src/common/xstring.h"
 #include "src/plugins/cli_filter/common/cli_filter_common.h"
 
 #define USER_DEFAULTS_FILE ".slurm/defaults"
