@@ -57,6 +57,7 @@
 #include "src/common/gpu.h"
 #include "src/common/gres.h"
 #include "src/common/list.h"
+#include "src/common/strnatcmp.h"
 #include "src/common/xstring.h"
 
 #include "../common/gres_common.h"
@@ -86,7 +87,7 @@
  * plugin_version - an unsigned 32-bit integer containing the Slurm version
  * (major.minor.micro combined into a single number).
  */
-const char	*plugin_name		= "Gres GPU plugin";
+const char plugin_name[] = "Gres GPU plugin";
 const char	plugin_type[]		= "gres/gpu";
 const uint32_t	plugin_version		= SLURM_VERSION_NUMBER;
 static char	*gres_name		= "gpu";
@@ -103,67 +104,18 @@ extern void gres_p_step_hardware_fini(void)
 	gpu_g_step_hardware_fini();
 }
 
-static void _set_env(char ***env_ptr, bitstr_t *gres_bit_alloc,
-		     bitstr_t *usable_gres, uint64_t gres_cnt,
-		     bool *already_seen, int *local_inx,
-		     bool is_task, bool is_job, gres_internal_flags_t flags)
+/* Sort strings in natural sort ascending order, except sort nulls last */
+static int _sort_string_null_last(char *x, char *y)
 {
-	char *global_list = NULL, *local_list = NULL, *slurm_env_var = NULL;
+	/* Make NULLs greater than non-NULLs, so NULLs are sorted last */
+	if (!x && y)
+		return 1;
+	else if (x && !y)
+		return -1;
+	else if (!x && !y)
+		return 0;
 
-	if (is_job)
-		slurm_env_var = "SLURM_JOB_GPUS";
-	else
-		slurm_env_var = "SLURM_STEP_GPUS";
-
-	if (*already_seen) {
-		global_list = xstrdup(getenvp(*env_ptr, slurm_env_var));
-
-		/*
-		 * Determine which existing env to check for local list.  We
-		 * only need one since they are all the same and this is only
-		 * for printing an error later.
-		 */
-		if (node_flags & GRES_CONF_ENV_NVML)
-			local_list = xstrdup(getenvp(*env_ptr,
-						     "CUDA_VISIBLE_DEVICES"));
-		else if (node_flags & GRES_CONF_ENV_RSMI)
-			local_list = xstrdup(getenvp(*env_ptr,
-						     "ROCR_VISIBLE_DEVICES"));
-		else if (node_flags & GRES_CONF_ENV_OPENCL)
-			local_list = xstrdup(getenvp(*env_ptr,
-						     "GPU_DEVICE_ORDINAL"));
-	}
-
-	common_gres_set_env(gres_devices, env_ptr,
-			    usable_gres, "", local_inx,  gres_bit_alloc,
-			    &local_list, &global_list, is_task, is_job, NULL,
-			    flags, false);
-
-	if (gres_cnt) {
-		char *gpus_on_node = xstrdup_printf("%"PRIu64, gres_cnt);
-		env_array_overwrite(env_ptr, "SLURM_GPUS_ON_NODE",
-				    gpus_on_node);
-		xfree(gpus_on_node);
-	}
-
-	if (global_list) {
-		env_array_overwrite(env_ptr, slurm_env_var, global_list);
-		xfree(global_list);
-	}
-
-	if (local_list) {
-		if (node_flags & GRES_CONF_ENV_NVML)
-			env_array_overwrite(env_ptr, "CUDA_VISIBLE_DEVICES",
-					    local_list);
-		if (node_flags & GRES_CONF_ENV_RSMI)
-			env_array_overwrite(env_ptr, "ROCR_VISIBLE_DEVICES",
-					    local_list);
-		if (node_flags & GRES_CONF_ENV_OPENCL)
-			env_array_overwrite(env_ptr, "GPU_DEVICE_ORDINAL",
-					    local_list);
-		xfree(local_list);
-		*already_seen = true;
-	}
+	return strnatcmp(x, y);
 }
 
 /*
@@ -172,21 +124,21 @@ static void _set_env(char ***env_ptr, bitstr_t *gres_bit_alloc,
  */
 static int _sort_gpu_by_type_name(void *x, void *y)
 {
-	gres_slurmd_conf_t *gres_record1 = *(gres_slurmd_conf_t **)x;
-	gres_slurmd_conf_t *gres_record2 = *(gres_slurmd_conf_t **)y;
+	gres_slurmd_conf_t *gres_slurmd_conf1 = *(gres_slurmd_conf_t **)x;
+	gres_slurmd_conf_t *gres_slurmd_conf2 = *(gres_slurmd_conf_t **)y;
 	int val1, val2, ret;
 
-	if (!gres_record1->type_name && !gres_record2->type_name)
+	if (!gres_slurmd_conf1->type_name && !gres_slurmd_conf2->type_name)
 		return 0;
 
-	if (gres_record1->type_name && !gres_record2->type_name)
+	if (gres_slurmd_conf1->type_name && !gres_slurmd_conf2->type_name)
 		return 1;
 
-	if (!gres_record1->type_name && gres_record2->type_name)
+	if (!gres_slurmd_conf1->type_name && gres_slurmd_conf2->type_name)
 		return -1;
 
-	val1 = strlen(gres_record1->type_name);
-	val2 = strlen(gres_record2->type_name);
+	val1 = strlen(gres_slurmd_conf1->type_name);
+	val2 = strlen(gres_slurmd_conf2->type_name);
 	/*
 	 * By default, qsort orders in ascending order (smallest first). We want
 	 * descending order (longest first), so invert order by negating.
@@ -195,11 +147,13 @@ static int _sort_gpu_by_type_name(void *x, void *y)
 
 	/* Sort by type name value if type name length is equal */
 	if (ret == 0)
-		ret = xstrcmp(gres_record1->type_name, gres_record2->type_name);
+		ret = xstrcmp(gres_slurmd_conf1->type_name,
+			      gres_slurmd_conf2->type_name);
 
 	/* Sort by file name if type name value is equal */
 	if (ret == 0)
-		ret = xstrcmp(gres_record1->file, gres_record2->file);
+		ret = _sort_string_null_last(gres_slurmd_conf1->file,
+					     gres_slurmd_conf2->file);
 
 	return ret;
 }
@@ -210,24 +164,37 @@ static int _sort_gpu_by_type_name(void *x, void *y)
  */
 static int _find_type_in_gres_list(void *x, void *key)
 {
-	gres_slurmd_conf_t *conf_gres = (gres_slurmd_conf_t *)x;
+	gres_slurmd_conf_t *gres_slurmd_conf = (gres_slurmd_conf_t *)x;
 	char *sys_gres_type = (char *)key;
 
-	if (!conf_gres)
+	if (!gres_slurmd_conf)
 		return 0;
 
 	/* If count is 0, then we already accounted for it */
-	if (conf_gres->count == 0)
+	if (gres_slurmd_conf->count == 0)
 		return 0;
 
-	xassert(conf_gres->count == 1);
+	xassert(gres_slurmd_conf->count == 1);
 
-	if (xstrcasestr(sys_gres_type, conf_gres->type_name))
+	if (xstrcasestr(sys_gres_type, gres_slurmd_conf->type_name))
 		return 1;
 	else
 		return 0;
 }
 
+static int _find_nonnull_type_in_gres_list(void *x, void *key)
+{
+	gres_slurmd_conf_t *gres_slurmd_conf = (gres_slurmd_conf_t *) x;
+
+	if (!gres_slurmd_conf)
+		return 0;
+
+	if (gres_slurmd_conf->type_name && gres_slurmd_conf->type_name[0])
+		return 1;
+
+	return 0;
+}
+
 /*
  * Sync the GRES type of each device detected on the system (gres_list_system)
  * to its corresponding GRES type specified in [gres|slurm].conf. In effect, the
@@ -241,11 +208,21 @@ static void _normalize_sys_gres_types(List gres_list_system,
 {
 	gres_slurmd_conf_t *sys_gres, *conf_gres;
 	ListIterator itr;
+	bool strip_type = true;
 
 	/* No need to sync anything if configured GRES list is empty */
 	if (!gres_list_conf_single || list_count(gres_list_conf_single) == 0)
 		return;
 
+	/*
+	 * Determine if any of the existing GRES have their types defined. If
+	 * have a type, then all GRES must have a type defined and stripping the
+	 * type is not helpful
+	 */
+	if (list_find_first(gres_list_conf_single,
+			    _find_nonnull_type_in_gres_list, NULL))
+		strip_type = false;
+
 	/*
 	 * Sort conf and sys gres lists by longest GRES type to shortest, so we
 	 * can avoid issues if e.g. `k20m` and `k20m1` are both specified.
@@ -260,10 +237,12 @@ static void _normalize_sys_gres_types(List gres_list_system,
 					    _find_type_in_gres_list,
 					    sys_gres->type_name);
 		if (!conf_gres) {
-			info("Could not find an unused configuration record with a GRES type that is a substring of system device `%s`. Setting system GRES type to NULL",
-			     sys_gres->type_name);
-			xfree(sys_gres->type_name);
-			sys_gres->config_flags &= ~GRES_CONF_HAS_TYPE;
+			if (strip_type) {
+				info("Could not find an unused configuration record with a GRES type that is a substring of system device `%s`. Setting system GRES type to NULL",
+				     sys_gres->type_name);
+				xfree(sys_gres->type_name);
+				sys_gres->config_flags &= ~GRES_CONF_HAS_TYPE;
+			}
 			continue;
 		}
 
@@ -347,53 +326,14 @@ static int _validate_cpus_links(gres_slurmd_conf_t *conf_gres,
 	return 1;
 }
 
-/* Given a file name return its numeric suffix */
-static int _file_inx(char *fname)
+/* Sort gres/gpu records by "File" value in ascending order, with nulls last */
+static int _sort_gpu_by_file(void *x, void *y)
 {
-	int i, len, mult = 1, num, val = 0;
+	gres_slurmd_conf_t *gres_slurmd_conf1 = *(gres_slurmd_conf_t **) x;
+	gres_slurmd_conf_t *gres_slurmd_conf2 = *(gres_slurmd_conf_t **) y;
 
-	if (!fname)
-		return 0;
-	len = strlen(fname);
-	if (len == 0)
-		return val;
-	for (i = 1; i <= len; i++) {
-		if ((fname[len - i] < '0') ||
-		    (fname[len - i] > '9'))
-			break;
-		num = fname[len - i] - '0';
-		val += (num * mult);
-		mult *= 10;
-	}
-	return val;
-}
-
-/* Sort gres/gpu records by "File" value */
-static int _sort_gpu_by_file(void *x, void *y, bool asc)
-{
-	gres_slurmd_conf_t *gres_record1 = *(gres_slurmd_conf_t **) x;
-	gres_slurmd_conf_t *gres_record2 = *(gres_slurmd_conf_t **) y;
-	int val1, val2;
-
-	val1 = _file_inx(gres_record1->file);
-	val2 = _file_inx(gres_record2->file);
-
-	if (asc)
-		return (val1 - val2);
-	else
-		return -(val1 - val2);
-}
-
-/* Sort gres/gpu records by "File" value in descending order (nulls first) */
-static int _sort_gpu_by_file_desc(void *x, void *y)
-{
-	return _sort_gpu_by_file(x, y, false);
-}
-
-/* Sort gres/gpu records by "File" value in ascending order */
-static int _sort_gpu_by_file_asc(void *x, void *y)
-{
-	return _sort_gpu_by_file(x, y, true);
+	return _sort_string_null_last(gres_slurmd_conf1->file,
+				      gres_slurmd_conf2->file);
 }
 
 /*
@@ -413,18 +353,18 @@ static int _sort_gpu_by_file_asc(void *x, void *y)
  */
 static int _sort_gpu_by_links_order(void *x, void *y)
 {
-	gres_slurmd_conf_t *gres_record_x = *(gres_slurmd_conf_t **)x;
-	gres_slurmd_conf_t *gres_record_y = *(gres_slurmd_conf_t **)y;
+	gres_slurmd_conf_t *gres_slurmd_conf_x = *(gres_slurmd_conf_t **)x;
+	gres_slurmd_conf_t *gres_slurmd_conf_y = *(gres_slurmd_conf_t **)y;
 	int index_x, index_y;
 
 	/* Make null links appear last in sort */
-	if (!gres_record_x->links && gres_record_y->links)
+	if (!gres_slurmd_conf_x->links && gres_slurmd_conf_y->links)
 		return 1;
-	if (gres_record_x->links && !gres_record_y->links)
+	if (gres_slurmd_conf_x->links && !gres_slurmd_conf_y->links)
 		return -1;
 
-	index_x = gres_links_validate(gres_record_x->links);
-	index_y = gres_links_validate(gres_record_y->links);
+	index_x = gres_links_validate(gres_slurmd_conf_x->links);
+	index_y = gres_links_validate(gres_slurmd_conf_y->links);
 
 	if (index_x < -1 || index_y < -1)
 		error("%s: invalid links value found", __func__);
@@ -460,7 +400,7 @@ static int _sort_gpu_by_links_order(void *x, void *y)
 static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 {
 	ListIterator itr, itr2;
-	gres_slurmd_conf_t *gres_record, *sys_record;
+	gres_slurmd_conf_t *gres_slurmd_conf, *gres_slurmd_conf_sys;
 	List gres_list_conf_single, gres_list_gpu = NULL, gres_list_non_gpu;
 
 	if (gres_list_conf == NULL) {
@@ -477,44 +417,44 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 
 	// Break down gres_list_conf into 1 device per record
 	itr = list_iterator_create(gres_list_conf);
-	while ((gres_record = list_next(itr))) {
+	while ((gres_slurmd_conf = list_next(itr))) {
 		int i;
 		hostlist_t hl;
 		char *hl_name;
 
-		if (!gres_record->count)
+		if (!gres_slurmd_conf->count)
 			continue;
 
-		if (xstrcasecmp(gres_record->name, "gpu")) {
+		if (xstrcasecmp(gres_slurmd_conf->name, "gpu")) {
 			/* Move record into non-GPU GRES list */
-			gres_record = list_remove(itr);
+			gres_slurmd_conf = list_remove(itr);
 			debug2("preserving original `%s` GRES record",
-			       gres_record->name);
-			list_append(gres_list_non_gpu, gres_record);
+			       gres_slurmd_conf->name);
+			list_append(gres_list_non_gpu, gres_slurmd_conf);
 			continue;
 		}
 
-		if (gres_record->count == 1) {
+		if (gres_slurmd_conf->count == 1) {
 			/* Already count of 1; move into single-GPU GRES list */
-			gres_record = list_remove(itr);
-			list_append(gres_list_conf_single, gres_record);
+			gres_slurmd_conf = list_remove(itr);
+			list_append(gres_list_conf_single, gres_slurmd_conf);
 			continue;
-		} else if (!gres_record->file) {
+		} else if (!gres_slurmd_conf->file) {
 			/*
 			 * Split this record into multiple single-GPU records
 			 * and add them to the single-GPU GRES list
 			 */
-			for (i = 0; i < gres_record->count; i++)
+			for (i = 0; i < gres_slurmd_conf->count; i++)
 				add_gres_to_list(gres_list_conf_single,
-						 gres_record->name, 1,
-						 gres_record->cpu_cnt,
-						 gres_record->cpus,
-						 gres_record->cpus_bitmap,
-						 gres_record->file,
-						 gres_record->type_name,
-						 gres_record->links,
-						 gres_record->unique_id,
-						 gres_record->config_flags);
+						 gres_slurmd_conf->name, 1,
+						 gres_slurmd_conf->cpu_cnt,
+						 gres_slurmd_conf->cpus,
+						 gres_slurmd_conf->cpus_bitmap,
+						 gres_slurmd_conf->file,
+						 gres_slurmd_conf->type_name,
+						 gres_slurmd_conf->links,
+						 gres_slurmd_conf->unique_id,
+						 gres_slurmd_conf->config_flags);
 			continue;
 		}
 
@@ -522,21 +462,21 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 		 * count > 1 and we have devices;
 		 * Break down record into individual devices.
 		 */
-		hl = hostlist_create(gres_record->file);
+		hl = hostlist_create(gres_slurmd_conf->file);
 		while ((hl_name = hostlist_shift(hl))) {
 			/*
 			 * Split this record into multiple single-GPU,
 			 * single-file records and add to single-GPU GRES list
 			 */
 			add_gres_to_list(gres_list_conf_single,
-					 gres_record->name, 1,
-					 gres_record->cpu_cnt,
-					 gres_record->cpus,
-					 gres_record->cpus_bitmap, hl_name,
-					 gres_record->type_name,
-					 gres_record->links,
-					 gres_record->unique_id,
-					 gres_record->config_flags);
+					 gres_slurmd_conf->name, 1,
+					 gres_slurmd_conf->cpu_cnt,
+					 gres_slurmd_conf->cpus,
+					 gres_slurmd_conf->cpus_bitmap, hl_name,
+					 gres_slurmd_conf->type_name,
+					 gres_slurmd_conf->links,
+					 gres_slurmd_conf->unique_id,
+					 gres_slurmd_conf->config_flags);
 			free(hl_name);
 		}
 		hostlist_destroy(hl);
@@ -548,17 +488,22 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 	 */
 	_normalize_sys_gres_types(gres_list_system, gres_list_conf_single);
 
-	/* Sort so null files are last for _match_gres() */
-	list_sort(gres_list_conf_single, _sort_gpu_by_file_desc);
+	/*
+	 *  Sort null files last, so that conf records with a specified File
+	 *  are matched first in _match_gres(). Then, conf records without a
+	 *  File can fill in any remaining holes.
+	 */
+	list_sort(gres_list_conf_single, _sort_gpu_by_file);
 	/* Sort system devices in the same way for convenience */
-	list_sort(gres_list_system, _sort_gpu_by_file_desc);
+	list_sort(gres_list_system, _sort_gpu_by_file);
 
 	itr = list_iterator_create(gres_list_conf_single);
 	itr2 = list_iterator_create(gres_list_system);
-	while ((gres_record = list_next(itr))) {
+	while ((gres_slurmd_conf = list_next(itr))) {
 		list_iterator_reset(itr2);
-		while ((sys_record = list_next(itr2))) {
-			if (!_match_gres(gres_record, sys_record)) {
+		while ((gres_slurmd_conf_sys = list_next(itr2))) {
+			if (!_match_gres(gres_slurmd_conf,
+					 gres_slurmd_conf_sys)) {
 				continue;
 			}
 
@@ -568,21 +513,24 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 			 * does not match the system, emit error. If null, just
 			 * use the system-detected value.
 			 */
-			if (!_validate_cpus_links(gres_record, sys_record)) {
+			if (!_validate_cpus_links(gres_slurmd_conf,
+						  gres_slurmd_conf_sys)) {
 				/* What was specified differs from system */
 				error("This GPU specified in [slurm|gres].conf has mismatching Cores or Links from the device found on the system. Ignoring it.");
 				error("[slurm|gres].conf:");
-				print_gres_conf(gres_record, LOG_LEVEL_ERROR);
+				print_gres_conf(gres_slurmd_conf,
+						LOG_LEVEL_ERROR);
 				error("system:");
-				print_gres_conf(sys_record, LOG_LEVEL_ERROR);
+				print_gres_conf(gres_slurmd_conf_sys,
+						LOG_LEVEL_ERROR);
 
-				xassert(sys_record->count == 1);
+				xassert(gres_slurmd_conf_sys->count == 1);
 
 				/*
 				 * Temporarily set the sys record count to 0 to
 				 * mark it as already "used up"
 				 */
-				sys_record->count = 0;
+				gres_slurmd_conf_sys->count = 0;
 				break;
 			}
 
@@ -590,12 +538,12 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 			break;
 		}
 
-		if (sys_record) {
+		if (gres_slurmd_conf_sys) {
 			/*
 			 * Completely ignore this conf record if Cores and/or
 			 * Links do not match the corresponding system GPU
 			 */
-			if (sys_record->count == 0)
+			if (gres_slurmd_conf_sys->count == 0)
 				continue;
 
 			/*
@@ -603,7 +551,7 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 			 * configured GPU, add the system GPU to the final list
 			 */
 			debug("Including the following GPU matched between system and configuration:");
-			print_gres_conf(sys_record, LOG_LEVEL_DEBUG);
+			print_gres_conf(gres_slurmd_conf_sys, LOG_LEVEL_DEBUG);
 
 			/*
 			 * If the conf record did not fall back to default env
@@ -611,28 +559,30 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 			 * the conf's env flags. Otherwise, use the AutoDetected
 			 * env flags.
 			 */
-			if (!(gres_record->config_flags & GRES_CONF_ENV_DEF)) {
-				sys_record->config_flags &= ~GRES_CONF_ENV_SET;
-				sys_record->config_flags |=
-					gres_record->config_flags &
+			if (!(gres_slurmd_conf->config_flags &
+			      GRES_CONF_ENV_DEF)) {
+				gres_slurmd_conf_sys->config_flags &=
+					~GRES_CONF_ENV_SET;
+				gres_slurmd_conf_sys->config_flags |=
+					gres_slurmd_conf->config_flags &
 					GRES_CONF_ENV_SET;
 			}
 
 			list_remove(itr2);
-			list_append(gres_list_gpu, sys_record);
+			list_append(gres_list_gpu, gres_slurmd_conf_sys);
 			continue;
 		}
 
 		/* Else, config-only GPU */
-		if (gres_record->file) {
+		if (gres_slurmd_conf->file) {
 			/*
 			 * Add the "extra" configured GPU to the final list, but
 			 * only if file is specified
 			 */
 			debug("Including the following config-only GPU:");
-			print_gres_conf(gres_record, LOG_LEVEL_DEBUG);
+			print_gres_conf(gres_slurmd_conf, LOG_LEVEL_DEBUG);
 			list_remove(itr);
-			list_append(gres_list_gpu, gres_record);
+			list_append(gres_list_gpu, gres_slurmd_conf);
 		} else {
 			/*
 			 * Either the conf GPU was specified in slurm.conf only,
@@ -640,7 +590,7 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 			 * specified in gres.conf. Either way, ignore it.
 			 */
 			error("Discarding the following config-only GPU due to lack of File specification:");
-			print_gres_conf(gres_record, LOG_LEVEL_ERROR);
+			print_gres_conf(gres_slurmd_conf, LOG_LEVEL_ERROR);
 		}
 
 	}
@@ -648,9 +598,9 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 
 	/* Reset the system GPU counts, in case system list is used after */
 	list_iterator_reset(itr2);
-	while ((sys_record = list_next(itr2)))
-		if (sys_record->count == 0)
-			sys_record->count = 1;
+	while ((gres_slurmd_conf_sys = list_next(itr2)))
+		if (gres_slurmd_conf_sys->count == 0)
+			gres_slurmd_conf_sys->count = 1;
 	list_iterator_destroy(itr2);
 
 	/* Print out all the leftover system GPUs that are not being used */
@@ -663,7 +613,7 @@ static void _merge_system_gres_conf(List gres_list_conf, List gres_list_system)
 	list_flush(gres_list_conf);
 	if (gres_list_gpu && list_count(gres_list_gpu)) {
 		/* Sort by device file first, in case no links */
-		list_sort(gres_list_gpu, _sort_gpu_by_file_asc);
+		list_sort(gres_list_gpu, _sort_gpu_by_file);
 		/* Sort by links, which is a stand-in for PCI bus ID order */
 		list_sort(gres_list_gpu, _sort_gpu_by_links_order);
 		debug2("gres_list_gpu");
@@ -786,7 +736,7 @@ static void _add_fake_gpus_from_file(List gres_list_system,
 			fatal("bit_unfmt() failed for CPU range: %s",
 			      cpu_range);
 
-		flags = gres_flags_parse(flags_str, NULL);
+		flags = gres_flags_parse(flags_str, NULL, NULL);
 
 		// Add the GPU specified by the parsed line
 		add_gres_to_list(gres_list_system, "gpu", 1, cpu_count,
@@ -844,24 +794,6 @@ extern int fini(void)
 	return SLURM_SUCCESS;
 }
 
-static int _set_env_types_on_node_flags(void *x, void *arg)
-{
-	gres_slurmd_conf_t *gres_slurmd_conf = (gres_slurmd_conf_t *)x;
-
-	if (gres_slurmd_conf->config_flags & GRES_CONF_ENV_NVML)
-		node_flags |= GRES_CONF_ENV_NVML;
-	if (gres_slurmd_conf->config_flags & GRES_CONF_ENV_RSMI)
-		node_flags |= GRES_CONF_ENV_RSMI;
-	if (gres_slurmd_conf->config_flags & GRES_CONF_ENV_OPENCL)
-		node_flags |= GRES_CONF_ENV_OPENCL;
-
-	/* No need to continue if all are set */
-	if ((node_flags & GRES_CONF_ENV_SET) == GRES_CONF_ENV_SET)
-		return -1;
-
-	return 0;
-}
-
 /*
  * We could load gres state or validate it using various mechanisms here.
  * This only validates that the configuration was specified in gres.conf or
@@ -882,8 +814,11 @@ extern int gres_p_node_config_load(List gres_conf_list,
 	}
 
 	gres_list_system = _get_system_gpu_list_fake();
-	// Only query real system devices if there is no fake override
-	if (!gres_list_system)
+	/*
+	 * Only query real system devices if there is no fake override and we
+	 * are running in the slurmd.
+	 */
+	if (!gres_list_system && node_config->in_slurmd)
 		gres_list_system = gpu_g_get_system_gpu_list(node_config);
 
 	if (slurm_conf.debug_flags & DEBUG_FLAG_GRES)
@@ -895,22 +830,23 @@ extern int gres_p_node_config_load(List gres_conf_list,
 			log_var(log_lvl,
 				"There were 0 GPUs detected on the system");
 		log_var(log_lvl,
-			"%s: Merging gres.conf with system GPUs",
+			"%s: Merging configured GRES with system GPUs",
 			plugin_name);
 		_merge_system_gres_conf(gres_conf_list, gres_list_system);
 		FREE_NULL_LIST(gres_list_system);
 
 		if (!gres_conf_list || list_is_empty(gres_conf_list))
-			log_var(log_lvl, "%s: Final merged gres.conf list is empty",
+			log_var(log_lvl, "%s: Final merged GRES list is empty",
 				plugin_name);
 		else {
-			log_var(log_lvl, "%s: Final merged gres.conf list:",
+			log_var(log_lvl, "%s: Final merged GRES list:",
 				plugin_name);
 			print_gres_list(gres_conf_list, log_lvl);
 		}
 	}
 
-	rc = common_node_config_load(gres_conf_list, gres_name, &gres_devices);
+	rc = common_node_config_load(gres_conf_list, gres_name, node_config,
+				     &gres_devices);
 
 	/*
 	 * See what envs the gres_slurmd_conf records want to set (if one
@@ -919,8 +855,8 @@ extern int gres_p_node_config_load(List gres_conf_list,
 	 */
 	node_flags = 0;
 	(void) list_for_each(gres_conf_list,
-			     _set_env_types_on_node_flags,
-			     NULL);
+			     gres_common_set_env_types_on_node_flags,
+			     &node_flags);
 
 	if (rc != SLURM_SUCCESS)
 		fatal("%s failed to load configuration", plugin_name);
@@ -948,8 +884,9 @@ extern void gres_p_job_set_env(char ***job_env_ptr,
 	int local_inx = 0;
 	bool already_seen = false;
 
-	_set_env(job_env_ptr, gres_bit_alloc, NULL, gres_cnt,
-		 &already_seen, &local_inx, false, true, flags);
+	gres_common_gpu_set_env(job_env_ptr, gres_bit_alloc, NULL,
+				&already_seen, &local_inx, false, true, flags,
+				node_flags, gres_devices);
 }
 
 /*
@@ -964,8 +901,9 @@ extern void gres_p_step_set_env(char ***step_env_ptr,
 	static int local_inx = 0;
 	static bool already_seen = false;
 
-	_set_env(step_env_ptr, gres_bit_alloc, NULL, gres_cnt,
-		 &already_seen, &local_inx, false, false, flags);
+	gres_common_gpu_set_env(step_env_ptr, gres_bit_alloc, NULL,
+				&already_seen, &local_inx, false, false, flags,
+				node_flags, gres_devices);
 }
 
 /*
@@ -981,8 +919,10 @@ extern void gres_p_task_set_env(char ***step_env_ptr,
 	static int local_inx = 0;
 	static bool already_seen = false;
 
-	_set_env(step_env_ptr, gres_bit_alloc, usable_gres, gres_cnt,
-		 &already_seen, &local_inx, true, false, flags);
+	gres_common_gpu_set_env(
+		step_env_ptr, gres_bit_alloc, usable_gres,
+		&already_seen, &local_inx, true, false, flags,
+		node_flags, gres_devices);
 }
 
 /* Send GPU-specific GRES information to slurmstepd via a buffer */
@@ -1016,7 +956,7 @@ unpack_error:
  *            DO NOT FREE: This is a pointer into the job's data structure
  * RET - SLURM_SUCCESS or error code
  */
-extern int gres_p_get_job_info(gres_job_state_t *job_gres_data,
+extern int gres_p_get_job_info(gres_job_state_t *gres_js,
 			       uint32_t node_inx,
 			       enum gres_job_data_type data_type, void *data)
 {
@@ -1025,7 +965,7 @@ extern int gres_p_get_job_info(gres_job_state_t *job_gres_data,
 
 /*
  * get data from a step's GRES data structure
- * IN step_gres_data  - step's GRES data structure
+ * IN gres_ss  - step's GRES data structure
  * IN node_inx - zero-origin index of the node within the job's allocation
  *	for which data is desired. Note this can differ from the step's
  *	node allocation index.
@@ -1034,7 +974,7 @@ extern int gres_p_get_job_info(gres_job_state_t *job_gres_data,
  *            DO NOT FREE: This is a pointer into the step's data structure
  * RET - SLURM_SUCCESS or error code
  */
-extern int gres_p_get_step_info(gres_step_state_t *step_gres_data,
+extern int gres_p_get_step_info(gres_step_state_t *gres_ss,
 				uint32_t node_inx,
 				enum gres_step_data_type data_type, void *data)
 {
@@ -1055,24 +995,24 @@ extern List gres_p_get_devices(void)
  * prolog or epilog based GRES allocated to the job.
  */
 extern gres_epilog_info_t *gres_p_epilog_build_env(
-	gres_job_state_t *gres_job_ptr)
+	gres_job_state_t *gres_js)
 {
 	int i;
-	gres_epilog_info_t *epilog_info;
+	gres_epilog_info_t *gres_ei;
 
-	epilog_info = xmalloc(sizeof(gres_epilog_info_t));
-	epilog_info->node_cnt = gres_job_ptr->node_cnt;
-	epilog_info->gres_bit_alloc = xcalloc(epilog_info->node_cnt,
-					      sizeof(bitstr_t *));
-	for (i = 0; i < epilog_info->node_cnt; i++) {
-		if (gres_job_ptr->gres_bit_alloc &&
-		    gres_job_ptr->gres_bit_alloc[i]) {
-			epilog_info->gres_bit_alloc[i] =
-				bit_copy(gres_job_ptr->gres_bit_alloc[i]);
+	gres_ei = xmalloc(sizeof(gres_epilog_info_t));
+	gres_ei->node_cnt = gres_js->node_cnt;
+	gres_ei->gres_bit_alloc = xcalloc(gres_ei->node_cnt,
+					  sizeof(bitstr_t *));
+	for (i = 0; i < gres_ei->node_cnt; i++) {
+		if (gres_js->gres_bit_alloc &&
+		    gres_js->gres_bit_alloc[i]) {
+			gres_ei->gres_bit_alloc[i] =
+				bit_copy(gres_js->gres_bit_alloc[i]);
 		}
 	}
 
-	return epilog_info;
+	return gres_ei;
 }
 
 /*
@@ -1080,87 +1020,8 @@ extern gres_epilog_info_t *gres_p_epilog_build_env(
  * GRES allocated to the job.
  */
 extern void gres_p_epilog_set_env(char ***epilog_env_ptr,
-				  gres_epilog_info_t *epilog_info, int node_inx)
+				  gres_epilog_info_t *gres_ei, int node_inx)
 {
-	int dev_inx_first = -1, dev_inx_last, dev_inx;
-	int env_inx = 0;
-	gres_device_t *gres_device;
-	char *vendor_gpu_str = NULL;
-	char *slurm_gpu_str = NULL;
-	char *sep = "";
-	ListIterator iter;
-
-	xassert(epilog_env_ptr);
-
-	if (!epilog_info)
-		return;
-
-	if (!gres_devices)
-		return;
-
-	if (epilog_info->node_cnt == 0)	/* no_consume */
-		return;
-
-	if (node_inx > epilog_info->node_cnt) {
-		error("bad node index (%d > %u)",
-		      node_inx, epilog_info->node_cnt);
-		return;
-	}
-
-	if (*epilog_env_ptr) {
-		for (env_inx = 0; (*epilog_env_ptr)[env_inx]; env_inx++)
-			;
-		xrealloc(*epilog_env_ptr, sizeof(char *) * (env_inx + 5));
-	} else {
-		*epilog_env_ptr = xcalloc(5, sizeof(char *));
-	}
-
-	if (epilog_info->gres_bit_alloc &&
-	    epilog_info->gres_bit_alloc[node_inx]) {
-		dev_inx_first = bit_ffs(epilog_info->gres_bit_alloc[node_inx]);
-	}
-	if (dev_inx_first >= 0)
-		dev_inx_last = bit_fls(epilog_info->gres_bit_alloc[node_inx]);
-	else
-		dev_inx_last = -2;
-	for (dev_inx = dev_inx_first; dev_inx <= dev_inx_last; dev_inx++) {
-		if (!bit_test(epilog_info->gres_bit_alloc[node_inx], dev_inx))
-			continue;
-		iter = list_iterator_create(gres_devices);
-		while ((gres_device = list_next(iter))) {
-			if (gres_device->index != dev_inx)
-				continue;
-
-			if (gres_device->unique_id)
-				xstrfmtcat(vendor_gpu_str, "%s%s", sep,
-					   gres_device->unique_id);
-			else
-				xstrfmtcat(vendor_gpu_str, "%s%d", sep,
-					   gres_device->index);
-			xstrfmtcat(slurm_gpu_str, "%s%d", sep,
-				   gres_device->index);
-			sep = ",";
-			break;
-		}
-		list_iterator_destroy(iter);
-	}
-	if (vendor_gpu_str) {
-		if (node_flags & GRES_CONF_ENV_NVML)
-			xstrfmtcat((*epilog_env_ptr)[env_inx++],
-				   "CUDA_VISIBLE_DEVICES=%s", vendor_gpu_str);
-		if (node_flags & GRES_CONF_ENV_RSMI)
-			xstrfmtcat((*epilog_env_ptr)[env_inx++],
-				   "ROCR_VISIBLE_DEVICES=%s", vendor_gpu_str);
-		if (node_flags & GRES_CONF_ENV_OPENCL)
-			xstrfmtcat((*epilog_env_ptr)[env_inx++],
-				   "GPU_DEVICE_ORDINAL=%s", vendor_gpu_str);
-		xfree(vendor_gpu_str);
-	}
-	if (slurm_gpu_str) {
-		xstrfmtcat((*epilog_env_ptr)[env_inx++], "SLURM_JOB_GPUS=%s",
-			   slurm_gpu_str);
-		xfree(slurm_gpu_str);
-	}
-
-	return;
+	(void) gres_common_epilog_set_env(epilog_env_ptr, gres_ei,
+					  node_inx, node_flags, gres_devices);
 }
