@@ -55,7 +55,7 @@
 
 #include "src/slurmrestd/operations.h"
 
-#include "src/plugins/openapi/dbv0.0.38/api.h"
+#include "src/plugins/openapi/dbv0.0.36/api.h"
 
 #define MAGIC_FOREACH_USER 0xa13efef2
 typedef struct {
@@ -65,11 +65,6 @@ typedef struct {
 	List qos_list;
 } foreach_user_t;
 
-typedef struct {
-	data_t *errors;
-	slurmdb_user_cond_t *user_cond;
-} foreach_query_search_t;
-
 static int _foreach_user(void *x, void *arg)
 {
 	slurmdb_user_rec_t *user = x;
@@ -89,35 +84,8 @@ static int _foreach_user(void *x, void *arg)
 		return 0;
 }
 
-static data_for_each_cmd_t _foreach_query_search(const char *key,
-						 data_t *data,
-						 void *arg)
-{
-	foreach_query_search_t *args = arg;
-	data_t *errors = args->errors;
-
-	if (!xstrcasecmp("with_deleted", key)) {
-		if (data_convert_type(data, DATA_TYPE_BOOL) != DATA_TYPE_BOOL) {
-			resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				   "must be a Boolean", NULL);
-			return DATA_FOR_EACH_FAIL;
-		}
-
-		if (data->data.bool_u)
-			args->user_cond->with_deleted = true;
-		else
-			args->user_cond->with_deleted = false;
-
-		return DATA_FOR_EACH_CONT;
-	}
-
-	resp_error(errors, ESLURM_REST_INVALID_QUERY, "Unknown query field",
-		   NULL);
-	return DATA_FOR_EACH_FAIL;
-}
-
-static int _dump_users(data_t *resp, data_t *errors, void *auth, char *user_name,
-		       slurmdb_user_cond_t *user_cond)
+static int _dump_users(data_t *resp, data_t *errors, void *auth,
+		       char *user_name)
 {
 	int rc = SLURM_SUCCESS;
 	List user_list = NULL;
@@ -132,12 +100,13 @@ static int _dump_users(data_t *resp, data_t *errors, void *auth, char *user_name
 		.users = data_set_list(data_key_set(resp, "users")),
 	};
 	slurmdb_assoc_cond_t assoc_cond = { 0 };
-
-	user_cond->assoc_cond = &assoc_cond;
-	user_cond->with_assocs = true;
-	user_cond->with_coords = true;
-	/* with_deleted defaults to false */
-	user_cond->with_wckeys = true;
+	slurmdb_user_cond_t user_cond = {
+		.assoc_cond = &assoc_cond,
+		.with_assocs = true,
+		.with_coords = true,
+		.with_deleted = true,
+		.with_wckeys = true,
+	};
 
 	if (user_name) {
 		assoc_cond.user_list = list_create(NULL);
@@ -145,7 +114,7 @@ static int _dump_users(data_t *resp, data_t *errors, void *auth, char *user_name
 	}
 
 	if (!(rc = db_query_list(errors, auth, &user_list, slurmdb_users_get,
-				 user_cond)) &&
+				 &user_cond)) &&
 	    !(rc = db_query_list(errors, auth, &args.tres_list,
 				 slurmdb_tres_get, &tres_cond)) &&
 	    !(rc = db_query_list(errors, auth, &args.qos_list, slurmdb_qos_get,
@@ -175,7 +144,7 @@ typedef struct {
 	int magic;
 	List user_list;
 	data_t *errors;
-	rest_auth_context_t *auth;
+	void *auth;
 } foreach_update_user_t;
 
 static data_for_each_cmd_t _foreach_update_user(data_t *data, void *arg)
@@ -268,7 +237,7 @@ static int _foreach_user_coord_split(void *x, void *arg)
 #define MAGIC_USER_COORD_ADD 0x8e8ffee2
 typedef struct {
 	int magic;
-	rest_auth_context_t *auth;
+	void *auth;
 	int rc;
 	data_t *errors;
 } _foreach_user_coord_add_t;
@@ -278,12 +247,16 @@ static int _foreach_user_coord_add(void *x, void *arg)
 	int rc = SLURM_SUCCESS;
 	add_user_coord_t *uc = x;
 	_foreach_user_coord_add_t *args = arg;
+	void *db_conn;
 
 	xassert(uc->magic == MAGIC_USER_COORD);
 	xassert(args->magic == MAGIC_USER_COORD_ADD);
 
-	if ((args->rc = slurmdb_coord_add(openapi_get_db_conn(args->auth),
-					  uc->acct_list, &uc->user_cond)))
+	if (!(db_conn = openapi_get_db_conn(args->auth)))
+		resp_error(args->errors, ESLURM_DB_CONNECTION_INVALID, NULL,
+			   "openapi_get_db_conn");
+	else if ((args->rc = slurmdb_coord_add(db_conn, uc->acct_list,
+					       &uc->user_cond)))
 		rc = resp_error(args->errors, args->rc, NULL,
 				"slurmdb_coord_add");
 
@@ -301,8 +274,7 @@ static void _destroy_user_coord_t(void *x)
 	xfree(uc);
 }
 
-static int _update_users(data_t *query, data_t *resp, void *auth,
-			 bool commit)
+static int _update_users(data_t *query, data_t *resp, void *auth, bool commit)
 {
 	int rc = SLURM_SUCCESS;
 	data_t *errors = populate_response_format(resp);
@@ -358,8 +330,8 @@ static int _foreach_delete_user(void *x, void *arg)
 	return DATA_FOR_EACH_CONT;
 }
 
-static int _delete_user(data_t *resp, void *auth,
-			char *user_name, data_t *errors)
+static int _delete_user(data_t *resp, void *auth, char *user_name,
+			data_t *errors)
 {
 	int rc = SLURM_SUCCESS;
 	slurmdb_assoc_cond_t assoc_cond = { .user_list = list_create(NULL) };
@@ -370,7 +342,7 @@ static int _delete_user(data_t *resp, void *auth,
 		.with_deleted = false,
 		.with_wckeys = true,
 	};
-	List user_list = NULL;
+	List user_list;
 
 	list_append(assoc_cond.user_list, user_name);
 
@@ -394,32 +366,17 @@ static int _delete_user(data_t *resp, void *auth,
 
 /* based on sacctmgr_list_user() */
 extern int op_handler_users(const char *context_id,
-			    http_request_method_t method,
-			    data_t *parameters, data_t *query, int tag,
-			    data_t *resp, void *auth)
+			    http_request_method_t method, data_t *parameters,
+			    data_t *query, int tag, data_t *resp, void *auth)
 {
 	data_t *errors = populate_response_format(resp);
 
-	if (method == HTTP_REQUEST_GET) {
-		slurmdb_user_cond_t user_cond = {0};
-		if (query && data_get_dict_length(query)) {
-			/* Default to no deleted users */
-			foreach_query_search_t args = {
-				.errors = errors,
-				.user_cond = &user_cond,
-			};
-
-			if (data_dict_for_each(query, _foreach_query_search,
-					       &args) < 0)
-				return ESLURM_REST_INVALID_QUERY;
-		}
-
-		return _dump_users(resp, errors, auth, NULL, &user_cond);
-	} else if (method == HTTP_REQUEST_POST) {
+	if (method == HTTP_REQUEST_GET)
+		return _dump_users(resp, errors, auth, NULL);
+	else if (method == HTTP_REQUEST_POST)
 		return _update_users(query, resp, auth, (tag != CONFIG_OP_TAG));
-	} else {
+	else
 		return ESLURM_REST_INVALID_QUERY;
-	}
 }
 
 static int op_handler_user(const char *context_id, http_request_method_t method,
@@ -430,36 +387,27 @@ static int op_handler_user(const char *context_id, http_request_method_t method,
 	data_t *errors = populate_response_format(resp);
 	char *user_name = get_str_param("user_name", errors, parameters);
 
-	if (!user_name) {
+	if (!user_name)
 		rc = ESLURM_REST_INVALID_QUERY;
-	} else if (method == HTTP_REQUEST_GET) {
-		slurmdb_user_cond_t user_cond = {0};
-		if (query && data_get_dict_length(query)) {
-			/* Default to no deleted users */
-			foreach_query_search_t args = {
-				.errors = errors,
-				.user_cond = &user_cond,
-			};
+	else if (method == HTTP_REQUEST_GET)
+		rc = _dump_users(resp, errors, auth, user_name);
+	else
+		rc = ESLURM_NOT_SUPPORTED;
 
-			if (data_dict_for_each(query, _foreach_query_search,
-					       &args) < 0)
-				return ESLURM_REST_INVALID_QUERY;
-		}
-
-		rc = _dump_users(resp, errors, auth, user_name, &user_cond);
-	} else if (method == HTTP_REQUEST_DELETE) {
+	if ((!rc) && (method == HTTP_REQUEST_GET))
+		rc = _dump_users(resp, errors, auth, user_name);
+	else if ((!rc) && (method == HTTP_REQUEST_DELETE))
 		rc = _delete_user(resp, auth, user_name, errors);
-	} else {
+	else
 		rc = ESLURM_REST_INVALID_QUERY;
-	}
 
 	return rc;
 }
 
 extern void init_op_users(void)
 {
-	bind_operation_handler("/slurmdb/v0.0.38/users/", op_handler_users, 0);
-	bind_operation_handler("/slurmdb/v0.0.38/user/{user_name}",
+	bind_operation_handler("/slurmdb/v0.0.36/users/", op_handler_users, 0);
+	bind_operation_handler("/slurmdb/v0.0.36/user/{user_name}",
 			       op_handler_user, 0);
 }
 
