@@ -53,7 +53,7 @@
 
 #include "src/slurmrestd/operations.h"
 
-#include "src/plugins/openapi/dbv0.0.38/api.h"
+#include "src/plugins/openapi/dbv0.0.37/api.h"
 
 /* based on sacctmgr_list_stats() */
 static int _op_handler_diag(const char *context_id,
@@ -64,11 +64,15 @@ static int _op_handler_diag(const char *context_id,
 	data_t *errors = populate_response_format(resp);
 	parser_env_t penv = { 0 };
 	slurmdb_stats_rec_t *stats_rec = NULL;
-	int rc;
+	int rc = SLURM_SUCCESS;
+	void *db_conn;
 
 	debug4("%s:[%s] diag handler called", __func__, context_id);
 
-	if ((rc = slurmdb_get_stats(openapi_get_db_conn(auth), &stats_rec)))
+	if (!(db_conn = openapi_get_db_conn(auth)))
+		resp_error(errors, ESLURM_DB_CONNECTION_INVALID, NULL,
+			   "openapi_get_db_conn");
+	else if ((rc = slurmdb_get_stats(db_conn, &stats_rec)))
 		resp_error(errors, rc, NULL, "slurmdb_get_stats");
 	else
 		rc = dump(PARSE_STATS_REC, stats_rec,
@@ -82,7 +86,7 @@ static int _op_handler_diag(const char *context_id,
 
 extern void init_op_diag(void)
 {
-	bind_operation_handler("/slurmdb/v0.0.38/diag/", _op_handler_diag, 0);
+	bind_operation_handler("/slurmdb/v0.0.37/diag/", _op_handler_diag, 0);
 }
 
 extern void destroy_op_diag(void)
