@@ -72,7 +72,7 @@
 /* For all nodes in a cluster
  * 1) set default values and
  * 2) return global power allocation/consumption information */
-extern void get_cluster_power(node_record_t *node_record_table_ptr,
+extern void get_cluster_power(node_record_t **node_record_table_ptr,
 			      int node_record_count,
 			      uint32_t *alloc_watts, uint32_t *used_watts)
 {
@@ -84,8 +84,7 @@ extern void get_cluster_power(node_record_t *node_record_table_ptr,
 	if (!(slurm_conf.debug_flags & DEBUG_FLAG_POWER))
 		return;
 
-	for (i = 0, node_ptr = node_record_table_ptr; i < node_record_count;
-	     i++, node_ptr++) {
+	for (i = 0; (node_ptr = next_node(&i)); i++) {
 		if (node_ptr->power) {
 			if (!node_ptr->power->cap_watts) {	/* No limit */
 				if (!node_ptr->power->max_watts)
@@ -114,7 +113,7 @@ extern void get_cluster_power(node_record_t *node_record_table_ptr,
  * NOTE: Job data structure must be locked on function entry
  * NOTE: Call list_delete() to free return value
  * NOTE: This function is currently unused. */
-extern List get_job_power(List job_list, node_record_t *node_record_table_ptr)
+extern List get_job_power(List job_list, node_record_t **node_record_table_ptr)
 {
 	node_record_t *node_ptr;
 	job_record_t *job_ptr;
@@ -144,7 +143,7 @@ extern List get_job_power(List job_list, node_record_t *node_record_table_ptr)
 		for (i = i_first; i <= i_last; i++) {
 			if (!bit_test(job_ptr->node_bitmap, i))
 				continue;
-			node_ptr = node_record_table_ptr + i;
+			node_ptr = node_record_table_ptr[i];
 			if (node_ptr->power) {
 				power_ptr->alloc_watts +=
 					node_ptr->power->cap_watts;
@@ -380,7 +379,7 @@ extern char *power_run_script(char *script_name, char *script_path,
 /* For a newly starting job, set "new_job_time" in each of it's nodes
  * NOTE: The job and node data structures must be locked on function entry */
 extern void set_node_new_job(job_record_t *job_ptr,
-			     node_record_t *node_record_table_ptr)
+			     node_record_t **node_record_table_ptr)
 {
 	int i, i_first, i_last;
 	node_record_t *node_ptr;
@@ -399,7 +398,7 @@ extern void set_node_new_job(job_record_t *job_ptr,
 	for (i = i_first; i <= i_last; i++) {
 		if (!bit_test(job_ptr->node_bitmap, i))
 			continue;
-		node_ptr = node_record_table_ptr + i;
+		node_ptr = node_record_table_ptr[i];
 		if (node_ptr->power)
 			node_ptr->power->new_job_time = now;
 	}
