@@ -89,7 +89,7 @@ _slurm_cgroup_is_pid_a_slurm_task(uint64_t id, pid_t pid)
 	int fstatus = -1;
 	int fd;
 	pid_t ppid;
-	char file_path[PATH_MAX], buf[2048];
+	char file_path[PATH_MAX], buf[2048] = {0};
 
 	if (snprintf(file_path, PATH_MAX, "/proc/%ld/stat",
 		     (long)pid) >= PATH_MAX) {
@@ -133,7 +133,7 @@ _slurm_cgroup_is_pid_a_slurm_task(uint64_t id, pid_t pid)
 extern int init (void)
 {
 	/* initialize cpuinfo internal data */
-	if (xcpuinfo_init() != XCPUINFO_SUCCESS) {
+	if (xcpuinfo_init() != SLURM_SUCCESS) {
 		return SLURM_ERROR;
 	}
 
@@ -157,7 +157,15 @@ extern int fini (void)
  */
 extern int proctrack_p_create (stepd_step_rec_t *job)
 {
-	return cgroup_g_step_create(CG_TRACK, job);
+	int rc;
+
+	if ((rc = cgroup_g_step_create(CG_TRACK, job)) != SLURM_SUCCESS)
+		return rc;
+
+	/* Use slurmstepd pid as the id of the container. */
+	job->cont_id = (uint64_t)job->jmgr_pid;
+
+	return cgroup_g_step_addto(CG_TRACK, &job->jmgr_pid, 1);
 }
 
 extern int proctrack_p_add (stepd_step_rec_t *job, pid_t pid)
@@ -192,8 +200,11 @@ extern int proctrack_p_signal (uint64_t id, int signal)
 	}
 
 	for (i = 0 ; i<npids ; i++) {
-		/* do not kill slurmstepd (it should not be part
-		 * of the list, but just to not forget about that ;))
+		/*
+		 * Be on the safe side and do not kill slurmstepd (ourselves),
+		 * since a call to proctrack_g_get_pids can return all the pids
+		 * in the container which can include us, depending on the
+		 * plugin used (e.g. cgroup/v2).
 		 */
 		if (pids[i] == (pid_t)id)
 			continue;
@@ -235,36 +246,48 @@ extern bool proctrack_p_has_pid(uint64_t cont_id, pid_t pid)
 	return cgroup_g_has_pid(pid);
 }
 
-extern int proctrack_p_wait(uint64_t cont_id)
-{
-	int delay = 1;
-	time_t start = time(NULL);
-
-	if (cont_id == 0 || cont_id == 1) {
-		errno = EINVAL;
-		return SLURM_ERROR;
-	}
-
-	/* Spin until the container is successfully destroyed */
-	/* This indicates that all tasks have exited the container */
-	while (proctrack_p_destroy(cont_id) != SLURM_SUCCESS) {
-		time_t now = time(NULL);
-
-		if (now > (start + slurm_conf.unkillable_timeout)) {
-			error("Unable to destroy container %"PRIu64" in cgroup plugin, giving up after %lu sec",
-			      cont_id, (now - start));
-			break;
-		}
-		proctrack_p_signal(cont_id, SIGKILL);
-		sleep(delay);
-		if (delay < 32)
-			delay *= 2;
-	}
-
-	return SLURM_SUCCESS;
-}
 
 extern int proctrack_p_get_pids(uint64_t cont_id, pid_t **pids, int *npids)
 {
 	return cgroup_g_step_get_pids(pids, npids);
 }
+
+extern int proctrack_p_wait(uint64_t cont_id)
+{
+	int delay = 1;
+	time_t start = time(NULL), now;
+	pid_t *pids = NULL;
+	int npids = 0, rc;
+
+	if (cont_id == 0 || cont_id == 1)
+		return SLURM_ERROR;
+
+	/*
+	 * Spin until the container is empty. This indicates that all tasks have
+	 * exited the container.
+	 */
+	rc = proctrack_p_get_pids(cont_id, &pids, &npids);
+	while ((rc == SLURM_SUCCESS) && npids) {
+		if ((npids == 1) && (pids[0] == cont_id))
+			break;
+
+		now = time(NULL);
+		if (now > (start + slurm_conf.unkillable_timeout)) {
+			error("Container %"PRIu64" in cgroup plugin has %d processes, giving up after %lu sec",
+			      cont_id, npids, (now - start));
+			break;
+		}
+		/*
+		 * The call to proctrack_p_signal is safe since it takes care of
+		 * not killing slurmstepd processes (ourselves).
+		 */
+		proctrack_p_signal(cont_id, SIGKILL);
+		sleep(delay);
+		if (delay < 32)
+			delay *= 2;
+		xfree(pids);
+		rc = proctrack_p_get_pids(cont_id, &pids, &npids);
+	}
+	xfree(pids);
+	return SLURM_SUCCESS;
+}
