@@ -36,16 +36,19 @@
 
 #include "config.h"
 
+#include <yaml.h>
+
 #include "slurm/slurm.h"
 #include "src/common/slurm_xlator.h"
 
+#include "src/common/data.h"
 #include "src/common/log.h"
+#include "src/common/pack.h"
+#include "src/common/read_config.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
-#include <yaml.h>
-
 /*
  * These variables are required by the generic plugin interface.  If they
  * are not found in the plugin, the plugin loader will ignore it.
@@ -71,7 +74,7 @@
  * plugin_version - an unsigned 32-bit integer containing the Slurm version
  * (major.minor.micro combined into a single number).
  */
-const char *plugin_name = "Serializer YAML plugin";
+const char plugin_name[] = "Serializer YAML plugin";
 const char plugin_type[] = "serializer/yaml";
 const uint32_t plugin_version = SLURM_VERSION_NUMBER;
 /*
@@ -84,9 +87,6 @@ const char *mime_types[] = {
 	NULL
 };
 
-/* Default to about 1MB */
-static const size_t yaml_buffer_size = 4096 * 256;
-
 /* YAML parser doesn't give constants for the well defined scalars */
 #define YAML_NULL "null"
 #define YAML_TRUE "true"
@@ -824,10 +824,42 @@ yaml_fail:
 	return SLURM_ERROR;
 }
 
-static int _dump_yaml(const data_t *data, yaml_emitter_t *emitter,
-		      yaml_char_t *buffer, const size_t buffer_len)
+static int _yaml_write_handler(void *data, unsigned char *buffer, size_t size)
+{
+	buf_t *buf = data;
+	xassert(buf->magic == BUF_MAGIC);
+
+	/*
+	 * If the remaining buffer size equals the required argument size, we
+	 * still want to grow to allocate space for an extra '\0'. That's why in
+	 * this case we compare with '<=' instead of '<'.
+	 */
+	if (remaining_buf(buf) <= size) {
+		if ((size + size_buf(buf)) >= MAX_BUF_SIZE) {
+			error("%s: attempting to write too large of YAML output",
+			      __func__);
+			return 0;
+		}
+
+		grow_buf(buf, size);
+	}
+
+	memcpy(buf->head + buf->processed, buffer, size);
+
+	buf->processed += size;
+	/*
+	 * buf->processed points to one position after the memcpy'd payload,
+	 * so we can set the '\0' there. This position will be overridden by
+	 * the first character of the next chunk except for the last handler
+	 * call, effectively resulting in a NULL-terminated buffer.
+	 */
+	buf->head[buf->processed] = '\0';
+
+	return 1;
+}
+
+static int _dump_yaml(const data_t *data, yaml_emitter_t *emitter, buf_t *buf)
 {
-	size_t written = 0;
 	yaml_event_t event;
 
 	//TODO: only version 1.1 is currently supported by libyaml
@@ -839,7 +871,7 @@ static int _dump_yaml(const data_t *data, yaml_emitter_t *emitter,
 	if (!yaml_emitter_initialize(emitter))
 		_yaml_emitter_error;
 
-	yaml_emitter_set_output_string(emitter, buffer, buffer_len, &written);
+	yaml_emitter_set_output(emitter, _yaml_write_handler, buf);
 
 	//TODO defaulted to UTF8 but maybe this should be a flag?
 	if (!yaml_stream_start_event_initialize(&event, YAML_UTF8_ENCODING))
@@ -855,7 +887,7 @@ static int _dump_yaml(const data_t *data, yaml_emitter_t *emitter,
 		_yaml_emitter_error;
 
 	if (_data_to_yaml(data, emitter))
-		return SLURM_ERROR;
+		goto yaml_fail;
 
 	if (!yaml_document_end_event_initialize(&event, 0))
 		_yaml_emitter_error;
@@ -881,20 +913,24 @@ extern int serializer_p_serialize(char **dest, const data_t *data,
 				  data_serializer_flags_t flags)
 {
 	yaml_emitter_t emitter;
-	yaml_char_t *buffer = xmalloc(yaml_buffer_size);
+	buf_t *buf = init_buf(0);
 
-	if (_dump_yaml(data, &emitter, buffer, yaml_buffer_size)) {
+	if (_dump_yaml(data, &emitter, buf)) {
 		error("%s: dump yaml failed", __func__);
 
-		xfree(buffer);
+		FREE_NULL_BUFFER(buf);
 		return ESLURM_DATA_CONV_FAILED;
 	}
 
 	yaml_emitter_delete(&emitter);
 
-	/* recast as signed as that is the Slurm default */
-	*dest = (char *)buffer;
-	return SLURM_SUCCESS;
+	*dest = xfer_buf_data(buf);
+	buf = NULL;
+
+	if (*dest)
+		return SLURM_SUCCESS;
+	else
+		return SLURM_ERROR;
 }
 
 extern int serializer_p_deserialize(data_t **dest, const char *src,
