@@ -40,9 +40,13 @@
 #include <signal.h>
 #include <sys/types.h>
 
+#include "slurm/slurm.h"
 #include "slurm/slurm_errno.h"
 #include "src/common/slurm_xlator.h"
 
+#include "src/slurmctld/slurmctld.h"
+#include "src/slurmd/slurmstepd/slurmstepd_job.h"
+
 /*
  * These variables are required by the generic plugin interface.  If they
  * are not found in the plugin, the plugin loader will ignore it.
@@ -236,42 +240,6 @@ extern int switch_p_get_jobinfo(switch_jobinfo_t *switch_job,
 	return SLURM_ERROR;
 }
 
-/*
- * node switch state monitoring functions
- * required for IBM Federation switch
- */
-extern int switch_p_clear_node_state(void)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_alloc_node_info(switch_node_info_t **switch_node)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_build_node_info(switch_node_info_t *switch_node)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_pack_node_info(switch_node_info_t *switch_node,
-				   buf_t *buffer, uint16_t protocol_version)
-{
-	return 0;
-}
-
-extern int switch_p_unpack_node_info(switch_node_info_t **switch_node,
-				     buf_t *buffer, uint16_t protocol_version)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_free_node_info(switch_node_info_t **switch_node)
-{
-	return SLURM_SUCCESS;
-}
-
 extern int switch_p_job_step_complete(switch_jobinfo_t *jobinfo,
 	char *nodelist)
 {
@@ -303,3 +271,8 @@ extern int switch_p_job_step_post_resume( stepd_step_rec_t *job )
 {
 	return SLURM_SUCCESS;
 }
+
+extern void switch_p_job_complete(uint32_t job_id)
+{
+	return;
+}
