@@ -57,6 +57,9 @@
 
 #include "slurm/slurm_errno.h"
 #include "src/common/slurm_xlator.h"
+
+#include "src/common/timers.h"
+#include "src/common/xstring.h"
 #include "src/slurmd/slurmstepd/slurmstepd_job.h"
 
 #ifdef HAVE_NATIVE_CRAY
@@ -224,7 +227,7 @@ extern int task_p_slurmd_batch_request (batch_job_launch_msg_t *req)
  * task_p_slurmd_launch_request()
  */
 extern int task_p_slurmd_launch_request (launch_tasks_request_msg_t *req,
-					 uint32_t node_id)
+					 uint32_t node_id, char **err_msg)
 {
 	return SLURM_SUCCESS;
 }
@@ -500,7 +503,7 @@ extern int task_p_post_step (stepd_step_rec_t *job)
 	if (job->step_id.step_id == SLURM_BATCH_SCRIPT) {
 		// Batch Job Step
 		rc = snprintf(path, sizeof(path),
-			      "/dev/cpuset/slurm/uid_%d/job_%"
+			      "/dev/cpuset/slurm/uid_%u/job_%"
 			      PRIu32 "/step_batch", job->uid, jobid);
 		if (rc < 0) {
 			CRAY_ERR("snprintf failed. Return code: %d", rc);
@@ -509,12 +512,21 @@ extern int task_p_post_step (stepd_step_rec_t *job)
 	} else if (job->step_id.step_id == SLURM_EXTERN_CONT) {
 		// Container for PAM to use for externally launched processes
 		rc = snprintf(path, sizeof(path),
-			      "/dev/cpuset/slurm/uid_%d/job_%"
+			      "/dev/cpuset/slurm/uid_%u/job_%"
 			      PRIu32 "/step_extern", job->uid, jobid);
 		if (rc < 0) {
 			CRAY_ERR("snprintf failed. Return code: %d", rc);
 			return SLURM_ERROR;
 		}
+	} else if (job->step_id.step_id == SLURM_INTERACTIVE_STEP) {
+		/* Interactive Job Step */
+		rc = snprintf(path, sizeof(path),
+			      "/dev/cpuset/slurm/uid_%u/job_%u/step_interactive",
+			      job->uid, jobid);
+		if (rc < 0) {
+			CRAY_ERR("snprintf failed. Return code: %d", rc);
+			return SLURM_ERROR;
+		}
 	} else {
 		// Normal Job Step
 
@@ -522,7 +534,7 @@ extern int task_p_post_step (stepd_step_rec_t *job)
 		_step_epilogue();
 
 		rc = snprintf(path, sizeof(path),
-			      "/dev/cpuset/slurm/uid_%d/job_%"
+			      "/dev/cpuset/slurm/uid_%u/job_%"
 			      PRIu32 "/step_%" PRIu32,
 			      job->uid, jobid, job->step_id.step_id);
 		if (rc < 0) {
@@ -582,7 +594,7 @@ static void _alpsc_debug(const char *file, int line, const char *func,
 	} else if (err_msg) {
 		info("%s: %s", alpsc_func, err_msg);
 	} else
-		log_flag(TASK, "Called %s", alpsc_func);
+		debug2("Called %s", alpsc_func);
 
 	free(err_msg);
 }
@@ -811,7 +823,7 @@ again:
 		return -1;
 	}
 
-	log_flag(TASK, "Bitmask %#lx size: %lu sizeof(*(bm->maskp)): %zu weight: %u",
+	log_flag(CPU_BIND, "Bitmask %#lx size: %lu sizeof(*(bm->maskp)): %zu weight: %u",
 		 *(bm->maskp), bm->size, sizeof(*(bm->maskp)), *cnt);
 
 	*numa_array = xmalloc(*cnt * sizeof(int32_t));
@@ -819,7 +831,7 @@ again:
 	index = 0;
 	for (i = 0; i < bm->size; i++) {
 		if (*(bm->maskp) & ((long unsigned) 1 << i)) {
-			log_flag(TASK, "(%s: %d: %s) NUMA Node %d is present",
+			log_flag(CPU_BIND, "(%s: %d: %s) NUMA Node %d is present",
 				 THIS_FILE, __LINE__, __func__, i);
 			(*numa_array)[index++] = i;
 		}
@@ -923,7 +935,7 @@ static int _get_cpu_masks(int num_numa_nodes, int32_t *numa_array,
 		}
 	}
 
-	if (slurm_conf.debug_flags & DEBUG_FLAG_TASK) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_CPU_BIND) {
 		bitmask_str = NULL;
 		for (i = 0; i < num_numa_nodes; i++) {
 			for (j = 0; j < NUM_INTS_TO_HOLD_ALL_CPUS; j++) {
@@ -969,7 +981,7 @@ static int _get_cpu_masks(int num_numa_nodes, int32_t *numa_array,
 				CPU_SET(j, &cpusetptr[i]);
 			}
 		}
-		log_flag(TASK, "CPU_COUNT() of set: %d",
+		log_flag(CPU_BIND, "CPU_COUNT() of set: %d",
 			 CPU_COUNT(&cpusetptr[i]));
 	}
 
@@ -1058,7 +1070,7 @@ static int _update_num_steps(int val)
 		TEMP_FAILURE_RETRY(close(fd));
 		return -1;
 	}
-	log_flag(TASK, "Wrote %d steps to %s", num_steps, NUM_STEPS_FILE);
+	debug2("Wrote %d steps to %s", num_steps, NUM_STEPS_FILE);
 
 	TEMP_FAILURE_RETRY(close(fd));
 	return num_steps;
@@ -1110,8 +1122,7 @@ static int _step_epilogue(void)
 			return SLURM_ERROR;
 		}
 	} else
-		log_flag(TASK, "Skipping epilogue, %d other steps running",
-			 num_steps);
+		debug2("Skipping epilogue, %d other steps running", num_steps);
 
 	return SLURM_SUCCESS;
 }
