@@ -198,8 +198,9 @@ static int _set_rec(int *start, int argc, char **argv,
 					MAX(command_len, 2))) {
 			if (*(argv[i]+end) == '\0' &&
 			    (option == '+' || option == '-')) {
-				printf(" You didn't specify any features to %s\n",
-				       (option == '-') ? "remove" : "add");
+				fprintf(stderr,
+					" You didn't specify any features to %s\n",
+					(option == '-') ? "remove" : "add");
 				exit_code = 1;
 				break;
 			}
@@ -1122,7 +1123,7 @@ extern int sacctmgr_dump_cluster (int argc, char **argv)
 		return SLURM_ERROR;
 
 	} else {
-		if (my_uid != slurm_get_slurm_user_id() && my_uid != 0
+		if ((my_uid != slurm_conf.slurm_user_id) && (my_uid != 0)
 		    && user->admin_level < SLURMDB_ADMIN_SUPER_USER) {
 			exit_code = 1;
 			fprintf(stderr, " Your user does not have sufficient "
