@@ -59,6 +59,8 @@
 #define OPT_LONG_LOCAL     0x103
 #define OPT_LONG_NOCONVERT 0x104
 #define OPT_LONG_FEDR      0x105
+#define OPT_LONG_JSON      0x106
+#define OPT_LONG_YAML      0x107
 
 /* FUNCTIONS */
 static List  _build_state_list( char* str );
@@ -118,13 +120,14 @@ extern void parse_command_line(int argc, char **argv)
 		{"usage",     no_argument,       0, OPT_LONG_USAGE},
 		{"verbose",   no_argument,       0, 'v'},
 		{"version",   no_argument,       0, 'V'},
+                {"json", no_argument, 0, OPT_LONG_JSON},
+                {"yaml", no_argument, 0, OPT_LONG_YAML},
 		{NULL,        0,                 0, 0}
 	};
 
 	params.convert_flags = CONVERT_NUM_UNIT_EXACT;
 
-	if (slurmctld_conf.fed_params &&
-	    strstr(slurmctld_conf.fed_params, "fed_display"))
+	if (xstrstr(slurm_conf.fed_params, "fed_display"))
 		params.federation_flag = true;
 
 	if (getenv("SINFO_ALL")) {
@@ -291,6 +294,14 @@ extern void parse_command_line(int argc, char **argv)
 		case OPT_LONG_LOCAL:
 			params.local = true;
 			break;
+		case OPT_LONG_JSON:
+			params.mimetype = MIME_TYPE_JSON;
+			data_init(MIME_TYPE_JSON_PLUGIN, NULL);
+			break;
+		case OPT_LONG_YAML:
+			params.mimetype = MIME_TYPE_YAML;
+			data_init(MIME_TYPE_YAML_PLUGIN, NULL);
+			break;
 		}
 	}
 
@@ -307,16 +318,14 @@ extern void parse_command_line(int argc, char **argv)
 
 	if (params.federation_flag && !params.clusters && !params.local) {
 		void *ptr = NULL;
-		char *cluster_name = slurm_get_cluster_name();
 		if (slurm_load_federation(&ptr) ||
-		    !cluster_in_federation(ptr, cluster_name)) {
+		    !cluster_in_federation(ptr, slurm_conf.cluster_name)) {
 			/* Not in federation */
 			params.local = true;
 			slurm_destroy_federation_rec(ptr);
 		} else {
 			params.fed = (slurmdb_federation_rec_t *) ptr;
 		}
-		xfree(cluster_name);
 	}
 
 	if ( params.format == NULL ) {
@@ -324,20 +333,20 @@ extern void parse_command_line(int argc, char **argv)
 		if ( params.summarize ) {
 			long_form = true;
 			params.part_field_flag = true;	/* compute size later */
-			params.format = "partition:9 ,available:.5 ,time:.10 ,nodeaiot:.16 ,nodelist:0";
+			params.format = xstrdup("partition:9 ,available:.5 ,time:.10 ,nodeaiot:.16 ,nodelist:0");
 		} else if ( params.node_flag ) {
 			long_form = true;
 			params.node_field_flag = true;	/* compute size later */
 			params.part_field_flag = true;	/* compute size later */
 			params.format = params.long_output ?
-			  "nodelist:0 ,nodes:.6 ,partition:.9 ,statelong:.11 ,cpus:4 ,socketcorethread:.8 ,memory:.6 ,disk:.8 ,weight:.6 ,features:.8 ,reason:20" :
-			  "nodelist:0 ,nodes:.6 ,partition:.9 ,statecompact:6";
+				xstrdup("nodelist:0 ,nodes:.6 ,partition:.9 ,statelong:.11 ,cpus:4 ,socketcorethread:.8 ,memory:.6 ,disk:.8 ,weight:.6 ,features:.8 ,reason:20") :
+				xstrdup("nodelist:0 ,nodes:.6 ,partition:.9 ,statecompact:6");
 
 		} else if (params.list_reasons) {
 			long_form = true;
 			params.format = params.long_output ?
-			  "reason:20 ,userlong:12 ,timestamp:19 ,statecompact:6 ,nodelist:0" :
-			  "reason:20 ,user:9 ,timestamp:19 ,nodelist:0";
+				xstrdup("reason:20 ,userlong:12 ,timestamp:19 ,statecompact:6 ,nodelist:0") :
+				xstrdup("reason:20 ,user:9 ,timestamp:19 ,nodelist:0");
 
 		} else if ((env_val = getenv ("SINFO_FORMAT"))) {
 			params.format = xstrdup(env_val);
@@ -346,14 +355,14 @@ extern void parse_command_line(int argc, char **argv)
 			long_form = true;
 			params.part_field_flag = true;	/* compute size later */
 			params.format = params.long_output ?
-			  "partition:9 ,cluster:8 ,available:.5 ,time:.10 ,size:.10 ,root:.4 ,oversubscribe:.8 ,groups:.10 ,nodes:.6 ,statelong:.11 ,nodelist:0" :
-			  "partition:9 ,cluster:8 ,available:.5 ,time:.10 ,nodes:.6 ,statecompact:.6 ,nodelist:0";
+				xstrdup("partition:9 ,cluster:8 ,available:.5 ,time:.10 ,size:.10 ,root:.4 ,oversubscribe:.8 ,groups:.10 ,nodes:.6 ,statelong:.11 ,nodelist:0") :
+				xstrdup("partition:9 ,cluster:8 ,available:.5 ,time:.10 ,nodes:.6 ,statecompact:.6 ,nodelist:0");
 		} else {
 			long_form = true;
 			params.part_field_flag = true;	/* compute size later */
 			params.format = params.long_output ?
-			  "partition:9 ,available:.5 ,time:.10 ,size:.10 ,root:.4 ,oversubscribe:.8 ,groups:.10 ,nodes:.6 ,statelong:.11 ,nodelist:0" :
-			  "partition:9 ,available:.5 ,time:.10 ,nodes:.6 ,statecompact:.6 ,nodelist:0";
+				xstrdup("partition:9 ,available:.5 ,time:.10 ,size:.10 ,root:.4 ,oversubscribe:.8 ,groups:.10 ,nodes:.6 ,statelong:.11 ,nodelist:0") :
+				xstrdup("partition:9 ,available:.5 ,time:.10 ,nodes:.6 ,statecompact:.6 ,nodelist:0");
 		}
 	}
 
@@ -421,13 +430,18 @@ _build_state_list (char *state_str)
 	orig = str = xstrdup (state_str);
 	state_ids = list_create (NULL);
 
-	while ((state = _next_tok (",", &str))) {
+	if (xstrstr(state_str, "&"))
+	    params.state_list_and = true;
+
+	state = strtok_r(state_str, ",&", &str);
+	while (state) {
 		int *id = xmalloc (sizeof (*id));
 		if ((*id = _node_state_id (state)) < 0) {
 			error ("Bad state string: \"%s\"", state);
 			return (NULL);
 		}
 		list_append (state_ids, id);
+		state = strtok_r(NULL, ",&", &str);
 	}
 
 	xfree (orig);
@@ -504,19 +518,27 @@ _node_state_list (void)
 	xstrcat(all_states,
 		",DRAIN,DRAINED,DRAINING,NO_RESPOND,RESERVED,PERFCTRS");
 	xstrcat(all_states, ",");
+	xstrcat(all_states, node_state_string(NODE_STATE_CLOUD));
+	xstrcat(all_states, ",");
 	xstrcat(all_states, node_state_string(NODE_STATE_COMPLETING));
 	xstrcat(all_states, ",");
 	xstrcat(all_states, node_state_string(NODE_STATE_POWERING_DOWN));
 	xstrcat(all_states, ",");
-	xstrcat(all_states, node_state_string(NODE_STATE_POWER_SAVE));
+	xstrcat(all_states, node_state_string(NODE_STATE_POWERED_DOWN));
 	xstrcat(all_states, ",");
-	xstrcat(all_states, node_state_string(NODE_STATE_POWER_UP));
+	xstrcat(all_states, node_state_string(NODE_STATE_POWER_DOWN));
+	xstrcat(all_states, ",");
+	xstrcat(all_states, node_state_string(NODE_STATE_POWERING_UP));
 	xstrcat(all_states, ",");
 	xstrcat(all_states, node_state_string(NODE_STATE_FAIL));
 	xstrcat(all_states, ",");
 	xstrcat(all_states, node_state_string(NODE_STATE_MAINT));
 	xstrcat(all_states, ",");
-	xstrcat(all_states, node_state_string(NODE_STATE_REBOOT));
+	xstrcat(all_states, node_state_string(NODE_STATE_REBOOT_REQUESTED));
+	xstrcat(all_states, ",");
+	xstrcat(all_states, node_state_string(NODE_STATE_REBOOT_ISSUED));
+	xstrcat(all_states, ",");
+	xstrcat(all_states, node_state_string(NODE_STATE_PLANNED));
 
 	for (i = 0; i < strlen (all_states); i++)
 		all_states[i] = tolower (all_states[i]);
@@ -553,6 +575,9 @@ _node_state_id (char *str)
 			return (i);
 	}
 
+	if ((xstrncasecmp("PLANNED", str, len) == 0) ||
+	    (xstrncasecmp("PLND", str, len) == 0))
+		return NODE_STATE_PLANNED;
 	if (xstrncasecmp("DRAIN", str, len) == 0)
 		return NODE_STATE_DRAIN;
 	if (xstrncasecmp("DRAINED", str, len) == 0)
@@ -572,16 +597,22 @@ _node_state_id (char *str)
 		return NODE_STATE_NO_RESPOND;
 	if (_node_state_equal (NODE_STATE_POWERING_DOWN, str))
 		return NODE_STATE_POWERING_DOWN;
-	if (_node_state_equal (NODE_STATE_POWER_SAVE, str))
-		return NODE_STATE_POWER_SAVE;
-	if (_node_state_equal (NODE_STATE_POWER_UP, str))
-		return NODE_STATE_POWER_UP;
+	if (_node_state_equal (NODE_STATE_POWERED_DOWN, str))
+		return NODE_STATE_POWERED_DOWN;
+	if (_node_state_equal (NODE_STATE_POWER_DOWN, str))
+		return NODE_STATE_POWER_DOWN;
+	if (_node_state_equal (NODE_STATE_POWERING_UP, str))
+		return NODE_STATE_POWERING_UP;
 	if (_node_state_equal (NODE_STATE_FAIL, str))
 		return NODE_STATE_FAIL;
 	if (_node_state_equal (NODE_STATE_MAINT, str))
 		return NODE_STATE_MAINT;
-	if (_node_state_equal (NODE_STATE_REBOOT, str))
-		return NODE_STATE_REBOOT;
+	if (_node_state_equal (NODE_STATE_REBOOT_REQUESTED, str))
+		return NODE_STATE_REBOOT_REQUESTED;
+	if (_node_state_equal (NODE_STATE_REBOOT_ISSUED, str))
+		return NODE_STATE_REBOOT_ISSUED;
+	if (_node_state_equal(NODE_STATE_CLOUD, str))
+		return NODE_STATE_CLOUD;
 
 	return (-1);
 }
@@ -604,7 +635,6 @@ _parse_format( char* format )
 		exit( 1 );
 	}
 
-	params.format_list = list_create( NULL );
 	if ((prefix = _get_prefix(format)))
 		format_add_prefix( params.format_list, 0, 0, prefix);
 
@@ -904,16 +934,15 @@ static int _parse_long_format (char* format_long)
 		exit( 1 );
 	}
 
-	params.format_list = list_create(NULL);
 	tmp_format = xstrdup(format_long);
 	token = strtok_r(tmp_format, ",",&str_tmp);
 
 	while (token) {
 		_parse_long_token( token, sep, &field_size, &right_justify,
 				   &suffix);
-
 		if (!xstrcasecmp(token, "all")) {
 			_parse_format ("%all");
+			xfree(suffix);
 		} else if (!xstrcasecmp(token, "allocmem")) {
 			params.match_flags.alloc_mem_flag = true;
 			format_add_alloc_mem( params.format_list,
@@ -936,6 +965,12 @@ static int _parse_long_format (char* format_long)
 						field_size,
 						right_justify,
 						suffix);
+		} else if (!xstrcasecmp(token, "comment")) {
+			params.match_flags.comment_flag = true;
+			format_add_comment(params.format_list,
+					   field_size,
+					   right_justify,
+					   suffix);
 		} else if (!xstrcasecmp(token, "cpus")) {
 			params.match_flags.cpus_flag = true;
 			format_add_cpus( params.format_list,
@@ -978,6 +1013,10 @@ static int _parse_long_format (char* format_long)
 					 field_size,
 					 right_justify,
 					 suffix );
+		} else if (!xstrcasecmp(token, "extra")) {
+			params.match_flags.extra_flag = true;
+			format_add_extra(params.format_list, field_size,
+					 right_justify, suffix);
 		} else if (!xstrcasecmp(token, "features")) {
 			params.match_flags.features_flag = true;
 			format_add_features( params.format_list,
@@ -1119,6 +1158,11 @@ static int _parse_long_format (char* format_long)
 						  field_size,
 						  right_justify,
 						  suffix );
+		} else if (!xstrcasecmp(token, "statecomplete")) {
+			params.match_flags.statecomplete_flag = true;
+			format_add_state_complete(params.format_list,
+						  field_size, right_justify,
+						  suffix );
 		} else if (!xstrcasecmp(token, "statelong")) {
 			params.match_flags.state_flag = true;
 			format_add_state_long( params.format_list,
@@ -1181,6 +1225,7 @@ static int _parse_long_format (char* format_long)
 					   suffix );
 		} else if (format_all) {
 			/* ignore */
+			xfree(suffix);
 		} else {
 			format_add_invalid( params.format_list,
 					    field_size,
@@ -1233,7 +1278,7 @@ _parse_token( char *token, char *field, int *field_size, bool *right_justify,
 {
 	int i = 0;
 
-	assert (token != NULL);
+	xassert(token);
 
 	if (token[i] == '.') {
 		*right_justify = true;
@@ -1256,6 +1301,7 @@ _parse_long_token( char *token, char *sep, int *field_size, bool *right_justify,
 {
 	char *end_ptr = NULL, *ptr;
 
+	*suffix = NULL;
 	xassert(token);
 	ptr = strchr(token, ':');
 	if (ptr) {
@@ -1306,12 +1352,16 @@ void _print_options( void )
 			"true" : "false");
 	printf("avail_flag      = %s\n", params.match_flags.avail_flag ?
 			"true" : "false");
+	printf("comment_flag    = %s\n", params.match_flags.comment_flag ?
+			"true" : "false");
 	printf("cpus_flag       = %s\n", params.match_flags.cpus_flag ?
 			"true" : "false");
 	printf("default_time_flag =%s\n", params.match_flags.default_time_flag ?
 					"true" : "false");
 	printf("disk_flag       = %s\n", params.match_flags.disk_flag ?
 			"true" : "false");
+	printf("extra_flag      = %s\n",
+	       params.match_flags.extra_flag ? "true" : "false");
 	printf("features_flag   = %s\n", params.match_flags.features_flag ?
 			"true" : "false");
 	printf("features_flag_act = %s\n", params.match_flags.features_act_flag?
@@ -1354,6 +1404,8 @@ void _print_options( void )
 			"true" : "false");
 	printf("state_flag      = %s\n", params.match_flags.state_flag ?
 			"true" : "false");
+	printf("statecomplete_flag = %s\n",
+	       params.match_flags.statecomplete_flag ? "true" : "false");
 	printf("weight_flag     = %s\n", params.match_flags.weight_flag ?
 			"true" : "false");
 	printf("-----------------------------\n\n");
@@ -1378,6 +1430,7 @@ Usage: sinfo [OPTIONS]\n\
   -h, --noheader             no headers on output\n\
   --hide                     do not show hidden or non-accessible partitions\n\
   -i, --iterate=seconds      specify an iteration period\n\
+      --json                 Produce JSON output\n\
       --local                show only local cluster in a federation.\n\
                              Overrides --federation.\n\
   -l, --long                 long output - displays more information\n\
@@ -1398,6 +1451,7 @@ Usage: sinfo [OPTIONS]\n\
   -T, --reservation          show only reservation information\n\
   -v, --verbose              verbosity level\n\
   -V, --version              output version information and exit\n\
+      --yaml                 Produce YAML output\n\
 \nHelp options:\n\
   --help                     show this help message\n\
   --usage                    display brief usage message\n");
