@@ -53,7 +53,7 @@
 #include "src/common/daemonize.h"
 #include "src/common/log.h"
 #include "src/common/macros.h"
-#include "src/common/node_select.h"
+#include "src/common/select.h"
 #include "src/common/slurm_auth.h"
 #include "src/common/slurm_accounting_storage.h"
 #include "src/common/switch.h"
@@ -392,8 +392,8 @@ static void *_background_rpc_mgr(void *no_data)
 		slurm_msg_t_init(&msg);
 		if (slurm_receive_msg(newsockfd, &msg, 0) != 0)
 			error("slurm_receive_msg: %m");
-
-		_background_process_msg(&msg);
+		else
+			_background_process_msg(&msg);
 
 		slurm_free_msg_members(&msg);
 
@@ -414,6 +414,10 @@ static int _background_process_msg(slurm_msg_t *msg)
 	int error_code = SLURM_SUCCESS;
 	bool send_rc = true;
 
+	if (!msg->auth_uid_set)
+		fatal("%s: received message without previously validated auth",
+		      __func__);
+
 	if (msg->msg_type != REQUEST_PING) {
 		bool super_user = false;
 		uid_t uid = auth_g_get_uid(msg->auth_cred);
@@ -445,7 +449,7 @@ static int _background_process_msg(slurm_msg_t *msg)
 			 * to the correct controller.  This usually indicates a
 			 * configuration issue.
 			 */
-			error("REQUEST_CONFIG recieved while in standby.");
+			error("REQUEST_CONFIG received while in standby.");
 			error_code = ESLURM_IN_STANDBY_USE_BACKUP;
 		} else {
 			error("Invalid RPC received %s while in standby mode",
@@ -469,6 +473,7 @@ static void *_ping_ctld_thread(void *arg)
 	slurm_msg_t_init(&req);
 	slurm_set_addr(&req.address, ping->slurmctld_port, ping->control_addr);
 	req.msg_type = REQUEST_CONTROL_STATUS;
+	slurm_msg_set_r_uid(&req, SLURM_AUTH_UID_ANY);
 	if (slurm_send_recv_node_msg(&req, &resp, 0) == SLURM_SUCCESS) {
 		switch (resp.msg_type) {
 		case RESPONSE_CONTROL_STATUS:
@@ -610,6 +615,7 @@ static void *_shutdown_controller(void *arg)
 	xfree(arg);
 
 	slurm_msg_t_init(&req);
+	slurm_msg_set_r_uid(&req, slurm_conf.slurm_user_id);
 	slurm_set_addr(&req.address, slurm_conf.slurmctld_port,
 	               slurm_conf.control_addr[shutdown_inx]);
 	if (do_shutdown) {
