@@ -44,8 +44,8 @@
 #include <time.h>
 
 #include "src/common/hostlist.h"
-#include "src/common/node_select.h"
 #include "src/common/read_config.h"
+#include "src/common/select.h"
 #include "src/slurmctld/agent.h"
 #include "src/slurmctld/front_end.h"
 #include "src/slurmctld/ping_nodes.h"
@@ -260,8 +260,7 @@ void ping_nodes (void)
 		ping_agent_args->node_count++;
 	}
 #else
-	for (i = 0, node_ptr = node_record_table_ptr;
-	     i < node_record_count; i++, node_ptr++) {
+	for (i = 0; (node_ptr = next_node(&i)); i++) {
 		if (IS_NODE_FUTURE(node_ptr) ||
 		    IS_NODE_POWERED_DOWN(node_ptr) ||
 		    IS_NODE_POWERING_DOWN(node_ptr) ||
@@ -300,7 +299,8 @@ void ping_nodes (void)
 		 * once in a while). We limit these requests since they
 		 * can generate a flood of incoming RPCs. */
 		if (IS_NODE_UNKNOWN(node_ptr) || (node_ptr->boot_time == 0) ||
-		    ((i >= offset) && (i < (offset + max_reg_threads)))) {
+		    ((node_ptr->index >= offset) &&
+		     (node_ptr->index < (offset + max_reg_threads)))) {
 			if (reg_agent_args->protocol_version >
 			    node_ptr->protocol_version)
 				reg_agent_args->protocol_version =
@@ -342,6 +342,7 @@ void ping_nodes (void)
 		debug("Spawning ping agent for %s", host_str);
 		xfree(host_str);
 		ping_begin();
+		set_agent_arg_r_uid(ping_agent_args, SLURM_AUTH_UID_ANY);
 		agent_queue_request(ping_agent_args);
 	}
 
@@ -356,6 +357,7 @@ void ping_nodes (void)
 		      host_str, reg_agent_args->node_count);
 		xfree(host_str);
 		ping_begin();
+		set_agent_arg_r_uid(reg_agent_args, SLURM_AUTH_UID_ANY);
 		agent_queue_request(reg_agent_args);
 	}
 
@@ -373,13 +375,13 @@ extern void run_health_check(void)
 {
 #ifdef HAVE_FRONT_END
 	front_end_record_t *front_end_ptr;
+	int i;
 #else
 	node_record_t *node_ptr;
 	int node_test_cnt = 0, node_limit, node_states, run_cyclic;
-	static int base_node_loc = -1;
+	static int base_node_loc = 0;
 	static time_t cycle_start_time = (time_t) 0;
 #endif
-	int i;
 	char *host_str = NULL;
 	agent_arg_t *check_agent_args = NULL;
 
@@ -419,7 +421,7 @@ extern void run_health_check(void)
 		time_t now = time(NULL);
 		if (cycle_start_time == (time_t) 0)
 			cycle_start_time = now;
-		else if (base_node_loc >= 0)
+		else if (base_node_loc > 0)
 			;	/* mid-cycle */
 		else if (difftime(now, cycle_start_time) <
 		         slurm_conf.health_check_interval) {
@@ -443,19 +445,12 @@ extern void run_health_check(void)
 	check_agent_args->retry = 0;
 	check_agent_args->protocol_version = SLURM_PROTOCOL_VERSION;
 	check_agent_args->hostlist = hostlist_create(NULL);
-	for (i = 0; i < node_record_count; i++) {
-		if (run_cyclic) {
-			if (node_test_cnt++ >= node_limit)
+	for (; base_node_loc < node_record_count; base_node_loc++) {
+		if (!(node_ptr = node_record_table_ptr[base_node_loc]))
+			continue;
+		if (run_cyclic &&
+		    (node_test_cnt++ >= node_limit))
 				break;
-			base_node_loc++;
-			if (base_node_loc >= node_record_count) {
-				base_node_loc = -1;
-				break;
-			}
-			node_ptr = node_record_table_ptr + base_node_loc;
-		} else {
-			node_ptr = node_record_table_ptr + i;
-		}
 		if (IS_NODE_NO_RESPOND(node_ptr) ||
 		    IS_NODE_FUTURE(node_ptr) ||
 		    IS_NODE_POWERING_DOWN(node_ptr) ||
@@ -500,8 +495,8 @@ extern void run_health_check(void)
 		hostlist_push_host(check_agent_args->hostlist, node_ptr->name);
 		check_agent_args->node_count++;
 	}
-	if (run_cyclic && (i >= node_record_count))
-		base_node_loc = -1;
+	if (base_node_loc >= node_record_count)
+		base_node_loc = 0;
 #endif
 
 	if (check_agent_args->node_count == 0) {
@@ -514,6 +509,7 @@ extern void run_health_check(void)
 		debug("Spawning health check agent for %s", host_str);
 		xfree(host_str);
 		ping_begin();
+		set_agent_arg_r_uid(check_agent_args, SLURM_AUTH_UID_ANY);
 		agent_queue_request(check_agent_args);
 	}
 }
@@ -550,8 +546,7 @@ extern void update_nodes_acct_gather_data(void)
 		agent_args->node_count++;
 	}
 #else
-	for (i = 0, node_ptr = node_record_table_ptr;
-	     i < node_record_count; i++, node_ptr++) {
+	for (i = 0; (node_ptr = next_node(&i)); i++) {
 		if (IS_NODE_NO_RESPOND(node_ptr) ||
 		    IS_NODE_FUTURE(node_ptr) ||
 		    IS_NODE_POWERING_DOWN(node_ptr) ||
@@ -574,6 +569,7 @@ extern void update_nodes_acct_gather_data(void)
 		log_flag(ENERGY, "Updating acct_gather data for %s", host_str);
 		xfree(host_str);
 		ping_begin();
+		set_agent_arg_r_uid(agent_args, SLURM_AUTH_UID_ANY);
 		agent_queue_request(agent_args);
 	}
 }
