@@ -71,6 +71,7 @@
 #include "src/common/fd.h"
 #include "src/common/log.h"
 #include "src/common/plugstack.h"
+#include "src/common/run_command.h"
 #include "src/common/slurm_mpi.h"
 #include "src/common/strlcpy.h"
 #include "src/common/switch.h"
@@ -185,84 +186,50 @@ rwfail:		 /* process rest of script output */
  * path IN: pathname of program to run
  * job IN/OUT: pointer to associated job, can update job->env
  *	if prolog
- * RET 0 on success, -1 on failure.
+ * RET the exit status of the script or 1 on generic error and 0 on success
  */
 static int
 _run_script_and_set_env(const char *name, const char *path,
 			stepd_step_rec_t *job)
 {
-	int status, rc;
-	pid_t cpid;
-	int pfd[2];
-	char buf[4096];
-	FILE *f;
+	int status = 0, rc = 0;
+	char *argv[2];
+	char *buf = NULL;
+	run_command_args_t args = {
+		.job_id = job->step_id.job_id,
+		.max_wait = -1,
+		.script_path = path,
+		.script_type = name,
+		.status = &status
+	};
+
+	if (path == NULL || path[0] == '\0')
+		return rc;
 
 	xassert(job->env);
-	if (path == NULL || path[0] == '\0')
-		return 0;
+	setenvf(&job->env, "SLURM_SCRIPT_CONTEXT", "prolog_task");
+	args.env = job->env;
+
+	argv[0] = xstrdup(path);
+	argv[1] = NULL;
+	args.script_argv = argv;
 
 	debug("[job %u] attempting to run %s [%s]",
 	      job->step_id.job_id, name, path);
+	buf = run_command(&args);
 
-	if (access(path, R_OK | X_OK) < 0) {
-		error("Could not run %s [%s]: %m", name, path);
-		return -1;
-	}
-	if (pipe(pfd) < 0) {
-		error("executing %s: pipe: %m", name);
-		return -1;
-	}
-	if ((cpid = fork()) < 0) {
-		error("executing %s: fork: %m", name);
-		return -1;
-	}
-	if (cpid == 0) {
-		char *argv[2];
-
-		setenvf(&job->env, "SLURM_SCRIPT_CONTEXT", "prolog_task");
-
-		argv[0] = xstrdup(path);
-		argv[1] = NULL;
-		if (dup2(pfd[1], STDOUT_FILENO) == -1)
-			error("couldn't do the dup: %m");
-		close(STDERR_FILENO);
-		close(STDIN_FILENO);
-		close(pfd[0]);
-		close(pfd[1]);
-		setpgid(0, 0);
-		execve(path, argv, job->env);
-		error("execve(%s): %m", path);
-		_exit(127);
-	}
-
-	close(pfd[1]);
-	f = fdopen(pfd[0], "r");
-	if (f == NULL) {
-		error("Cannot open pipe device: %m");
-		log_fini();
-		exit(1);
-	}
-	while (feof(f) == 0) {
-		if (fgets(buf, sizeof(buf) - 1, f) != NULL) {
+	if (WIFEXITED(status)) {
+		if (buf)
 			_proc_stdout(buf, job);
-		}
-	}
-	fclose(f);
-
-	while (1) {
-		rc = waitpid(cpid, &status, 0);
-		if (rc < 0) {
-			if (errno == EINTR)
-				continue;
-			error("waidpid: %m");
-			return 0;
-		} else  {
-			killpg(cpid, SIGKILL);  /* kill children too */
-			return status;
-		}
+		rc = WEXITSTATUS(status);
+	} else {
+		error("%s did not exit normally. reason: %s", name, buf);
+		rc = 1;
 	}
 
-	/* NOTREACHED */
+	xfree(argv[0]);
+	xfree(buf);
+	return rc;
 }
 
 /* Given a program name, translate it to a fully qualified pathname as needed
@@ -349,7 +316,7 @@ _setup_mpi(stepd_step_rec_t *job, int ltaskid)
 		info->client  = job->envtp->cli;
 	}
 
-	return mpi_hook_slurmstepd_task(info, &job->env);
+	return mpi_g_slurmstepd_task(info, &job->env);
 }
 
 /*
@@ -360,7 +327,7 @@ extern void exec_task(stepd_step_rec_t *job, int local_proc_id)
 	int fd, j;
 	stepd_step_task_info_t *task = job->task[local_proc_id];
 	char **tmp_env;
-	int saved_errno;
+	int saved_errno, status;
 	uint32_t node_offset = 0, task_offset = 0;
 
 	if (job->het_job_node_offset != NO_VAL)
@@ -402,7 +369,7 @@ extern void exec_task(stepd_step_rec_t *job, int local_proc_id)
 	 */
 	job->envtp->env = env_array_copy((const char **) job->env);
 	setup_env(job->envtp, false);
-	setenvf(&job->envtp->env, "SLURM_JOB_GID", "%d", job->gid);
+	setenvf(&job->envtp->env, "SLURM_JOB_GID", "%u", job->gid);
 	setenvf(&job->envtp->env, "SLURMD_NODENAME", "%s", conf->node_name);
 	if (job->tres_bind) {
 		setenvf(&job->envtp->env, "SLURMD_TRES_BIND", "%s",
@@ -444,7 +411,8 @@ extern void exec_task(stepd_step_rec_t *job, int local_proc_id)
 		error("Failed to invoke task plugins: task_p_pre_launch error");
 		_exit(1);
 	}
-	if (!job->batch && (job->accel_bind_type || job->tres_bind)) {
+	if (!job->batch && (job->step_id.step_id != SLURM_INTERACTIVE_STEP) &&
+	    (job->accel_bind_type || job->tres_bind)) {
 		/*
 		 * Modify copy of job's environment as needed for GRES. Do not
 		 * alter in place or concurrent searches of the environment can
@@ -477,12 +445,21 @@ extern void exec_task(stepd_step_rec_t *job, int local_proc_id)
 	}
 #endif
 
-	if (slurm_conf.task_prolog)
-		_run_script_and_set_env("slurm task_prolog",
-					slurm_conf.task_prolog, job);
+	if (slurm_conf.task_prolog) {
+		status = _run_script_and_set_env("slurm task_prolog",
+						 slurm_conf.task_prolog, job);
+		if (status) {
+			error("TaskProlog failed status=%d", status);
+			_exit(status);
+		}
+	}
 	if (job->task_prolog) {
-		_run_script_and_set_env("user task_prolog",
-					job->task_prolog, job);
+		status = _run_script_and_set_env("user task_prolog",
+						 job->task_prolog, job);
+		if (status) {
+			error("--task-prolog failed status=%d", status);
+			_exit(status);
+		}
 	}
 
 	/*
